/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    BaseImgLoadTime.c
 *
 *   Path    $(TESTSUITEROOT)/Analysis/DynLink/
 * 
 *   Desc:
 *           Measures the Base Image Load time for images 
 *			 with different number of symbols.
 *
 *  Usage:
 *        IMGLOADTIME [-v] [-l "log path/filename"] 
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *        -n: Number of Transfers for each Iteration
 *        -s: Buffer Size
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUN 23, 2003    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <semaphore.h>
#include <DSPProcessor_OEM.h>
#include "dynLink.h"

#define DEFNAMESIZE      100
#define DEFBUFSIZE        64
#define DEFNUMBUFS         1
#define DEFNUMTXFRS    10000
#define NUMOFCOMPARGS      2	/* Compulsory args: n,s */
#define PASS               0
#define FAIL               1

#define SHORT short

#define DSPEXEC         "dataxfr_nortdx_int_1510.dof55L"
#define DLLNAME         "inoutlgmem.dll55L"

/* Function to exit gracefully */
static INT graceful_exit()
{
	/*printf("FATAL ERROR: Exiting the Test Case !\n");
	exit(0);*/
}



/*
 *  ======== main ========
 *
 */
INT baseImgLoadTime_ANALYSIS(INT argc, CHAR * argv[])
{
    BOOL           			bUseLog=FALSE;
    
    FILE           			*pHandle = NULL; 

    INT            			iNumTransfers = DEFNUMTXFRS;
    INT            			iBuffSize = DEFBUFSIZE;
    
	CHAR           			szLogName[DEFNAMESIZE];
	INT            			iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS     			status;
    struct DSP_PROCESSORATTRIN		*pAttrIn;
	DSP_HPROCESSOR			hProcessor;
    CHAR					*aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
	INT						iArgc = 1;
	INT						opt;
    INT                 	retVal = PASS;
	CHAR        			*aDllName[DEFNAMESIZE] = { DLLNAME, "arg3" };
	
	DWORD			dwStartTime;  
	DWORD			dwAvgTime = 0;
	DWORD			dwEndTime;
	DWORD         	dwTotalElapsedTime;
	INT           	i = 0;
	struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId;





	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* For each Image to be loaded */
	/* start the timer and load the image */
	/* log the time taken to load the image */
	/* End !! */


	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt_test(argc, argv, "+vl:x:n:s:d:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DynamicLink_BEHAVIOUR: Verbose Mode On\n");
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "dynLink test:");
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            break;
		case 'n':
			/* Number of Transfers */
			iNumTransfers = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of transfers set to %d\n",iNumTransfers);
            TSTLOG_appendLog(pHandle, "Number of transfer set to %d\n",iNumTransfers);
			break;
		case 's':
			/* Buffer Size */
			iBuffSize = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Buffer Size set to %d\n",iBuffSize);
            TSTLOG_appendLog(pHandle, "Buffer Size set to %d\n",iBuffSize);
			break;
		case 'd':
        	/* use given dll */
        	aDllName[0] = (char *) optarg;
            break;	
        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt_test(argc, argv, "+vl:x:n:s:d:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
	}

    TST_verbose("DSP Executable: %s\n", aArgv[0]);


    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		// Error
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		graceful_exit();
	}
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Attach to Processor */
	 while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT)sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == DSPTYPE_55)
                {
                        TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
        }
    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, NULL,&hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",
                status);
        }
    }
        else
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");


    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Attach()\n");
        }
        graceful_exit();
    }

    /* Load default program */
    OsSpecificTimerStart(&dwStartTime);
    
    for (i=0; i < 100; i++)
    {
    	status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);
    }
    
    OsSpecificTimerStop(&dwEndTime);
    OsSpecificTimerDifference(&dwEndTime, &dwStartTime, &dwTotalElapsedTime);
    dwAvgTime = dwTotalElapsedTime / 100;
    
    /*printf("SSSSSSSSSSSSSSSS StartTime = %d\n", dwStartTime); 
    printf("EEEEEEEEEEEEEEEE EndTime = %d\n", dwEndTime); 
    printf("EEEEEEEEEEEEEEEE ElapsedTime = %d mSec\n", dwTotalElapsedTime); 
    printf("Average ElapsedTime = %d mSec\n", dwAvgTime); */
    
    if (!DSP_SUCCEEDED(status)) 
	{
		TSTLOG_appendLog(pHandle, "Status Val: %x\n",status);
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Load()\n");
        }
		graceful_exit();
    }

		
    DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);

    if (retVal == PASS) 
	{
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

