/*  ============================================================================
*   File    GetMsg.c
*
*   Path    $(TESTSUITEROOT)\Analysis\GetMsg
* 
*   Desc    Test node messaging. Get a fixed number of messages
*   to a message node.
*
*   Rev     1.0.0
*
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2003
*
*   ============================================================================
*   Revision History
*
*   SEP 15, 2002  REF=ORG             Raghunandan NagarajaRao
*      Original Version.
*   ============================================================================
*/
#include <stdio.h> 
#include <dbapi.h>
#include <rms_sh.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <string.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>   
#include <msgtxfr.h>

/* 
 *  ======== BssChk ========
 */


static SHORT            *pMsgBuf = NULL;

int BssChk(int argc, char ** argv)
{

   	COPY_TASK       copyTask;
	NODEARGS	NodeArgs;
        DSP_STATUS      status = DSP_SOK;
        INT             opt = 0;


	copyTask.hProcessor = NULL;
	copyTask.hNode = NULL;
    	copyTask.hInStream = NULL;

	optind = 0;
	

	while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:n:s:ZPR:")) != EOF) {
        switch (opt) {
         case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;

	      case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
			pHandle = TSTLOG_openLog(szLogName);
			fOpenLog = TRUE;
			break;

        case 'x':
          /* use given executable */
            aArgv[0] = (char *) optarg;
			fLoadDsp = TRUE;
			break;

		case 'n':
            /* set Number of transfers */
            		NodeArgs.iNumTxfrs = CSL_Atoi(optarg);
			break;

		case 'Z':
			/* set Transfer Mode as STRMMODE_ZEROCOPY */
			NodeArgs.iMode = STRMMODE_ZEROCOPY;
			NodeArgs.iSegmentId = DSP_SHMSEG0;
			break;
		case 'R':
			/* set Transfer Mode as STRMMODE_RDMA */
			NodeArgs.iMode = STRMMODE_RDMA;
			break;
		
        default:
				TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            break;
        }
    }

    if (DSP_SUCCEEDED(status)) 
	{
        /* Load user specified DSP Executable and start the processor. */
	        if(fLoadDsp)
			  status = InitializeProcessor(&copyTask);
    }


    if (DSP_SUCCEEDED(status)) {
        /* Perform node level initialization. */
        status = InitializeNode(&copyTask, &NodeArgs);
        if (DSP_SUCCEEDED(status)) 
		{
            status = RunTask(&copyTask, &NodeArgs);
        }
    }

    CleanupNode(&copyTask);
    if (copyTask.hProcessor) 
	{
        DSPProcessor_Detach(copyTask.hProcessor);
        copyTask.hProcessor = NULL;
    }

    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== InitializeProcessor ========
 *  Perform processor related initialization.
 */
static DSP_STATUS InitializeProcessor(COPY_TASK *copyTask)
{
    DSP_STATUS          status = DSP_SOK;
	INT					iArgc = 1;
	struct DSP_PROCESSORINFO   dspInfo;
    UINT                numProcs;
    UINT                index=0;
    INT                 procId=0;

        /* Attach to an available DSP (in this case, the 1st DSP). */
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64)
        {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }
    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, NULL,&copyTask->hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",
                status);
        }
    }
        else
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");

    if (DSP_SUCCEEDED(status)) 
	{
		status = DSPProcessor_Load(copyTask->hProcessor, iArgc, aArgv, aEnvp);  
		if (DSP_SUCCEEDED(status)) 
		{
			status = DSPProcessor_Start(copyTask->hProcessor);
			if (DSP_FAILED(status)) 
			{
				if(fOpenLog)
					TSTLOG_appendLog(pHandle, "Error in DSPProcessor Start. STATUS = %x\n", status);
			}
		}
		else
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Load. STATUS = %x\n", status);
		}
    }
    else 
	{
		if(fOpenLog)
			TSTLOG_appendLog(pHandle, "Error in DSPProcessor_Attach. Status = 0x%x\n", status);
    }
	return (status);
}

/*
 *  ======== InitializeNode ========
 *  Perform node related initialization.
 */
static DSP_STATUS InitializeNode(COPY_TASK *copyTask, NODEARGS *NodeArgs)
{
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA     *pArgs;
    struct DSP_NODEATTRIN  nodeAttrIn;
    struct DSP_UUID        uuid;
    DSP_STATUS      status = DSP_SOK;
    
    uuid = Msguuid;

    nodeAttrIn.uTimeout = 1000;
    nodeAttrIn.iPriority = 1;

#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrIn.uProfileID = 0xff;
#endif


    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;
    pArgs->cData[0] = 0;

    /* Allocate the message node. */
    status = DSPNode_Allocate(copyTask->hProcessor, &uuid, pArgs,&nodeAttrIn, &copyTask->hNode);
    if (DSP_SUCCEEDED(status)) 
	{
		/* Create the msg node on the DSP. */
		status = DSPNode_Create(copyTask->hNode);
		if (DSP_SUCCEEDED(status)) 
		{
			/* Start the msg node on the DSP. */
			status = DSPNode_Run(copyTask->hNode);
			if (DSP_FAILED(status)) 
			{
				if(fOpenLog)
					TSTLOG_appendLog(pHandle, "DSPNode_Run failed. Status = 0x%x\n",status);
				return (status);
			}
		}
		else 
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle,"DSPNode_Create failed. Status = 0x%x\n",status);
			return (status);
		}
    }
    else 
	{
	   TSTLOG_appendLog(pHandle, "DSPNode_Allocate failed\n");
	   TSTLOG_appendLog(pHandle, "Status = 0x%x\n",status);
	   return (status);
    }
    return (status);
}

/*
 *  ======== RunTask ========
 *  Run message task.
 */
static DSP_STATUS RunTask(COPY_TASK *copyTask, NODEARGS *NodeArgs)
{
    DSP_STATUS      status = DSP_SOK;
    DSP_STATUS      exitStatus = DSP_SOK;
    
    UINT            uTimeout = 10000;
    struct DSP_MSG         msg;
    UINT            k;
    SHORT           RcvMsg;
    INT             chkFlag=0;

	msg.dwCmd = RMS_USER + 15;
	TST_verbose("Checking .bss for initialization\n");
	TST_verbose("Please wait....\n");

	status = DSPNode_PutMessage(copyTask->hNode, &msg, uTimeout);

	if (DSP_SUCCEEDED(status)) 
	{			

				TSTLOG_appendLog(pHandle,"PUT Message is success\n");

				status = DSPNode_GetMessage(copyTask->hNode, &msg, uTimeout);
				if (DSP_FAILED(status)) 
				{
					if(fOpenLog)
					{
						TSTLOG_appendLog(pHandle, "DSPNode_GetMessage failed. Status = 0x%x\n",
						status);
						return(status);
					}
				}
				if(DSP_SUCCEEDED(status))
				{
					RcvMsg = (SHORT)msg.dwArg1;
					if(RcvMsg!=0)
					{
					 	TSTLOG_appendLog(pHandle,".bss section is not initialized to zero and the value of the variable is %d\n",RcvMsg);      
						chkFlag=1;
					}		
					
				}
				status = DSPNode_GetMessage(copyTask->hNode, &msg, uTimeout);
				if (DSP_FAILED(status))
                                {
                                        if(fOpenLog)
                                        {
                                                TSTLOG_appendLog(pHandle, "DSPNode_GetMessage failed. Status = 0x%x\n",status);
						return(status);
                                        }
                                }
                                if(DSP_SUCCEEDED(status)&&(chkFlag==0))
                                {
					RcvMsg = (SHORT)msg.dwArg1;
					if(RcvMsg!=0)
					{
						chkFlag=1;
					 	TSTLOG_appendLog(pHandle,".bss section is not initialized to zero and the value of the variable is %d\n",RcvMsg);      
					}		

                                }
				status = DSPNode_GetMessage(copyTask->hNode, &msg, uTimeout);
				if (DSP_FAILED(status))
                                {
                                        if(fOpenLog)
                                        {
                                                TSTLOG_appendLog(pHandle, "DSPNode_GetMessage failed. Status = 0x%x\n",status);
						return(status);
                                        }
                                }
				else
				{
					TSTLOG_appendLog(pHandle, "Address of the global Var is  = 0x%x\n",msg.dwArg1);
				}
				status = DSPNode_GetMessage(copyTask->hNode, &msg, uTimeout);
				if (DSP_FAILED(status))
                                {
                                        if(fOpenLog)
                                        {
                                                TSTLOG_appendLog(pHandle, "DSPNode_GetMessage failed. Status = 0x%x\n",status);
						return(status);
                                        }
                                }
				else
				{
					TSTLOG_appendLog(pHandle, "Address of the global Static Var is  = 0x%x\n",msg.dwArg1);
				}


	}
        else
        {
  	    if(fOpenLog)
            {
			
                TSTLOG_appendLog(pHandle, "LOG: FINAL STATUS = 0x%x\n", status);
            }
        }


	    if(chkFlag==0)
	    {
	    TSTLOG_appendLog(pHandle,"BSS CHECK Test Passed\n");
            TST_verbose( "status = 0x%x \n",status); 
	    }
	    else 
	    {
	    TSTLOG_appendLog(pHandle,"BSS CHECK Test FAIL\n");
	    }
    /* Terminate the node */
    status = DSPNode_Terminate(copyTask->hNode, &exitStatus);
    if (DSP_FAILED(status)) 
    {
        TSTLOG_appendLog(pHandle, "DSPNode_Terminate failed.\n");
    }

    return (status);
}

/*
 *  ======== CleanupNode ========
 *  Perform node related cleanup.
 */ 
static DSP_STATUS CleanupNode(COPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
    
    if (copyTask->hNode) 
    {
        /* Delete DSP node. */
        status = DSPNode_Delete(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPNode_Delete failed.\n");
        }

        copyTask->hNode = NULL;
    }

    return (status);
}


