/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    AsyncPutGetData.h
 *
 *   Path    $(TESTSUITEROOT)\Analysis\include\
 * 
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   APR 24, 2002    REF=ORG         RAGHUNANDHAN NAGARAJARAO
 *   Original Version.
 *   ============================================================================
 */

#include <dbapi.h>
#define NLOOPS              10
#define DEFAULTNBUFS        2
#define DEFAULTBUFSIZE      32
#define DEFAULTDSPUNIT      0
#define DEFAULTIVAUNIT      1
#define DEFAULTTIMEOUT      10000
#define DEFAULTALIGN        0
#define DEFAULTSEGMENT      0
#define ARGSIZE             32
#define LOOPS               8
#define WORDSIZE            2
#define DEFNAMESIZE         80
#define SMBUFSIZE           32   /* Size of SM allocation in bytes */
#define DSPEXEC             "msg_nortdx_int_24xx.dof55L"

static CHAR             szLogName[DEFNAMESIZE];
static CONST CHAR       *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1"};
static BOOL             fLoadDsp = FALSE, fOpenLog = FALSE;;
static CONST CHAR       *aEnvp[DEFNAMESIZE] = {NULL};
static FILE             *pHandle;

typedef enum {
    USER_VIRTUAL,
    USER_MAPPED,
    PHYSICAL
} MAPModel_t;

/* copy task context data structure. */
typedef struct {
    DSP_HPROCESSOR      hProcessor;     /* Handle to processor. */
    struct DSP_UUID            hNodeID;        /* Handle to node ID GUID. */
    DSP_HNODE           hNode;          /* Handle to node. */
    DSP_HSTREAM         hInStream;      /* Handle to node input stream. */
    DSP_HSTREAM         hOutStream;     /* Handle to node output stream. */
    BYTE                *pInBufs[DEFAULTNBUFS];     /* Input stream buffers. */
    BYTE                *pOutBufs[DEFAULTNBUFS];    /* Output stream buffers. */
    UINT                nodeEventMask;  /* Event Mask for Node*/
    struct DSP_NOTIFICATION*   *notifications; /* Pointer to array of Notification Objects */
} COPY_TASK; 

typedef struct {
    UINT    iBufferSize;
    UINT    iSegmentId;
    UINT    iMode;
    UINT    iNumTxfrs;
}NODEARGS;

typedef struct {
    DSP_HPROCESSOR      hProcessorIva;     /* Handle to IVA processor. */
    DSP_HNODE           hNodeIva;          /* Handle to IVA node. */
    INT                 numMsg;
}IVAMSG_TASK;


/* Forward declarations: */
/* Initialization and cleanup routines. */
static DSP_STATUS InitializeProcessor(COPY_TASK *copyTask);
static DSP_STATUS InitializeNode(COPY_TASK *copyTask, NODEARGS *NodeArgs);
static DSP_STATUS CleanupNode(COPY_TASK *copyTask);

static DSP_STATUS RunTask(COPY_TASK *copyTask, NODEARGS *NodeArgs);

/*Initialize and Cleanup routines for IVA */
static DSP_STATUS InitializeIvaProcessor(IVAMSG_TASK *ivaMsgTask);
static DSP_STATUS InitializeIvaNode(IVAMSG_TASK *ivaMsgTask);
static DSP_STATUS RunIvaTask(IVAMSG_TASK *ivaMsgTask);
static DSP_STATUS CleanupIvaNode(IVAMSG_TASK *ivaMsgTask);
static DSP_STATUS CleanupIvaProcessor(IVAMSG_TASK *ivaMsgTask);
/*Initialize and Cleanup routines for IVA Overlay*/
static DSP_STATUS InitializeIvaOvlyProcessor(IVAMSG_TASK *IvaOvlyTask);
static DSP_STATUS InitializeIvaOvlyNode(IVAMSG_TASK *IvaOvlyTask, struct DSP_UUID *uuidPtr);
static DSP_STATUS CleanupIvaOvlyNode(IVAMSG_TASK *IvaOvlyTask);
static DSP_STATUS RunIvaOvlyTask(IVAMSG_TASK *IvaOvlyTask);
static DSP_STATUS CleanupIvaOvlyProcessor(IVAMSG_TASK *IvaOvlyTask);

/* UUID's for PutGetMsg, SendMsg and GetMsg */
/* F95CA077_EE7D_11D5_A8DD_00B0D055F6D1*/
static const struct DSP_UUID Msguuid = {
    0xf95ca077, 0xee7d, 0x11d5, 0xa8, 0xdd, {
    0x00, 0xb0, 0xd0, 0x55, 0xf6, 0xd1    }
};

/*UUID's for IVA PutGetMsg ,SendMsg and GetMsg */
/* IVA_PING_TI_uuid = 6562DF2A_7068_42EF_8459_75584C1CDB8F */
static const struct DSP_UUID Iva_Msguuid = {
        0x6562df2a, 0x7068, 0x42ef, 0x84, 0x59, {
        0x75, 0x58, 0x4c, 0x1c, 0xdb, 0x8f
    }
};

/*
 * IVA OVERLAY NODES
 */
static const struct DSP_UUID IVA_OVLY1_TI_uuid = {
        0x892cb877, 0x2201, 0x4db1, 0xbc, 0x55, {
        0x65, 0x59, 0xc5, 0x07, 0x47, 0x33
    }
};

static const struct DSP_UUID IVA_OVLY2_TI_uuid = {
        0xb2e9c16d, 0x6ea7, 0x488e, 0x91, 0x66, {
        0xbe, 0xf6, 0x45, 0x4f, 0xc9, 0xd1
    }
};
