/*  ============================================================================
*   File    IvaSendMsg.c
*
*   Path    $(TESTSUITEROOT)\Analysis\IvaSendMsg
* 
*	Desc	Tests Message pumping of IVA.
*
*   Rev     1.0.0
*
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2004
*
*   ============================================================================
*   Revision History
*
*   SEP 15, 2002  REF=ORG             Raghunandan NagarajaRao
*   JULY 22,2004  Modified for IVA              Ramesh Gupta 
*   ============================================================================
*/

#include <stdio.h> 
#include <dbapi.h>
#include <rms_sh.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <string.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>   
#include <msgtxfr.h>

#define PING 4
/* 
 *  ======== sendMsg ========
 */
int IvaSendMsg(int argc, char ** argv)
{

   	IVAMSG_TASK     ivaMsgTask;
	DSP_STATUS      status = DSP_SOK;
    	INT             opt = 0;
        fLoadDsp = FALSE;
	ivaMsgTask.hProcessorIva = NULL;
	ivaMsgTask.hNodeIva = NULL;
	ivaMsgTask.numMsg = NULL;
	struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId;


	optind = 0;
	  while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:n:s:ZPR:")) != EOF) {
        switch (opt) {
         case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;

	      case 'l':
            /* use log and store with name location */
		        strncpy(szLogName, optarg, sizeof(szLogName));
			pHandle = TSTLOG_openLog(szLogName);
			fOpenLog = TRUE;
            		TSTLOG_appendLog(pHandle, "Log File Set to %s\n:", szLogName);
			break;

          case 'x':
            /* use given executable */
    		        aArgv[0] = (char *) optarg;
			fLoadDsp = TRUE;
			TSTLOG_appendLog(pHandle, "New Executable %s\n:", aArgv[0]);
			break;

		case 'n':
            /* set Number of transfers */
                        ivaMsgTask.numMsg = CSL_Atoi(optarg);
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "Number of Data Transfers =  %i\n", ivaMsgTask.numMsg);
			break;

		case 'D':
			/* set Transfer Mode as DMMCOPY */
		/*	NodeArgs.iMode = STRMMODE_ZEROCOPY;
			NodeArgs.iSegmentId = DSP_SHMSEG0;
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "Transfer Mode Set to =  %i\n", STRMMODE_ZEROCOPY);
		*/
			break;
		
        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            break;
        }
    }

	 while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
        }

    /* Attach to IVA */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, NULL,&ivaMsgTask.hProcessorIva);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle,"IVA:DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach failed. Status = 0x%x\n",
                status);
        }
    }
        else
       TSTLOG_appendLog(pHandle,"Failed to get the desired processor \n");







    if (DSP_SUCCEEDED(status)) 
	{
      /* Load user specified IVA Image and start the processor. */
		if(fLoadDsp)
			status = InitializeIvaProcessor(&ivaMsgTask);
        }


    if (DSP_SUCCEEDED(status)) {
        /* Perform node level initialization. */
        status = InitializeIvaNode(&ivaMsgTask);
        if (DSP_SUCCEEDED(status)) 
		{
			status = RunIvaTask(&ivaMsgTask);
			if (DSP_SUCCEEDED(status)) 
			{
				if(fOpenLog)
					TSTLOG_appendLog(pHandle,  "RunTask Succedded\n");
			}
        }
    }

    CleanupIvaNode(&ivaMsgTask);
    CleanupIvaProcessor(&ivaMsgTask);
    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== InitializeIvaProcessor ========
 *  Perform processor related initialization.
 */
static DSP_STATUS InitializeIvaProcessor(IVAMSG_TASK *ivaMsgTask)
{
	DSP_STATUS          status = DSP_SOK;
	INT		    iArgc = 1 ;

		status = DSPProcessor_Load(ivaMsgTask->hProcessorIva, iArgc, aArgv, aEnvp);  
		if (DSP_SUCCEEDED(status)) 
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPProcessor_Load succeeded\n");
			status = DSPProcessor_Start(ivaMsgTask->hProcessorIva);
			if (DSP_FAILED(status)) 
			{
				if(fOpenLog)
					TSTLOG_appendLog(pHandle, "Error in DSPProcessor Start.Status = 0x%x\n", status);
			}
		}
		else
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Load. Status = 0x%x\n",status);
		}
	return (status);
}

/*
 *  ======== InitializeIvaNode ========
 *  Perform node related initialization.
 */
static DSP_STATUS InitializeIvaNode(IVAMSG_TASK *ivaMsgTask)
{


    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA     *pArgsDsp;
    struct DSP_NODEATTRIN  nodeAttrInIva;
    DSP_STATUS      status = DSP_SOK;
    struct DSP_UUID        uuid;

    nodeAttrInIva.uTimeout = 10000;
    nodeAttrInIva.iPriority = 5;

    pArgsDsp = (struct DSP_CBDATA *) argsBuf;
    pArgsDsp->cbData = ARGSIZE;
     
    uuid=Iva_Msguuid;
    pArgsDsp = (struct DSP_CBDATA *)argsBuf;
    pArgsDsp->cbData = ARGSIZE;

    /* Allocate the message node. */
    status = DSPNode_Allocate(ivaMsgTask->hProcessorIva, &uuid, pArgsDsp,&nodeAttrInIva, &ivaMsgTask->hNodeIva);
    if (DSP_SUCCEEDED(status)) 
	{
		/* Create the msg node on the IVA. */
		status = DSPNode_Create(ivaMsgTask->hNodeIva);
		if (DSP_SUCCEEDED(status)) 
		{
			/* Start the msg PF  on the IVA. */
			status = DSPNode_Run(ivaMsgTask->hNodeIva);
			if (DSP_FAILED(status)) 
			{
				if(fOpenLog)
					TSTLOG_appendLog(pHandle, "DSPNode_Run failed. Status = 0x%x\n",status);
			}
		}
		else 
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle,"DSPNode_Create failed. Status = 0x%x\n",status);
		}
    }
    else 
	{
	   if(fOpenLog)
			TSTLOG_appendLog(pHandle, "DSPNode_Allocate failed. Status = 0x%x\n",status);
    }
	return (status);
}

/*
 *  ======== RunIvaTask ========
 *  Run IVA message task.
 */
static DSP_STATUS RunIvaTask(IVAMSG_TASK *ivaMsgTask)
{
		DSP_STATUS      status = DSP_SOK;
		UINT            uTimeout = 15000;
		struct DSP_MSG         ivaMsg;
		UINT            k;
		DWORD		dwStartTime;  
		DWORD		dwElapsedTime;
		DWORD		dwEndTime;
		ULONG		ulNumBytesTransferred = ivaMsgTask->numMsg;
		ULONG		ulBytesPerSec;

	        OsSpecificTimerStart(&dwStartTime);
		for(k = 0; k < ivaMsgTask->numMsg; k++)
		{
				/* Set msg SM buffer descriptor cmd code. */
				ivaMsg.dwCmd =  PING;
				status = DSPNode_PutMessage(ivaMsgTask->hNodeIva, &ivaMsg, uTimeout);
        
				if (DSP_FAILED(status)) 
				{
					if(fOpenLog)
					TSTLOG_appendLog(pHandle, "DSPNode_PutMessage for IVA failed. Status = 0x%x\n", status);
					break;
				}
				if ((k % 500) == 0)
				{
					TST_verbose("IVASENDMSG %d\n", k);
				}
		
		}

		OsSpecificTimerStop(&dwEndTime);
	
		OsSpecificTimerDifference(&dwEndTime, &dwStartTime, &dwElapsedTime);

        if (DSP_SUCCEEDED(status)) 
		{
		    ulNumBytesTransferred = ivaMsgTask->numMsg;
		    ulBytesPerSec = (ulNumBytesTransferred * 1000) / dwElapsedTime;
		    if(fOpenLog)
		    {
			    TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
   			    TSTLOG_appendLog(pHandle, "LOG: *********IVASENDMSG STATISTICS**********\n");
        		    TSTLOG_appendLog(pHandle, "LOG: FINAL STATUS = 0x%x\n", status);
		            TSTLOG_appendLog(pHandle, "LOG: BufferSize = Not Applicable\n");
			    TSTLOG_appendLog(pHandle, "LOG: StartTime = %d\n", dwStartTime); 
			    TSTLOG_appendLog(pHandle, "LOG: EndTime = %d\n", dwEndTime); 
			    TSTLOG_appendLog(pHandle, "LOG: ElapsedTime = %d\n", dwElapsedTime); 
			    TSTLOG_appendLog(pHandle, "LOG: Throughput = %d Messages/sec\n", ulBytesPerSec); 
			    TSTLOG_appendLog(pHandle, "LOG: Number of Transfers = %d\n", ulNumBytesTransferred);
			    TSTLOG_appendLog(pHandle, "LOG: **********END STATISTICS***********\n");
			    TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
			    
			    
            }
        }
        else
        {
  	        if(fOpenLog)
            {
                TSTLOG_appendLog(pHandle, "LOG: FINAL STATUS = 0x%x\n", status);
            }
            TST_verbose("status = 0x%x \n",status); 
        }
		return (status);
}

/*
 *  ======== CleanupIvaNode ========
 *  Perform node related cleanup.
 */ 
static DSP_STATUS CleanupIvaNode(IVAMSG_TASK *ivaMsgTask)
{
    DSP_STATUS      status  = DSP_SOK;
    DSP_STATUS      exitStatus = DSP_SOK;


  if (ivaMsgTask->hNodeIva) 
	{

                // Terminate IVA node.
        status = DSPNode_Terminate(ivaMsgTask->hNodeIva, &exitStatus);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Terminate:IVA succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPNode_Terminate:IVA failed: 0x%x\n", status);
        }

                /* Delete IVA node. */
        status = DSPNode_Delete(ivaMsgTask->hNodeIva);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Delete:IVA succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPNode_Delete:IVA failed: 0x%x\n", status);
        }

                ivaMsgTask->hNodeIva = NULL;

        }
        else
        {
                TSTLOG_appendLog(pHandle, " Bad Node Handle \n");
        }

    return (status);

}
static DSP_STATUS CleanupIvaProcessor(IVAMSG_TASK *ivaMsgTask)
{
DSP_STATUS      status =DSP_SOK;
if(ivaMsgTask->hProcessorIva){
	status=DSPProcessor_Detach(ivaMsgTask->hProcessorIva);
	if (DSP_FAILED(status))
                {
                        if(fOpenLog)
                                TSTLOG_appendLog(pHandle,  "DSPProcessor_Detach failed.status = 0x%x\n", status);
                }

        ivaMsgTask->hProcessorIva=NULL;
}


}
