/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DoubleDSPImageLoad.c
 *
 *   Path    $(TESTSUITEROOT)\Behaviour\DoubleDSPImageLoad\
 * 
 *   Desc:   Double DSP Image test case
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Raghunandhan NagarajaRao
 *      Original Version.
 *   ============================================================================
 */


/*  bCleanupFlag : (checked between loading of first and second images)
If bCleanupFlag is equal to 0
	- do not detach the processor.
	- Loads the second image.
	- Checks for dataintegrity

If bCleanupFlag is equal to 1
	- Detaches the processor.
	- Attaches the processor again.
	- Loads the second image.
	- Checks for dataintegrity */


#include <stdio.h> 
#include <string.h>
#include <getopt_test.h>
#include <dbapi.h>
#include <tst.h>
#include <tstlog.h>
#include <DSPProcessor_OEM.h>
#include <DoubleImageLoad.h>
#include <testosal.h>

NODEARGS		    NodeArgsDdsp;
BOOL			    fAttachRequired = TRUE;

/* Loacal Function Declaration */
static DSP_STATUS SendTestConfiguration(SHORT *pInBuf, SHORT *pOutBuf, 
					STRMCOPY_TASK *copyTask, DWORD dwSendBufSize, 
					DWORD dwRecvBufSize);
                          
/* 
 *  ======== doubleDspImageLoad ========
 */
int doubleDspImageLoad(int argc, char ** argv)
{
    STRMCOPY_TASK   strmcopyTask;
    DSP_STATUS      status = DSP_SOK;
    
    /* Process command line arguments, open data files: */
    status = ProcessArgs(argc, argv);
   
    strmcopyTask.hProcessor = NULL;
    strmcopyTask.hNode = NULL;
    strmcopyTask.hOutStream = NULL;
    strmcopyTask.hInStream = NULL;

    strmcopyTask.ppInBufs = NULL;
    strmcopyTask.ppOutBufs = NULL;

    if (DSP_SUCCEEDED(status)) 
	{
			/* Perform processor level initialization. */
			if(fLoadFirstDsp)
			{
				status = InitializeProcessor(&strmcopyTask);
			}
			else
			{
				TSTLOG_appendLog(pHandleDdsp, "CAUTION: LOADING FIRST DEFAULT IMAGE.\n");
				TSTLOG_appendLog(pHandleDdsp, "DATA WILL BE TXFRD. INTEGRITY CANNOT BE CHECKED\n");
			}
            fLoadFirstDsp = FALSE;
    }

    if (DSP_SUCCEEDED(status)) 
	{
        
        /* Perform node level initialization. */
        status = InitializeNode(&strmcopyTask);
        if (DSP_SUCCEEDED(status)) 
		{
            
            /* Perform stream level initialization. */
            status = InitializeStreams(&strmcopyTask);
            if (DSP_SUCCEEDED(status)) 
			{
                
                /* Run task. */
                status = RunTask(&strmcopyTask);
                if (DSP_FAILED(status)) 
				{
                    TSTLOG_appendLog(pHandleDdsp, "RunTask failed.\n");
                }
            }
        }
    }
    
    CleanupStreams(&strmcopyTask);
    CleanupNode(&strmcopyTask);
    /* Close opened files. */
	if(NodeArgsDdsp.bCleanupFlag == 1)
	{
	
		CleanupProcessor(&strmcopyTask);
        fAttachRequired = TRUE;
    }
    else
    {
        fAttachRequired = FALSE;
    }
	if(fLoadSecondDsp)
	{
		status = InitializeProcessor(&strmcopyTask);
	}
	else
	{
		TSTLOG_appendLog(pHandleDdsp, "CAUTION: LOADING SECOND DEFAULT IMAGE.\n");
		TSTLOG_appendLog(pHandleDdsp, "DATA WILL BE TXFRD. INTEGRITY CANNOT BE CHECKED\n");
	}
    if (DSP_SUCCEEDED(status)) 
	{
    
		/* Perform node level initialization. */
		status = InitializeNode(&strmcopyTask);
        
		if (DSP_SUCCEEDED(status)) 
		{
        
			/* Perform stream level initialization. */
			status = InitializeStreams(&strmcopyTask);
			if (DSP_SUCCEEDED(status)) 
			{
            
				/* Run task. */
				status = RunTask(&strmcopyTask);
				if (DSP_FAILED(status)) 
				{
					TSTLOG_appendLog(pHandleDdsp, "RunTask failed.\n");
				}
			}
		}
	}

	CleanupStreams(&strmcopyTask);
	CleanupNode(&strmcopyTask);
	CleanupProcessor(&strmcopyTask);
    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== InitializeProcessor ========
 *  Perform processor related initialization.
 */
static DSP_STATUS InitializeProcessor(STRMCOPY_TASK *copyTask)
{
    DSP_STATUS          status = DSP_SOK;
	INT					iArgc = 1;

    /* Attach to an available DSP (in this case, the 1st DSP). */

	if(fAttachRequired == TRUE)
	{
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;
    
        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandleDdsp,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
        
            status = DSPProcessor_Attach(procId, NULL, &copyTask->hProcessor);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandleDdsp, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandleDdsp, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandleDdsp, "Failed to get the desired processor \n");
        }
	}
	else
	{
		TSTLOG_appendLog(pHandleDdsp,  "CleanupFlag = 0. Processor Already Attached.\n");
	}
    
	if (DSP_SUCCEEDED(status)) 
	{
		if((fAttachRequired == TRUE) && fLoadFirstDsp)
		{
			status = DSPProcessor_Load(copyTask->hProcessor, iArgc, (CONST CHAR **)aFirstImage, 
							(CONST CHAR **)aEnvp);  
		}
		else
		{
			status = DSPProcessor_Load(copyTask->hProcessor, iArgc, (CONST CHAR **)aSecondImage, 
							(CONST CHAR **)aEnvp);  
		}

		if (DSP_SUCCEEDED(status)) 
		{
				    /* Start Processor */
				    status = DSPProcessor_Start(copyTask->hProcessor);
				    if (DSP_FAILED(status)) 
				    {
					    TSTLOG_appendLog(pHandleDdsp, "FAIL\n");
					    TSTLOG_appendLog(pHandleDdsp, "Error in DSPProcessor Start");    
				    }
		}
		else
		{
				TSTLOG_appendLog(pHandleDdsp,"Error in DSPProcessor_Load.STATUS = %x\n", status);
		}
	}
	return (status);
}


/*
 *  ======== InitializeNode ========
 *  Perform node related initialization.
 */
static DSP_STATUS InitializeNode(STRMCOPY_TASK *copyTask)
{
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA     *pArgs;
    struct DSP_NODEATTRIN  nodeAttrIn;
    struct DSP_STRMATTR    attrs;
    struct DSP_UUID        uuid;
    DSP_STATUS      status = DSP_SOK;

    attrs.uSegid = 0;
    attrs.uBufsize = NodeArgsDdsp.iBufferSize;
    attrs.uNumBufs = DEFAULTNBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = DSP_FOREVER;

    uuid = inoutuuid;

    if(fLoadFirstDsp == FALSE)
        uuid = inoutuuid;

	switch(NodeArgsDdsp.iMode)
	{
		case '0' : 
			attrs.lMode = STRMMODE_PROCCOPY; 
			break;
		case '1' : 
			attrs.lMode = STRMMODE_ZEROCOPY; 
			break;
		case '2' : 
			attrs.lMode = STRMMODE_RDMA; 
			break;
		default:
			attrs.lMode = STRMMODE_PROCCOPY; 
			break;
	}
    
    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;
    nodeAttrIn.uTimeout = 10000;
    nodeAttrIn.iPriority = 5;

#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrIn.uProfileID = 0xff;
#endif
    /* Allocate the strmcopy node. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Allocate(copyTask->hProcessor, &uuid, pArgs,
                &nodeAttrIn, &copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleDdsp, "DSPNode_Allocate failed. Status = 0x%x\n",
                status);
        }
    }
    
    /* Connect the strmcopy node to the GPP. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, copyTask->hNode,
                0, &attrs);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleDdsp, "DSPNode_Connect failed. Status = 0x%x\n",
                    status);
        }
    }

    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Connect(copyTask->hNode, 0, (DSP_HNODE)DSP_HGPPNODE,
            0, &attrs);
        if (DSP_FAILED(status)) {
            TSTLOG_appendLog(pHandleDdsp, "DSPNode_Connect failed. Status = 0x%x\n",
                    status);
        }
    }

    /* Create the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Create(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleDdsp, "DSPNode_Create failed. Status = 0x%x\n",
                status);
        }
    }
    
    /* Start the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Run(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleDdsp, "DSPNode_Run failed. Status = 0x%x\n",
                status);
        }
    }
    
    return (status);
}




/*
 *  ======== InitializeStreams ========
 *  Perform stream related initialization.
 */
static DSP_STATUS InitializeStreams(STRMCOPY_TASK  *copyTask)
{
    DSP_STATUS      status = DSP_SOK;

    /* Open an input data stream from the host to the strmcopy node. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPStream_Open(copyTask->hNode, DSP_TONODE, 0, 
            NULL, &copyTask->hOutStream);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleDdsp, 
                "DSPStream_Open of input stream failed. Status = 0x%x\n",
                status);
        }
    }

    /* Open an output stream from the strmcopy node to the host. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPStream_Open(copyTask->hNode, DSP_FROMNODE, 0, 
            NULL, &copyTask->hInStream);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleDdsp, 
                "DSPStream_Open of output stream failed. Status = 0x%x\n",
                status);
        }
    }
    
    /* Allocate and issue buffer to input data stream. */
    if (DSP_SUCCEEDED(status)) 
    {
        copyTask->ppInBufs = (BYTE**)OsSpecificMemAlloc(sizeof(BYTE*) * DEFAULTNBUFS);

        status = DSPStream_AllocateBuffers(copyTask->hOutStream, 
            NodeArgsDdsp.iBufferSize, copyTask->ppInBufs, DEFAULTNBUFS);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleDdsp, 
                "DSPStream_AllocateBuffers for input stream failed. "
                "Status = 0x%x\n", status);
        }
    }
    
    /* Allocate and issue buffer to output data stream. */
    if (DSP_SUCCEEDED(status)) 
    {
        copyTask->ppOutBufs = (BYTE**)OsSpecificMemAlloc(sizeof(BYTE*) * DEFAULTNBUFS);

        status = DSPStream_AllocateBuffers(copyTask->hInStream, 
            NodeArgsDdsp.iBufferSize, copyTask->ppOutBufs, DEFAULTNBUFS);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleDdsp, 
                "DSPStream_AllocateBuffers for output stream failed. "
                "Status = 0x%x\n", status);
        }
    }
    
    return (status);
}


/*
 *  ======== RunTask ========
 *  Run strmcopy task.
 */
static DSP_STATUS RunTask(STRMCOPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
    BYTE            *pInBuf = NULL;
    BYTE            *pOutBuf = NULL;
    DWORD           dwArg = 0;
    DWORD           dwBufSize = NodeArgsDdsp.iBufferSize;
	UINT			k;
	DWORD			dwStartTime;  
    DWORD			dwElapsedTime;
	DWORD			dwEndTime;
	ULONG			ulNumBytesTransferred = 0;
	ULONG			ulBytesPerSec = 0;

    pInBuf = copyTask->ppInBufs[0];
    pOutBuf = copyTask->ppOutBufs[0];
	status = SendTestConfiguration((SHORT *)pInBuf, (SHORT *)pOutBuf, copyTask, dwBufSize, dwBufSize);
    
    /* 
    Read a fixed-size buffer from a data stream, send buffer to DSP, 
    and receive the same buffer back from the DSP.
    Read a block of data from the input file:
    cBytesRead = fread(pInBuf, sizeof(BYTE), DEFAULTBUFSIZE, 
    nFile);*/

    // Add I/O request to DSP.
	OsSpecificTimerStart(&dwStartTime);
	for ( k =0; k < NodeArgsDdsp.iBufferSize; k++)
	{
			pInBuf[k] = k;
	}
	for( k = 0; k < NodeArgsDdsp.iNumTxfrs; k++)
	{
            status = DSPStream_Issue(copyTask->hOutStream, pInBuf, 
            NodeArgsDdsp.iBufferSize, NodeArgsDdsp.iBufferSize, 0);
            if (DSP_FAILED(status)) 
			{
                TSTLOG_appendLog(pHandleDdsp, "DSPStream_Issue Failed.\n");
            }

					          
            // Wait for output completion to reclaim a queued buffer:
            status = DSPStream_Reclaim(copyTask->hOutStream, 
                &pInBuf, &dwBufSize, NULL, &dwArg);

			 if (DSP_FAILED(status)) 
             {
                 TSTLOG_appendLog(pHandleDdsp, "DSPStream_Reclaim Input Buffer Failed\n");
             }

   			// Put a buffer into the input channel to get back buffer sent.
            status = DSPStream_Issue(copyTask->hInStream, pOutBuf, 
                NodeArgsDdsp.iBufferSize, NodeArgsDdsp.iBufferSize, 0);
            
            if (DSP_FAILED(status)) 
			{
                 TSTLOG_appendLog(pHandleDdsp, "DSPStream_Issue OutBuffer Failed.\n");
            }
            
            // Wait for input notification to reclaim a queued buffer:
            status = DSPStream_Reclaim(copyTask->hInStream, 
                &pOutBuf, &dwBufSize, NULL, &dwArg);
            
            if (status == DSP_ETIMEOUT) 
			{
                TSTLOG_appendLog(pHandleDdsp, "DSPStream_Reclaim timed out\n");
            }
            else if (DSP_FAILED(status)) 
			{
                TSTLOG_appendLog(pHandleDdsp, "DSPStream_Reclaim OutBuffer Failed, 0x%x.\n", 
                    status);
            }
			if((k % 500 == 0) && (status == DSP_SOK))
				 TSTLOG_appendLog(pHandleDdsp, "Issue Reclaim Status at iteration %d is OK\n", k); 

	} // end of for( k = 0; k < NodeArgsDdsp.iNumTxfrs; k++)       
	OsSpecificTimerStop(&dwEndTime);

	//ulElapsedTime = (ulEndTime - ulStartTime);
	OsSpecificTimerDifference(&dwEndTime, &dwStartTime, &dwElapsedTime);



    if(fOpenLog)
	{
		TSTLOG_appendLog(pHandleDdsp, "LOG: ***********************************\n");
		TSTLOG_appendLog(pHandleDdsp, "LOG: *********ISSUE RECLAIM STATISTICS**********\n");
      	TSTLOG_appendLog(pHandleDdsp, "LOG: FINAL STATUS = 0x%x\n", status);
		TSTLOG_appendLog(pHandleDdsp, "LOG: BufferSize = %d\n", dwBufSize);
		TSTLOG_appendLog(pHandleDdsp, "LOG: StartTime = %d\n", dwStartTime); 
		TSTLOG_appendLog(pHandleDdsp, "LOG: EndTime = %d\n", dwEndTime); 
		TSTLOG_appendLog(pHandleDdsp, "LOG: ElapsedTime = %d\n", dwElapsedTime); 
		TSTLOG_appendLog(pHandleDdsp, "LOG: Throughput = %d Bytes/sec\n", ulBytesPerSec); 
		TSTLOG_appendLog(pHandleDdsp, "LOG: ulNumBytesTransferred = %d\n", ulNumBytesTransferred);
		TSTLOG_appendLog(pHandleDdsp, "LOG: **********END STATISTICS***********\n");
		TSTLOG_appendLog(pHandleDdsp, "LOG: ***********************************\n");
	}
	return (status);
}

/*
 *  ======== CleanupStreams ========
 *  Perform stream related cleanup.
 */ 
static DSP_STATUS CleanupStreams(STRMCOPY_TASK *copyTask)
{
    struct DSP_STREAMINFO  streamInfo;
    DSP_STATUS      status = DSP_SOK;
    DWORD           dwArg = 0;
    DWORD           dwBufsize = NodeArgsDdsp.iBufferSize;


    /* Reclaim and free stream buffers. */
    if (copyTask->hOutStream) 
    {
        DSPStream_Idle(copyTask->hOutStream, TRUE);
        status = DSPStream_GetInfo(copyTask->hOutStream, &streamInfo,
            sizeof(streamInfo));

        if (copyTask->ppInBufs) 
        {
            /* Reclaim the buffer if it's still in the stream */
            if (streamInfo.uNumberBufsInStream) 
            {
                status = DSPStream_Reclaim(copyTask->hOutStream,
                    &copyTask->ppInBufs[0], &dwBufsize, NULL, &dwArg);
            }
            status = DSPStream_FreeBuffers(copyTask->hOutStream, 
                    copyTask->ppInBufs, DEFAULTNBUFS);
            if (DSP_FAILED(status)) 
            {
                 TSTLOG_appendLog(pHandleDdsp,  "DSPStream_FreeBuffer of input buffer failed. "
                    "Status = 0x%x\n", status);
            }

            copyTask->ppInBufs = NULL;
            /* Close open streams. */
            status = DSPStream_Close(copyTask->hOutStream);
            if (DSP_FAILED(status)) 
            {
                 TSTLOG_appendLog(pHandleDdsp, "DSPStream_Close of input stream failed. "
                    "Status = 0x%x\n", status);
            }

            copyTask->hOutStream = NULL;
        }
    }
    
    if (copyTask->hInStream) 
    {
        DSPStream_Idle(copyTask->hInStream, TRUE);

        /* Need to determine the number of buffers left in the stream */
        status = DSPStream_GetInfo(copyTask->hInStream, &streamInfo,
            sizeof(streamInfo));

        if (copyTask->ppOutBufs) 
        {
            if (streamInfo.uNumberBufsInStream) {
                status = DSPStream_Reclaim(copyTask->hInStream, 
                        &copyTask->ppOutBufs[0], &dwBufsize, NULL, &dwArg);
            }

            /* Free buffer. */
            status = DSPStream_FreeBuffers(copyTask->hInStream, 
                copyTask->ppOutBufs, DEFAULTNBUFS);
            if (DSP_FAILED(status)) 
            {
                 TSTLOG_appendLog(pHandleDdsp, 
                    "DSPStream_FreeBuffer of output buffer failed. "
                    "Status = 0x%x\n", status);
            }
            copyTask->ppOutBufs = NULL;
        }

        status = DSPStream_Close(copyTask->hInStream);
        if (DSP_FAILED(status)) {
             TSTLOG_appendLog(pHandleDdsp, "DSPStream_Close of output stream failed . "
                "Status = 0x%x\n", status);
        }

        copyTask->hInStream = NULL;
    }
    
    return (status);
}


/*
 *  ======== CleanupNode ========
 *  Perform node related cleanup.
 */ 
static DSP_STATUS CleanupNode(STRMCOPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
    DSP_STATUS      exitStatus = DSP_SOK;
    
    if (copyTask->hNode) 
    {

        // Terminate DSP node.
        status = DSPNode_Terminate(copyTask->hNode, &exitStatus);
        if (DSP_FAILED(status)) 
        {
             TSTLOG_appendLog(pHandleDdsp, "DSPNode_Terminate failed: 0x%x\n", status);
        }

        /* Delete DSP node. */
        status = DSPNode_Delete(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
             TSTLOG_appendLog(pHandleDdsp, "DSPNode_Delete failed: 0x%x\n", status);
        }

        copyTask->hNode = NULL;
    }

    return (status);
}


/*
 *  ======== CleanupProcessor ========
 *  Perform processor related cleanup.
 */ 
static DSP_STATUS CleanupProcessor(STRMCOPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;

    if (copyTask->hProcessor) 
    {
        /* Detach from processor. */
        status = DSPProcessor_Detach(copyTask->hProcessor);
        if (DSP_FAILED(status)) 
        {
             TSTLOG_appendLog(pHandleDdsp, "DSPProcessor_Detach failed.\n");
        }

        copyTask->hProcessor = NULL;
    }

    return (status);
}


/* 
 *  ======== ProcessArgs ========
 *  Process command line arguments for this sample, returning input and
 *  output file handles.
 */
static DSP_STATUS ProcessArgs(int argc, char ** argv)
{
    DSP_STATUS      status = DSP_SOK;
	INT             opt = 0;
	NodeArgsDdsp.iSegmentId = DEFAULTSEGMENT;
	NodeArgsDdsp.iMode  = 0;
	optind = 0;

        while ((opt = getopt_test(argc, (char *const)argv, "+vl:x:y:c:b:n:ZPR:")) != EOF) {
        switch (opt) {
		 case 'v':
			/* verbose */
			TST_setVerbose(TRUE);
			break;

		  case 'l':
			/* use log and store with name location */
			strncpy(szLogNameDdsp, optarg, sizeof(szLogNameDdsp));
			pHandleDdsp = TSTLOG_openLog(szLogNameDdsp);
            fOpenLog = TRUE;
			break;

		  case 'x':
			/* use given executable */
			aFirstImage[0] = (char *) optarg;
			fLoadFirstDsp = TRUE;
			break;
		  case 'y':
			/* use given executable */
			aSecondImage[0] = (char *) optarg;
			fLoadSecondDsp = TRUE;
			break;
		  case 'c':
			/* set Buffer Size */
			NodeArgsDdsp.bCleanupFlag = CSL_Atoi(optarg);
			break;

		  case 'b':
			/* set Buffer Size */
			NodeArgsDdsp.iBufferSize = CSL_Atoi(optarg);
			break;

		  case 'n':
				/* set Number of transfers */
			NodeArgsDdsp.iNumTxfrs = CSL_Atoi(optarg);
			break;

          case 'Z':
			/* set Transfer Mode as STRMMODE_ZEROCOPY */
			NodeArgsDdsp.iMode = STRMMODE_ZEROCOPY;
			NodeArgsDdsp.iSegmentId = DSP_SHMSEG0;
			break;

          case 'R':
			/* set Transfer Mode as STRMMODE_RDMA */
			NodeArgsDdsp.iMode = STRMMODE_RDMA;
			break;
			
		  default:
			TST_verbose(("Invalid option: %c\n"), opt);
			break;
        }
    }
	
		return (status);
}



static DSP_STATUS SendTestConfiguration(SHORT *pInBuf, 
										SHORT *pOutBuf, 
										STRMCOPY_TASK *copyTask, 
										DWORD dwSendBufSize, 
										DWORD dwRecvBufSize)
{

		DSP_STATUS status = DSP_SOK;
		UINT			k;
		DWORD			dwArg;
		pInBuf[0] = dwSendBufSize / 2;
		pInBuf[1] = dwRecvBufSize / 2;
		pInBuf[2] = NodeArgsDdsp.iNumTxfrs & 0xffff ;
		pInBuf[3] = NodeArgsDdsp.iNumTxfrs >> 16 ;

        for ( k = 4; k < NodeArgsDdsp.iBufferSize / 2; k++)
		{
				pInBuf[k] = k;
		}

		status = DSPStream_Issue(copyTask->hOutStream, (BYTE *)pInBuf, 
        NodeArgsDdsp.iBufferSize, NodeArgsDdsp.iBufferSize, 0);
        if (DSP_FAILED(status)) 
		{
				TSTLOG_appendLog(pHandleDdsp, "DSPStream_Issue %d failed.\n", k);
        }

		status = DSPStream_Reclaim(copyTask->hOutStream, 
						(BYTE **)&pInBuf, &dwRecvBufSize, NULL, &dwArg);
        if (DSP_FAILED(status)) 
		{
				TSTLOG_appendLog(pHandleDdsp, "DSPStream_Reclaim %d failed.\n", k);
        }
		return status;
}
