/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    RegUnregNodes.c
 *
 *   Path    $(TESTSUITEROOT)/TestAPI/DSPManager/
 *
 *   Desc:
 *           This tool is used to register and unregister
 *			 dynamic Nodes into the DCD.
 *           The program takes UUID and DLL name as parameters and
 *			 based on the command, registers or unregisters the node.
 *
 *  Usage:
 *        REGUNREG [-v] [-l "log path/filename"]
 *                      [-d "valid dll name"] [-u "uuid"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -s: Buffer Size
 *		  -u: UUID string
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 *
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 23, 2003    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>

#define DEFNAMESIZE      100
#define NUMOFCOMPARGS      2	/* Compulsory args: n,s */
#define PASS               0
#define FAIL               1

#define SHORT short

#define DSPEXEC         "dataxfr_nortdxidl.dof"
#define DLLNAME         "/release/inoutdyn.dll55l"
#define UUIDNAME		"F95CA076_EE7D_11D5_A8DD_00B0D055F6D1"

typedef enum
{
	REGISTER,
	UNREGISTER
} NodeAction;

/*
 *  ======== function to register and unregister dynamic nodes ========
 *
 */
INT regunregDynNodes_API(INT argc, CHAR * argv[])
{
    BOOL           			bUseLog=FALSE;

    FILE           			*pHandle = NULL;

	CHAR           			szLogName[DEFNAMESIZE];
	INT            			iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS     			status;
	INT						opt;
    INT                 	retVal = PASS;
	CHAR        			*aDllName[DEFNAMESIZE] = { DLLNAME, "arg3" };
	CHAR        			*aUuidName[DEFNAMESIZE] = { UUIDNAME, "arg3" };
	struct DSP_UUID				gidNodeID;
	NodeAction				action;
	INT						nodetype;
	DSP_DCDOBJTYPE			objtype=DSP_DCDNODETYPE;


	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Register or Unregister the nodes into DCD based on command */
	/* End !! */


	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt_test(argc, argv, "+vl:d:i:UR:m:");
    while (opt != EOF) {
	    //printf("INSIDE WHILE LOOOOOOOOOOOOOOOOOOP opt = 0x%c\n",opt);
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DynamicLink_BEHAVIOUR: Verbose Mode On\n");
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "dynLink test:");
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
		case 'd':
        	/* use given dll */
        	//printf("Changing the dll name \n");
        	aDllName[0] = (char *) optarg;
            break;
		case 'i':
        	/* use given dll */
        	//printf("changing the UUID \n");
        	aUuidName[0] = (char *) optarg;
        	iMandArgCount--;
            break;
        case 'U':
        	//printf("ACTION UNREG\n");
        	action = UNREGISTER;
        	break;
        case 'R':
        	//printf("ACTION REGISTER\n");
        	action = REGISTER;
        	break;
		case 'm':
        	/* mode of registration of dll */
        	//printf("Node type for the DLL \n");
        	nodetype = CSL_Atoi(optarg);
        	iMandArgCount--;
        	switch(nodetype)
        	{
	        	case 0:
	        		//printf("NODETYPE\n");
	        		objtype = DSP_DCDNODETYPE;
	        	break;
	        	case 1:
	        		//printf("DSP_DCDLIBRARYTYPE\n");
	        		objtype = DSP_DCDLIBRARYTYPE;
	        	break;
	        	case 2:
	        		//printf("DSP_DCDCREATELIBTYPE\n");
	        		objtype = DSP_DCDCREATELIBTYPE;
	        	break;
	        	case 3:
	        		//printf("DSP_DCDEXECUTELIBTYPE\n");
	        		objtype = DSP_DCDEXECUTELIBTYPE;
	        	break;
	        	case 4:
	        		//printf("DSP_DCDDELETELIBTYPE\n");
	        		objtype = DSP_DCDDELETELIBTYPE;
	        	break;
        	}
            break;
        default:
        	//printf("INVALID OPTION %c\n",opt);
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt_test(argc, argv, "+vl:d:i:UR:m:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
	}

	TSTLOG_appendLog(pHandle,"UUID %s\n",aUuidName[0]);
	//printf("DDDDDDDDLLLLLLLLLLLLLLLLLL %s\n",aDllName[0]);
    UUID_UuidFromString((CHAR *)aUuidName[0], &gidNodeID);
    /* Un/Register the node information with DCD */

    if(action == REGISTER)
    {
	    TSTLOG_appendLog(pHandle,"Registering.....................\n");
	    status = DSPManager_RegisterObject(&gidNodeID, objtype, aDllName[0]);
	    if (!DSP_SUCCEEDED(status))
		{
	        TST_verbose("FAIL\n");
	        if (bUseLog)
			{
	            TSTLOG_appendLog(pHandle, "FAIL status= 0x%x\n",status);
	            TSTLOG_appendLog(pHandle,"Error in DSPManager_RegisterObject().\n");
	        }
	    }
    }
    else
    {
	    TSTLOG_appendLog(pHandle,"Unregistering ..................\n");
	    status = DSPManager_UnregisterObject(&gidNodeID, objtype);
	    //printf("UNREG STATUS = 0x%x\n",status);
    }

    if (DSP_SUCCEEDED(status))
	{
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

