/*  ============================================================================
 *   File    WaitForEvent.c
 *
 *   Path    $(TESTSUITEROOT)\Behaviour\Event\
 * 
 *   Desc:
 *           Behaviour test for DSPManager_WaitForEvent API.
 *
 *
 *   Usage:
 *
 *        WAITEVENT [-v] [-l "log path/filename"] 
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   NOV 14, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <dbdefs.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include "waitforevent.h"
#include <DSPProcessor_OEM.h>

/*
 * Function waitForEvent_BEHAVIOUR
 */

static DSP_STATUS CheckMmuEvents(TASK_CONTEXT *copyTask);
static CHAR	   *aArgv[DEFNAMESIZE] = { DSPEXEC, "iva.dof" };
static DSP_STATUS StopIvaProcessor(TASK_CONTEXT *copyTask);

INT IvaPwrWaitForEvent_BEHAVIOUR(INT argc, CHAR * argv[])
{

	/* Initialise Processor and Node
	 * Create the objects needed for Events
	 * Register the Processor and Node related Events
	 * Call Wait for Events with the list of Notification objects and the mask
	 * Based on the type of the event, take appropriate action
	 */

    BOOL           bUseLog=FALSE;
    FILE           *pHandle = NULL;
    INT            iNumTransfers;
    INT            iBuffSize;
    CHAR           szLogName[DEFNAMESIZE];
    INT            iMandArgCount = NUMOFCOMPARGS;
    DSP_STATUS     status;
    TASK_CONTEXT   contextTask;
    INT		   opt;

	
	contextTask.notifications = (struct DSP_NOTIFICATION* *)OsSpecificMemAlloc(NUMNOTIFOBJECTS * sizeof(struct DSP_NOTIFICATION*));
	contextTask.strmnotifications = (struct DSP_NOTIFICATION* *)OsSpecificMemAlloc(3 * sizeof(struct DSP_NOTIFICATION*));

	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt_test(argc, (char *const)argv, "+vl:x:n:s:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            pHandle = TSTLOG_openLog(szLogName);
			contextTask.pHandle = pHandle;
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            break;
		case 'n':
			/* Number of Transfers */
			iNumTransfers = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
		case 's':
			/* Buffer Size */
			iBuffSize = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);

            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt_test(argc, (char *const)argv, "+vl:x:n:s:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
   		TSTLOG_appendLog(pHandle, "FAIL : ");
		TSTLOG_appendLog(pHandle,"Number of Mandatory Arguments is incorrect.\n");
		if (bUseLog) 
		{
		    TSTLOG_closeLog(pHandle);
		} 
		return 0;
	}


	/* Start The actual Test Case */
    

    status = DspManager_Open(0, NULL);

    contextTask.hProcessor = NULL;
    contextTask.hNode = NULL;

    if (DSP_SUCCEEDED(status)) 
	{
        /* Perform processor level initialization. */
      TSTLOG_appendLog(pHandle,"Calling for MMU FAULT Events.\n");
      StopIvaProcessor(&contextTask);

      status = CheckMmuEvents(&contextTask);
    }


    CleanupNode(&contextTask);
//    StopIvaProcessor(&contextTask);
    CleanupProcessor(&contextTask);

    DspManager_Close(0,NULL);

	if(status == DSP_SOK)
	{
		TSTLOG_appendLog(pHandle, "WAITFOR EVENT TEST CASE PASS\n");
	}
	else
	{
		TSTLOG_appendLog(pHandle, "WAITFOR EVENT TEST CASE FAIL ststus = 0x%x\n",status);
	}
	return (status);

} /* End of waitForEvent_BEHAVIOUR */




/*
 *  ======== CheckMmuEvents ========
 *  Perform node related initialization.
 */
static DSP_STATUS CheckMmuEvents(TASK_CONTEXT *copyTask)
{
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA      *pArgs;
    struct DSP_NODEATTRIN  nodeAttrIn;
    struct DSP_STRMATTR    attrs;
    struct DSP_UUID        uuid;
    DSP_STATUS      status = DSP_SOK;
    UINT            index = 100;
    CHAR            *pszUuid;
    struct DSP_MSG         testMsg;
    UINT            timeout;
    FILE            *pHandle = copyTask->pHandle;

    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    pindex=0;
    INT                     procId;
    struct DSP_NOTIFICATION    mmuFaultHandle;
    struct DSP_NOTIFICATION*   ahEvents[2];

    pszUuid = "6562DF2A_7068_42EF_8459_75584C1CDB8F";  // IVA DOF
    UUID_UuidFromString((CHAR *)pszUuid, &uuid);


    attrs.uBufsize = DEFAULTBUFSIZE;
    attrs.uNumBufs = DEFAULTNBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = DSP_FOREVER;
    attrs.lMode = copyTask->nStrmMode;

    nodeAttrIn.uTimeout = 10000;
    nodeAttrIn.iPriority = 5;
    struct DSP_PROCESSORSTATE  procState;

    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;

    /* Allocate the strmcopy node. */

        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(pindex,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
                        procId = pindex;
                        status = DSP_SOK;
                        break;
                }
                pindex++;
        }
    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) 
	{
        	status = DSPProcessor_Attach(procId, NULL,&copyTask->hProcessor);
	        if (DSP_SUCCEEDED(status)) 
		{
          	TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        	}
        	else 
		{
            	TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        	}
       }
        else
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");

    if (DSP_SUCCEEDED(status))
    {
		copyTask->nodeEventMask = DSP_MMUFAULT;
		TSTLOG_appendLog(pHandle,"Event Mask = 0x%x\n",copyTask->nodeEventMask);
		copyTask->notifications[0] = OsSpecificMemAlloc(sizeof(struct DSP_NOTIFICATION));
		/* Register for Notifs for Proc State */
		status = DSPProcessor_RegisterNotify(copyTask->hProcessor, DSP_MMUFAULT,DSP_SIGNALEVENT, &mmuFaultHandle);
        if (DSP_FAILED(status)) 
        {
		    TSTLOG_appendLog(pHandle,"DSPProcessor_RegisterNotify failed status = 0x%x \n",status);
        }
	}


    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPProcessor_Load(copyTask->hProcessor, 1,aArgv,NULL);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Load failed. Status = 0x%x\n",status);
        }
	else
	{

        status = DSPProcessor_Start(copyTask->hProcessor);
	if(DSP_FAILED(status))
	{
            TSTLOG_appendLog(pHandle, "DSPProcessor_Start failed. Status = 0x%x\n",status);

	}
/*added for powermanagement*/

	status = DSPProcessor_Ctrl(copyTask->hProcessor,PROC_PWRMGT_ENABLE,NULL);
    	if (DSP_SUCCEEDED(status))
    	{
        	TSTLOG_appendLog(pHandle, "DSPProcessor_Ctrl:PROC_PWRMGT_ENABLE succeeded.\n") ;
    	}
    	else
    	{
        	TSTLOG_appendLog(pHandle, "DSPProcessor_Ctrl:PROC_PWRMGT_ENABLE failed. status = 0x%x\n",status) ;
    	}

    	return (status);


	}

    }

    

    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Allocate(copyTask->hProcessor, &uuid, pArgs,&nodeAttrIn, &copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPNode_Allocate failed. Status = 0x%x\n",
                status);
        }
    }

	/* Initialiase the Notification object */

    /* Create the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Create(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
                TSTLOG_appendLog(pHandle, "DSPNode_Create failed. Status = 0x%x\n",
                status);
        }
    }



    /* Start the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Run(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPNode_Run failed. Status = 0x%x\n",status);
        }
    }


	/* Do some message transfers now to check for Message related Events */
	testMsg.dwCmd = RMS_USER + 8;
	testMsg.dwArg1 = 10;
	testMsg.dwArg2 = 10;
	timeout = 1000;

	status = DSPNode_PutMessage(copyTask->hNode, &testMsg, timeout);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPNode_PutMessage failed. Status = 0x%x\n",status);
        }
	else
	{
            TSTLOG_appendLog(pHandle, "DSPNode_PutMessage PASS. Status = 0x%x\n",status);

	}

	/*status = DSPNode_GetMessage(copyTask->hNode, &testMsg, timeout);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPNode_GetMessage failed. Status = 0x%x\n",status);
        }
	else
	{
            TSTLOG_appendLog(pHandle, "DSPNode_GetMessage PASS , Message we got is %d and arg2 is %d and. Status = 0x%x\n",testMsg.dwArg1,testMsg.dwArg2,status);

	}*/

	if(DSP_SUCCEEDED(status))
	{
	ahEvents[0] = &mmuFaultHandle;
	status = DSPManager_WaitForEvents(ahEvents,1, &index,1000000);
	}


	if (DSP_SUCCEEDED(status) && index == 0 ) {
            TSTLOG_appendLog(pHandle, "Received MMU Fault From IVA .\n");
 
            DSPProcessor_GetState(copyTask->hProcessor,&procState, sizeof(procState));
 
            TSTLOG_appendLog(pHandle," dwErrMask = %0x \n",procState.errInfo.dwErrMask);
            TSTLOG_appendLog(pHandle," dwVal1 = %0x \n",procState.errInfo.dwVal1);
            TSTLOG_appendLog(pHandle," dwVal2 = %0x \n",procState.errInfo.dwVal2);
            TSTLOG_appendLog(pHandle," dwVal3 = %0x \n",procState.errInfo.dwVal3);
 
        }
        else {
           TSTLOG_appendLog(pHandle, "Error in WaitForEvents status is 0x%x and index value is %d.\n",status,index);
        }
    
	if (status == DSP_ETIMEOUT)
	{
		TSTLOG_appendLog(pHandle,"DSP TIMEOUT and the status is 0x%x\n",status);
		status = DSP_SOK;
	}

	CleanupNode(copyTask);
	CleanupProcessor(copyTask);
	OsSpecificMemFree(copyTask->notifications[0]);

    return (status);
}



/*
 *  ======== CleanupProcessor ========
 *  Perform processor related cleanup.
 */
static DSP_STATUS StopIvaProcessor(TASK_CONTEXT *copyTask)
{
	DSP_STATUS      status = DSP_SOK;
	FILE           *pHandle = copyTask->pHandle;

	status = DSPProcessor_Stop(copyTask->hProcessor) ;

        if (DSP_SUCCEEDED(status))
        {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Stop:IVA succeeded.\n") ;
        }
        else
        {
             TSTLOG_appendLog(pHandle, "DSPProcessor_Stop:IVA failed.status = 0x%x\n",status) ;
        }
	return (status);

}
static DSP_STATUS CleanupProcessor(TASK_CONTEXT *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
    FILE           *pHandle = copyTask->pHandle;

    if (copyTask->hProcessor) {
/*	status = DSPProcessor_Stop(copyTask->hProcessor) ;

        if (DSP_SUCCEEDED(status))
        {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Stop:IVA succeeded.\n") ;
        }
        else
        {
             TSTLOG_appendLog(pHandle, "DSPProcessor_Stop:IVA failed.status = 0x%x\n",status) ;
        }*/

        /* Detach from processor. */
        status = DSPProcessor_Detach(copyTask->hProcessor);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Detach failed.\n");
        }

        copyTask->hProcessor = NULL;
    }

    return (status);
}




/*
 *  ======== CleanupNode ========
 *  Perform node related cleanup.
 */
static DSP_STATUS CleanupNode(TASK_CONTEXT *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
    DSP_STATUS      exitStatus = DSP_SOK;
    FILE           *pHandle = copyTask->pHandle;

    if (copyTask->hNode) {

        // Terminate DSP node.
        status = DSPNode_Terminate(copyTask->hNode, &exitStatus);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "IVADSPNode_Terminate succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "IVADSPNode_Terminate failed: 0x%x\n", status);
        }

        /* Delete DSP node. */
        status = DSPNode_Delete(copyTask->hNode);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "IVADSPNode_Delete succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "IVADSPNode_Delete failed: 0x%x\n", status);
        }

        copyTask->hNode = NULL;
    }

    return (status);
}

