/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    MultiThreadData.c
 *
 *   Path    $(TESTSUITEROOT)\Stress\MultiThread\MultiThreadData.c
 * 
 *   Desc:
 *           This test case tests the Data Transfer by 
 *           introducing delays at certain points during Data Transfer.
 *
 *  Usage:
 *        DelayTransfer [-v] [-l "log path/filename"] 
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *        -t: Number of Threads to be created
 *        -n: Number of Transfers for each Delay
 *        -s: Buffer Size
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   AUG 05, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>
#include <waitforevent.h>
#include "MultiThread.h"
#define DEFAULTDSPUNIT      0
#define NUMOFCOMPARGS         9	/* Compulsory args: t,n,s */
#define PASS                  0
#define FAIL                  1

#define SHORT short

#define DSPEXEC             "fault_nortdx_ext_24xx.dof55L"

static FILE					    *pHandle;
static BOOL                        fOpenLog = 1;

/* Loacal function Declarations */
static DSP_STATUS RunTask(DSP_HSTREAM hInStream, DSP_HSTREAM hOutStream, 
				INT iBufferSize, BYTE **arrbuf, BYTE **arrbuf1, INT iNumTxfrs);
static DSP_STATUS SendTestConfiguration(SHORT *pInBuf, SHORT *pOutBuf, 
				DSP_HSTREAM hOutStream, DWORD dwSendBufSize, 
				DWORD dwRecvBufSize, INT iNumTxfrs);

/* Function to exit gracefully */
static INT graceful_exit()
{
	TSTLOG_appendLog(pHandle,"FATAL ERROR: Exiting the Test Case !\n");
	return(-1);
}

/* Thread function for Multi thread data transfer test case */
/*
 * Note: Address of args is being passed from the main thread,		*
 * dont alter the contents as this will be used/needed by all the	*
 * threads created. Copy the contents to local variables and use	*
 */
static INT mmuFaultFxn(MultiThreadArgs *targs)
{

    struct DSP_STRMATTR			attrs;
    BYTE					**arrBuf, **arrBuf1;
    struct DSP_NODEATTRIN			nodeAttrin;
    DSP_HNODE				hNode = NULL;
    DSP_HSTREAM				hOutStream;
    DSP_HSTREAM				hInStream;
    CHAR					*pszUuid;
    struct DSP_UUID				gidNodeID;
    struct DSP_CBDATA				*nodeargs;
    DSP_STATUS				status;
    DSP_STATUS				exitStatus = DSP_SOK;

	/* Processor related information */
    DSP_HPROCESSOR			hProcessor;
    INT						iNumTransfers;
    INT						iBuffSize;
	/* Test case specific information */
    typedef struct
    {
        ULONG               length;
        CHAR                iErrorType;
        CHAR                iErrorPhase;
        CHAR                iErrorIteration;
        CHAR                iErrorPosition;
        CHAR                iErrorCount;                            
        
    }fault_Arguments;
	/* Logging related information */
	BOOL					bUseLog=FALSE;
	BOOL					bVerbose=FALSE;
        INT				        threadID ;
        fault_Arguments         faultArgs;

	/* Copy the contents of passed args to local vars */
	/* This strategy can be changed if the variables are just
	   used/read and not modified. (Swamy) */
	hProcessor = targs->hProcessor;
	iNumTransfers = targs->iNumTransfers;
	iBuffSize = targs->iBuffSize;
	pHandle = targs->pHandle;
	bUseLog = targs->bUseLog;
	bVerbose = targs->bVerbose;
	threadID = targs->ithrdNum ;
    faultArgs.length      = sizeof(faultArgs) - sizeof(ULONG);
    faultArgs.iErrorType  = (CHAR)targs->iErrorType;
    faultArgs.iErrorPhase = (CHAR)targs->iErrorPhase;
    faultArgs.iErrorCount = (CHAR)targs->iErrorCount;
    faultArgs.iErrorPosition = (CHAR)targs->iErrorPosition;
    faultArgs.iErrorIteration = (CHAR)targs->iErrorIteration;
    
    /* Initialising attributes */
    attrs.uSegid = 0;
    attrs.uBufsize = iBuffSize;
    attrs.lMode = STRMMODE_PROCCOPY;
    attrs.uNumBufs = 1;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;

    nodeAttrin.iPriority = 5 ;
    nodeAttrin.uTimeout = 10000 ;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif

    nodeargs = (struct DSP_CBDATA *)&faultArgs;
    arrBuf = OsSpecificMemAlloc(sizeof(BYTE *));
    arrBuf1 = OsSpecificMemAlloc(sizeof(BYTE *));

    pszUuid = "F95CA073_EE7D_11D5_A8DD_00B0D055F6D1";    // taken from fault_nortdx_int.cdb file
	UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

        status = DSPNode_Allocate(hProcessor, &gidNodeID, nodeargs, &nodeAttrin,&hNode);
    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Allocate(). status = 0x%x\n", status);
        }
        return FAIL;
    }

    status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,&attrs);

    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect(). status = 0x%x\n", status);
        }
        return FAIL;
    }

    status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                             &attrs);

    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect. status = 0x%x\n", status);
        }
        return FAIL;
    }


    /* Create Node on DSP */
    status = DSPNode_Create(hNode);

    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Create. status = 0x%x\n", status);
        }
        return FAIL;
    }

    status = DSPNode_Run(hNode);

    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Run()\n");
        }
        return FAIL;
    }

    status = DSPStream_Open(hNode, DSP_TONODE, 0, NULL, &hOutStream);

    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPStraem_Open. status = 0x%x\n", status);
        }
        return FAIL;
    }

    status = DSPStream_Open(hNode, DSP_FROMNODE, 0, NULL, &hInStream);

    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPStraem_Open. status = 0x%x\n", status);
        }
        return FAIL;
    }

    status = DSPStream_AllocateBuffers(hInStream, 64, arrBuf, 1);
    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers. status = 0x%x\n", status);
        }
        return FAIL;
    }

    status = DSPStream_AllocateBuffers(hOutStream, 64, arrBuf1, 1);
    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers. status = 0x%x\n", status);
        }
        return FAIL;
    }

    status = RunTask(hInStream, hOutStream, iBuffSize, arrBuf, arrBuf1, iNumTransfers);
    status = DSPStream_Close(hOutStream);
    status = DSPStream_Close(hInStream);
    status = DSPNode_Terminate(hNode, &exitStatus);
    status = DSPNode_Delete(hNode);

    OsSpecificMemFree(arrBuf);

    /* Return from this Thread */
	return status;
    
}


static DSP_STATUS RunTask(DSP_HSTREAM hInStream, DSP_HSTREAM hOutStream, INT iBufferSize, BYTE **arrbuf, BYTE **arrbuf1, INT iNumTxfrs)

{
    DSP_STATUS      status = DSP_SOK;
    BYTE            *pInBuf = NULL;
    BYTE            *pOutBuf = NULL;
    DWORD           dwArg = 0;
    DWORD           dwBufSize = iBufferSize;
    DWORD           dummy;
    UINT	    k;

    pInBuf = arrbuf[0];
    pOutBuf = arrbuf1[0];
    status = SendTestConfiguration((SHORT *)pInBuf, (SHORT *)pOutBuf, 
				    hOutStream, dwBufSize, dwBufSize,iNumTxfrs);
    for( k = 0; k < iNumTxfrs; k++)
    {
		status = DSPStream_Issue(hOutStream, pOutBuf, dwBufSize, dwBufSize, 0);
		if (DSP_FAILED(status)) 
		{
                TSTLOG_appendLog(pHandle, "DSPStream_Issue Failed.status = 0x%x\n", status);
            	break;
		}

		// Wait for output completion to reclaim a queued buffer:
		status = DSPStream_Reclaim(hOutStream, &pOutBuf, &dummy, NULL, &dwArg);

		 if (DSP_FAILED(status)) {
				TSTLOG_appendLog(pHandle, "DSPStream_Reclaim Output Buffer Failed.status = 0x%x\n", status);
             	break;
		 }

        	status = DSPStream_Issue(hInStream, pInBuf, dwBufSize, dwBufSize, 0);
		if (DSP_FAILED(status)) 
		{
			if(fOpenLog)
                TSTLOG_appendLog(pHandle, "DSPStream_Issue Failed.status = 0x%x\n", status);
            	break;
		}

		// Wait for output completion to reclaim a queued buffer:
		status = DSPStream_Reclaim(hInStream, &pInBuf, &dummy, NULL, &dwArg);

		 if (DSP_FAILED(status)) {
			 if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPStream_Reclaim Input Buffer Failed.status = 0x%x\n", status);
             break;
		 }
      
    } // end of for( k = 0; k < NodeArgs.iNumTxfrs; k++)       

/*    if(fOpenLog)
    {
        TSTLOG_appendLog(pHandle, "LOG: BufferSize = %d\n", dwBufSize);
        TSTLOG_appendLog(pHandle, "LOG: FINAL STATUS = 0x%x\n", status);
    }
*/
    return (status);
}


/*
 *  ======== main ========
 *
 */
INT mmuFaultTest(INT argc, CHAR * argv[])
{
    	BOOL                    bUseLog=FALSE;
    	BOOL                    bVerbose=FALSE;
    	FILE                    *pHandle = NULL;
    	INT                     iNumThreads = 0;
    	INT                     iNumTransfers = 0;
    	INT                     iBuffSize = 0;
    	INT                     iErrorCount = 0;
    	INT                     iErrorType = 0;
    	INT                     iErrorPhase = 0;
    	INT                     iErrorIteration = 0;
    	INT                     iErrorPosition = 0;
    	INT                     iTaskIteration = 0;

	CHAR                    szLogName[DEFNAMESIZE];
	INT                     iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS              status;
	MultiThreadArgs         thrdArgs;

	DSP_HPROCESSOR			hProcessor;
     	CHAR			*aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
    	CHAR			*aEnvp[DEFNAMESIZE] = {NULL};
	INT			iArgc = 1;
	INT			opt;
	INT			index;
	BOOL			fLoadDsp = FALSE;
	struct DSP_PROCESSORINFO       dspInfo;
  	UINT                    numProcs;
	UINT                    pindex=0;
   	INT                     procId;

	 TASK_CONTEXT   contextTask;
	 struct DSP_NOTIFICATION    mmuFaultHandle;
	 struct DSP_NOTIFICATION*   ahEvents[2];
	     
	 struct DSP_PROCESSORSTATE  procState;


	 contextTask.notifications = (struct DSP_NOTIFICATION* *)OsSpecificMemAlloc(NUMNOTIFOBJECTS * sizeof(struct DSP_NOTIFICATION*));
	 contextTask.strmnotifications = (struct DSP_NOTIFICATION* *)OsSpecificMemAlloc(3 * sizeof(struct DSP_NOTIFICATION*));
			     



	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt_test(argc, (char *const)argv, "+vl:x:t:n:s:i:a:b:c:d:e:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            pHandle = TSTLOG_openLog(szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
	case 't':
	    /* Number of Threads to be created */
	    iNumThreads = CSL_Atoi(optarg);
	    iMandArgCount--;
	    break;
	case 'n':
	    /* Number of Transfers */
	    iNumTransfers = CSL_Atoi(optarg);
	    iMandArgCount--;
	    break;
	case 's':
	   /* Buffer Size */
	   iBuffSize = CSL_Atoi(optarg);
	   iMandArgCount--;
	   break;
        case 'i':
			/* Buffer Size */
			iTaskIteration = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        case 'a':
			/* Error Count */
			iErrorCount = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        case 'b':
			/*Error Type */
			iErrorType = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        case 'c':
			/* Error Phase */
			iErrorPhase = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        case 'd':
			/* Error Iteration */
			iErrorIteration = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        case 'e':
			/* Error Position */
			iErrorPosition = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);

            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt_test(argc, (char *const)argv, "+vl:x:t:n:s:i:a:b:c:d:e:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
	}
	

	while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(pindex,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == DSPTYPE_55||dspInfo.uProcessorType == DSPTYPE_64)
                {
                        TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                        procId = pindex;
                        status = DSP_SOK;
                        break;
                }
                index++;
        }
    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, NULL,&hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
        else
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");


	

	if (DSP_SUCCEEDED(status))
	 {
	   contextTask.nodeEventMask = DSP_MMUFAULT;
	   contextTask.hProcessor = hProcessor;
	   TSTLOG_appendLog(pHandle,"Event Mask = 0x%x\n",contextTask.nodeEventMask);
	   contextTask.notifications[0] = OsSpecificMemAlloc(sizeof(struct DSP_NOTIFICATION));
	   /* Register for Notifs for Proc State */
	   status = DSPProcessor_RegisterNotify(contextTask.hProcessor, DSP_MMUFAULT,DSP_SIGNALEVENT, &mmuFaultHandle);
	   if (DSP_FAILED(status))
	   {
	    TSTLOG_appendLog(pHandle,"DSPProcessor_RegisterNotify failed status = 0x%x \n",status);
	   }
	}
    
	
	if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "Error in DSPProcessor_Attach. status = 0x%x\n",status);
        }
        graceful_exit();
    }

    if(fLoadDsp)
    {
    /* Load default program */
    status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, (CONST CHAR **)aEnvp);
    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "Error in DSPProcessor_Load. status = 0x%x\n",status);
        }
		graceful_exit();
    }

    status = DSPProcessor_Start(hProcessor);
    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,  "Error in DSPProcessor_Start. status = 0x%x\n",status);
        }
        graceful_exit();
	    }
    }

	/* Fill the thrdArgs which are to be passed on to the Thread */
	thrdArgs.hProcessor = hProcessor;
	thrdArgs.iNumTransfers = iNumTransfers;
	thrdArgs.iBuffSize = iBuffSize;
	thrdArgs.pHandle = pHandle;
	thrdArgs.bUseLog = bUseLog;
	thrdArgs.bVerbose = bVerbose;
	thrdArgs.iErrorCount = iErrorCount;
    	thrdArgs.iErrorType = iErrorType;
    	thrdArgs.iErrorPhase = iErrorPhase;
    	thrdArgs.iErrorIteration = iErrorIteration;
    	thrdArgs.iErrorPosition = iErrorPosition;
	thrdArgs.ithrdNum = iNumThreads ;

	/* Create requested number of Threads and pass the
	   input params to these Threads */
	mmuFaultFxn(&thrdArgs);
    

	if(DSP_SUCCEEDED(status))
	        {
	           ahEvents[0] = &mmuFaultHandle;
	           status = DSPManager_WaitForEvents(ahEvents,1, &index,1000000);
	        }
	        if (DSP_SUCCEEDED(status) && index == 0 ) {
	        TSTLOG_appendLog(pHandle, "Received MMU Fault From DSP .\n");
	        DSPProcessor_GetState(contextTask.hProcessor,&procState, sizeof(procState));
	        TSTLOG_appendLog(pHandle," dwErrMask = %0x \n",procState.errInfo.dwErrMask);
	        TSTLOG_appendLog(pHandle," dwVal1 = %0x \n",procState.errInfo.dwVal1);
	        TSTLOG_appendLog(pHandle," dwVal2 = %0x \n",procState.errInfo.dwVal2);
	        TSTLOG_appendLog(pHandle," dwVal3 = %0x \n",procState.errInfo.dwVal3);
	        }
	        else {
	              TSTLOG_appendLog(pHandle, "Error in WaitForEvents status is 0x%x\n",status);
	        }
	        if (status == DSP_ETIMEOUT)
	        {
		        TSTLOG_appendLog(pHandle,"DSP TIMEOUT and the status is 0x%x\n",status);
	              status = DSP_SOK;
	        }
	                                                                                                                                                                                                           
    status=DSPProcessor_Detach(hProcessor);
    if (DSP_SUCCEEDED(status)) 
	{
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
     }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }
return 0;
}



static DSP_STATUS SendTestConfiguration(SHORT *pInBuf, 	SHORT *pOutBuf, DSP_HSTREAM hOutStream, DWORD dwSendBufSize, DWORD dwRecvBufSize, INT iNumTxfrs)
{

	DSP_STATUS status = DSP_SOK;
	UINT			k;
	DWORD			dwArg;
	pInBuf[0] = dwSendBufSize / 2;
	pInBuf[1] = dwRecvBufSize / 2;
	pInBuf[2] = iNumTxfrs & 0xffff ;
	pInBuf[3] = iNumTxfrs >> 16 ;

	for ( k = 4; k < dwSendBufSize / 2; k++)
	{
			pInBuf[k] = k;
	}

	status = DSPStream_Issue(hOutStream, (BYTE *)pInBuf, dwSendBufSize, dwSendBufSize, 0);
    if (DSP_FAILED(status)) 
	{
			TSTLOG_appendLog(pHandle, "DSPStream_Issue %d failed.\n", k);
        }

	status = DSPStream_Reclaim(hOutStream, 
					(BYTE **)&pInBuf, &dwRecvBufSize, NULL, &dwArg);
    if (DSP_FAILED(status)) 
	{
			TSTLOG_appendLog(pHandle, "DSPStream_Reclaim %d failed.\n", k);
        }


	return status;
}

