/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    MultiThreadData.c
 *
 *   Path    $(TESTSUITEROOT)\Stress\MultiThread\MultiThreadData.c
 * 
 *   Desc:
 *           This test case tests the Data Transfer by 
 *           introducing delays at certain points during Data Transfer.
 *
 *  Usage:
 *        DelayTransfer [-v] [-l "log path/filename"] 
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *        -t: Number of Threads to be created
 *        -n: Number of Transfers for each Delay
 *        -s: Buffer Size
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   AUG 05, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>

#include "MultiThread.h"
#define DEFAULTDSPUNIT      0
#define DEFNAMESIZE         100
#define NUMOFCOMPARGS         9	/* Compulsory args: t,n,s */
#define MAXNUMOFTHREADS       4    /* This number taken from Symbian Test Suite */
#define PASS                  0
#define FAIL                  1

#define SHORT short

#define DSPEXEC             "fault_nortdx_ext_24xx.dof55L"

FILE					    *pHandle;
BOOL                        fOpenLog = 1;

/* Loacal function Declarations */
static DSP_STATUS RunTask(DSP_HSTREAM hInStream, DSP_HSTREAM hOutStream, 
				INT iBufferSize, BYTE **arrbuf, BYTE **arrbuf1, INT iNumTxfrs);
static DSP_STATUS SendTestConfiguration(SHORT *pInBuf, SHORT *pOutBuf, 
				DSP_HSTREAM hOutStream, DWORD dwSendBufSize, 
				DWORD dwRecvBufSize, INT iNumTxfrs);

/* Function to exit gracefully */
static INT graceful_exit()
{
	/*printf("FATAL ERROR: Exiting the Test Case !\n");
	exit(0);*/
}

/* Thread function for Multi thread data transfer test case */
/*
 * Note: Address of args is being passed from the main thread,		*
 * dont alter the contents as this will be used/needed by all the	*
 * threads created. Copy the contents to local variables and use	*
 */
static INT threadFxn(MultiThreadArgs *targs)
{

    struct DSP_STRMATTR			attrs;
	BYTE					**arrBuf, **arrBuf1;
    struct DSP_NODEATTRIN			nodeAttrin;
    DSP_HNODE				hNode = NULL;
	DSP_HSTREAM				hOutStream;
    DSP_HSTREAM				hInStream;
	CHAR					*pszUuid;
	struct DSP_UUID				gidNodeID;
    struct DSP_CBDATA				*nodeargs;
	DSP_STATUS				status;
	DSP_STATUS				exitStatus = DSP_SOK;

	/* Processor related information */
	DSP_HPROCESSOR			hProcessor;
    INT						iNumTransfers;
    INT						iBuffSize;
	/* Test case specific information */
    typedef struct
    {
        ULONG               length;
        CHAR                iErrorType;
        CHAR                iErrorPhase;
        CHAR                iErrorIteration;
        CHAR                iErrorPosition;
        CHAR                iErrorCount;                            
        
    }fault_Arguments;
	/* Logging related information */
	BOOL					bUseLog=FALSE;
	BOOL					bVerbose=FALSE;
    INT				        threadID ;
    fault_Arguments         faultArgs;

	/* Copy the contents of passed args to local vars */
	/* This strategy can be changed if the variables are just
	   used/read and not modified. (Swamy) */
	hProcessor = targs->hProcessor;
	iNumTransfers = targs->iNumTransfers;
	iBuffSize = targs->iBuffSize;
	pHandle = targs->pHandle;
	bUseLog = targs->bUseLog;
	bVerbose = targs->bVerbose;
	threadID = targs->ithrdNum ;
    faultArgs.length      = sizeof(faultArgs) - sizeof(ULONG);
    faultArgs.iErrorType  = (CHAR)targs->iErrorType;
    faultArgs.iErrorPhase = (CHAR)targs->iErrorPhase;
    faultArgs.iErrorCount = (CHAR)targs->iErrorCount;
    faultArgs.iErrorPosition = (CHAR)targs->iErrorPosition;
    faultArgs.iErrorIteration = (CHAR)targs->iErrorIteration;
    
    /* Initialising attributes */
    attrs.uSegid = 0;
    attrs.uBufsize = iBuffSize;
    attrs.lMode = STRMMODE_PROCCOPY;
    attrs.uNumBufs = 1;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;

    nodeAttrin.iPriority = 5 ;
    nodeAttrin.uTimeout = 10000 ;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif

    nodeargs = (struct DSP_CBDATA *)&faultArgs;
    arrBuf = OsSpecificMemAlloc(sizeof(BYTE *));
    arrBuf1 = OsSpecificMemAlloc(sizeof(BYTE *));

    pszUuid = "F95CA073_EE7D_11D5_A8DD_00B0D055F6D1";    // taken from fault_nortdx_int.cdb file
	UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

    if(targs->ithrdNum ==  2)
        status = DSPNode_Allocate(hProcessor, &gidNodeID, nodeargs, &nodeAttrin,
                       &hNode);
    else
        status = DSPNode_Allocate(hProcessor, &gidNodeID, NULL, &nodeAttrin,
                       &hNode);
    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Allocate(). status = 0x%x\n", status);
        }
        return FAIL;
    }

    status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
                             &attrs);

    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect(). status = 0x%x\n", status);
        }
        return FAIL;
    }

    status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                             &attrs);

    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect. status = 0x%x\n", status);
        }
        return FAIL;
    }


    /* Create Node on DSP */
    status = DSPNode_Create(hNode);

    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Create. status = 0x%x\n", status);
        }
        return FAIL;
    }

    status = DSPNode_Run(hNode);

    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Run()\n");
        }
        return FAIL;
    }

    status = DSPStream_Open(hNode, DSP_TONODE, 0, NULL, &hOutStream);

    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPStraem_Open. status = 0x%x\n", status);
        }
        return FAIL;
    }

    status = DSPStream_Open(hNode, DSP_FROMNODE, 0, NULL, &hInStream);

    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPStraem_Open. status = 0x%x\n", status);
        }
        return FAIL;
    }

    status = DSPStream_AllocateBuffers(hInStream, 64, arrBuf, 1);
    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers. status = 0x%x\n", status);
        }
        return FAIL;
    }

    status = DSPStream_AllocateBuffers(hOutStream, 64, arrBuf1, 1);
    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers. status = 0x%x\n", status);
        }
        return FAIL;
    }

    status = RunTask(hInStream, hOutStream, iBuffSize, arrBuf, arrBuf1, iNumTransfers);
    status = DSPStream_Close(hOutStream);
    status = DSPStream_Close(hInStream);
    status = DSPNode_Terminate(hNode, &exitStatus);
    status = DSPNode_Delete(hNode);

    OsSpecificMemFree(arrBuf);

    /* Return from this Thread */
	return status;
    
}


static DSP_STATUS RunTask(DSP_HSTREAM hInStream, DSP_HSTREAM hOutStream, INT iBufferSize, BYTE **arrbuf, BYTE **arrbuf1, INT iNumTxfrs)

{
    DSP_STATUS      status = DSP_SOK;
    BYTE            *pInBuf = NULL;
    BYTE            *pOutBuf = NULL;
    DWORD           dwArg = 0;
    DWORD           dwBufSize = iBufferSize;
    DWORD           dummy;
    UINT			k;

    pInBuf = arrbuf[0];
    pOutBuf = arrbuf1[0];
    status = SendTestConfiguration((SHORT *)pInBuf, (SHORT *)pOutBuf, 
				    hOutStream, dwBufSize, dwBufSize,iNumTxfrs);
    /*printf("Please wait....\n");*/
    for( k = 0; k < iNumTxfrs; k++)
    {
		status = DSPStream_Issue(hOutStream, pOutBuf, 
		dwBufSize, dwBufSize, 0);
		if (DSP_FAILED(status)) 
		{
                TSTLOG_appendLog(pHandle, "DSPStream_Issue Failed.status = 0x%x\n", status);
            break;
		}

		// Wait for output completion to reclaim a queued buffer:
		status = DSPStream_Reclaim(hOutStream, 
			&pOutBuf, &dummy, NULL, &dwArg);

		 if (DSP_FAILED(status)) {
				TSTLOG_appendLog(pHandle, "DSPStream_Reclaim Output Buffer Failed.status = 0x%x\n", status);
             break;
		 }

        status = DSPStream_Issue(hInStream, pInBuf, 
		dwBufSize, dwBufSize, 0);
		if (DSP_FAILED(status)) 
		{
			if(fOpenLog)
                TSTLOG_appendLog(pHandle, "DSPStream_Issue Failed.status = 0x%x\n", status);
            break;
		}

		// Wait for output completion to reclaim a queued buffer:
		status = DSPStream_Reclaim(hInStream, 
			&pInBuf, &dummy, NULL, &dwArg);

		 if (DSP_FAILED(status)) {
			 if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPStream_Reclaim Input Buffer Failed.status = 0x%x\n", status);
             break;
		 }
      
    } // end of for( k = 0; k < NodeArgs.iNumTxfrs; k++)       

    if(fOpenLog)
    {
        TSTLOG_appendLog(pHandle, "LOG: BufferSize = %d\n", dwBufSize);
        TSTLOG_appendLog(pHandle, "LOG: FINAL STATUS = 0x%x\n", status);
    }
    return (status);
}


/*
 *  ======== main ========
 *
 */
INT mtFaultTest(INT argc, CHAR * argv[])
{
    BOOL                    bUseLog=FALSE;
    BOOL                    bVerbose=FALSE;
    FILE                    *pHandle = NULL;
	INT                     iNumThreads = 0;
    INT                     iNumTransfers = 0;
    INT                     iBuffSize = 0;
    INT                     iErrorCount = 0;
    INT                     iErrorType = 0;
    INT                     iErrorPhase = 0;
    INT                     iErrorIteration = 0;
    INT                     iErrorPosition = 0;
    INT                     iTaskIteration = 0;

	CHAR                    szLogName[DEFNAMESIZE];
	INT                     iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS              status;
	DWORD		            threadID[MAXNUMOFTHREADS];
	HANDLE		            thandle[MAXNUMOFTHREADS];
	MultiThreadArgs         thrdArgs;

	DSP_HPROCESSOR			hProcessor;
    CHAR					*aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
    CHAR					*aEnvp[DEFNAMESIZE] = {NULL};
	INT						iArgc = 1;
	INT					    opt;
	INT					    count;
	INT					    index;
    INT                     retVal = PASS;
	DWORD		            threadStatus  = 0;
	BOOL					fLoadDsp = FALSE;


	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Load the Image on DSP */
	/* Create requested number of Threads and pass the
	   input params to these Threads */
	/* In each of the Thread do the following: */
		/* Allocate Buffers for Transfer of Data */
		/* Set the Properties, Allocate and Create the Node on DSP */
		/* Open the Streams */
		/* Run the task */
		/* Do the Data Transfer */
	/* End !! */

	for (count=0; count < argc; count++)
	{
		/*printf("mtFaultTest: arg %d : %s\n",count,argv[count]);*/
	}


	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt_test(argc, (char *const)argv, "+vl:x:t:n:s:i:a:b:c:d:e:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            pHandle = TSTLOG_openLog(szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
		case 't':
			/* Number of Threads to be created */
			iNumThreads = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
		case 'n':
			/* Number of Transfers */
			iNumTransfers = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
		case 's':
			/* Buffer Size */
			iBuffSize = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        case 'i':
			/* Buffer Size */
			iTaskIteration = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        case 'a':
			/* Buffer Size */
			iErrorCount = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        case 'b':
			/* Buffer Size */
			iErrorType = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        case 'c':
			/* Buffer Size */
			iErrorPhase = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        case 'd':
			/* Buffer Size */
			iErrorIteration = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        case 'e':
			/* Buffer Size */
			iErrorPosition = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);

            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt_test(argc, (char *const)argv, "+vl:x:t:n:s:i:a:b:c:d:e:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
	}
    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    nindex=0;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(nindex,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = nindex;
            status = DSP_SOK;
            break;
        }
        nindex++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, NULL, &hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "Error in DSPProcessor_Attach. status = 0x%x\n",status);
        }
        graceful_exit();
    }

    if(fLoadDsp)
    {
    /* Load default program */
    status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, 
					(CONST CHAR **)aEnvp);
    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "Error in DSPProcessor_Load. status = 0x%x\n",status);
        }
		graceful_exit();
    }

    status = DSPProcessor_Start(hProcessor);
    if (DSP_FAILED(status)) 
	{
        /* Report Error */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,  "Error in DSPProcessor_Start. status = 0x%x\n",status);
        }
        graceful_exit();
	    }
    }

	/* Fill the thrdArgs which are to be passed on to the Thread */
	thrdArgs.hProcessor = hProcessor;
	thrdArgs.iNumTransfers = iNumTransfers;
	thrdArgs.iBuffSize = iBuffSize;
	thrdArgs.pHandle = pHandle;
	thrdArgs.bUseLog = bUseLog;
	thrdArgs.bVerbose = bVerbose;
    thrdArgs.iErrorCount = iErrorCount;
    thrdArgs.iErrorType = iErrorType;
    thrdArgs.iErrorPhase = iErrorPhase;
    thrdArgs.iErrorIteration = iErrorIteration;
    thrdArgs.iErrorPosition = iErrorPosition;

	/* Create requested number of Threads and pass the
	   input params to these Threads */
    
	for (index = 0; index < iNumThreads; index ++)
	{
		thrdArgs.ithrdNum = index ;
		thandle[index] = (HANDLE)
			OsSpecificThreadCreate(&threadFxn, &thrdArgs, (void *)&threadID[index]);
		if (thandle[index] == NULL)
		{
			// ERROR: Could not create Thread ! Exit !! 
		}
	}

	for (index = 0; index < iNumThreads; index ++)
	{
		status = OsSpecificWaitForThread((void *)threadID[index],(int)threadStatus);
		
		if(status)
		{
			TSTLOG_appendLog(pHandle, "ERROR: Thread %d Failed\n",index);
			retVal = FAIL;
		}
    }

    DSPProcessor_Detach(hProcessor);
    if (retVal == PASS) 
	{
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
     }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }
return 0;
}



static DSP_STATUS SendTestConfiguration(SHORT *pInBuf, 
										SHORT *pOutBuf, 
										DSP_HSTREAM hOutStream, 
										DWORD dwSendBufSize, 
										DWORD dwRecvBufSize, 
                                        INT iNumTxfrs)
{

	DSP_STATUS status = DSP_SOK;
	UINT			k;
	DWORD			dwArg;
	pInBuf[0] = dwSendBufSize / 2;
	pInBuf[1] = dwRecvBufSize / 2;
	pInBuf[2] = iNumTxfrs & 0xffff ;
	pInBuf[3] = iNumTxfrs >> 16 ;

	for ( k = 4; k < dwSendBufSize / 2; k++)
	{
			pInBuf[k] = k;
	}

	status = DSPStream_Issue(hOutStream, (BYTE *)pInBuf, 
    dwSendBufSize, dwSendBufSize, 0);
    if (DSP_FAILED(status)) 
	{
			TSTLOG_appendLog(pHandle, "DSPStream_Issue %d failed.\n", k);
    }

	status = DSPStream_Reclaim(hOutStream, 
					(BYTE **)&pInBuf, &dwRecvBufSize, NULL, &dwArg);
    if (DSP_FAILED(status)) 
	{
			TSTLOG_appendLog(pHandle, "DSPStream_Reclaim %d failed.\n", k);
    }


	return status;
}

