/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    Flush.c
 *
 *   Path    $(TESTSUITEROOT)\Behaviour\Node
 * 
 *   Desc:   
 *           Tests the Flush functionality on the DSP. 
 *           Uses STRM_Idle call on DSP.
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   AUG 27, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */



#include <stdio.h> 
#include <string.h>
#include <testosal.h>
#include <getopt_test.h>
#include <dbapi.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include "Flush.h"
#include <DSPProcessor_OEM.h>

static NODEARGS		NodeArgs;

                          
/* 
 *  ======== flush_BEHAVIOUR ========
 */
int flush_BEHAVIOUR(int argc, char ** argv)
{
    STRMCOPY_TASK   strmcopyTask;
    DSP_STATUS      status = DSP_SOK;
    
    /* Process command line arguments, open data files: */
    status = ProcessArgs(argc, argv);
   
    strmcopyTask.hProcessor = NULL;
    strmcopyTask.hNode = NULL;
    strmcopyTask.hOutStream = NULL;
    strmcopyTask.hInStream = NULL;

    strmcopyTask.ppInBufs = NULL;
    strmcopyTask.ppOutBufs = NULL;

    if (DSP_SUCCEEDED(status)) 
    {
			status = InitializeProcessor(&strmcopyTask);
    }

    if (DSP_SUCCEEDED(status)) 
    {

        /* Perform node level initialization. */
        status = InitializeNode(&strmcopyTask);
        if (DSP_SUCCEEDED(status)) 
        {
            
            /* Perform stream level initialization. */
            status = InitializeStreams(&strmcopyTask);
            if (DSP_SUCCEEDED(status)) 
            {

                /* Run task. */
                status = RunTask(&strmcopyTask);
                if (DSP_FAILED(status)) 
                {
                    TSTLOG_appendLog(pHandleFlush, "RunTask failed.\n");
                }
            }
        }
    }
    
    /* Close opened files. */
    CleanupStreams(&strmcopyTask);
    CleanupNode(&strmcopyTask);
    CleanupProcessor(&strmcopyTask);
    
    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== InitializeProcessor ========
 *  Perform processor related initialization.
 */
static DSP_STATUS InitializeProcessor(STRMCOPY_TASK *copyTask)
{
    DSP_STATUS  status = DSP_SOK;
    INT			iArgc = 1;

    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandleFlush,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, NULL, &copyTask->hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandleFlush, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandleFlush, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandleFlush, "Failed to get the desired processor \n");
    }
    
	if (DSP_SUCCEEDED(status)) 
	{
	    if(fLoadDsp)
	    {
		status = DSPProcessor_Load(copyTask->hProcessor, iArgc, (CONST CHAR **)aArgvFlush, 
						(CONST CHAR **)aEnvpFlush);  
		if (DSP_SUCCEEDED(status)) 
		{
				/* Start Processor */
				status = DSPProcessor_Start(copyTask->hProcessor);
				if (DSP_FAILED(status))  
				{
					TSTLOG_appendLog(pHandleFlush, "Error in DSPProcessor Start. Status = 0x%x\n", 
                    status);   
				}
		}
		else
		{
				TSTLOG_appendLog(pHandleFlush,"Error in DSPProcessor_Load. Status = 0x%x\n", status);
		}
	}
	}
	return (status);
}


/*
 *  ======== InitializeNode ========
 *  Perform node related initialization.
 */
static DSP_STATUS InitializeNode(STRMCOPY_TASK *copyTask)
{
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA     *pArgs;
    struct DSP_NODEATTRIN  nodeAttrIn;
    struct DSP_STRMATTR    attrs;
    struct DSP_UUID        uuid;
    DSP_STATUS      status = DSP_SOK;
	CHAR			*pszUuid;

	pszUuid = "F95CA074_EE7D_11D5_A8DD_00B0D055F6D1";
	UUID_UuidFromString((CHAR *)pszUuid, &uuid);


    attrs.uSegid = 0;
    attrs.uBufsize = NodeArgs.iBufferSize;
    attrs.uNumBufs = DEFAULTNBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = DSP_FOREVER;
	switch(NodeArgs.iMode)
	{
		case '0' : 
			attrs.lMode = STRMMODE_PROCCOPY; 
			break;
		case '1' : 
			attrs.lMode = STRMMODE_ZEROCOPY; 
			break;
		case '2' : 
			attrs.lMode = STRMMODE_RDMA; 
			break;
		default:
			attrs.lMode = STRMMODE_PROCCOPY; 
			break;
	}
    
    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;
    nodeAttrIn.uTimeout = 10000;
    nodeAttrIn.iPriority = 5;

#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrIn.uProfileID = 0xff;
#endif
    /* Allocate the strmcopy node. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Allocate(copyTask->hProcessor, &uuid, NULL,
                &nodeAttrIn, &copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleFlush, "DSPNode_Allocate failed. Status = 0x%x\n",
                status);
        }
    }
    
    /* Connect the strmcopy node to the GPP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, copyTask->hNode,
                0, &attrs);
        if (DSP_FAILED(status)) {
            TSTLOG_appendLog(pHandleFlush, "DSPNode_Connect failed. Status = 0x%x\n",
                    status);
        }
    }

    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Connect(copyTask->hNode, 0, (DSP_HNODE)DSP_HGPPNODE,
            0, &attrs);
        if (DSP_FAILED(status)) {
            TSTLOG_appendLog(pHandleFlush, "DSPNode_Connect failed. Status = 0x%x\n",
                    status);
        }
    }

    /* Create the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Create(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleFlush, "DSPNode_Create failed. Status = 0x%x\n",
                status);
        }
    }
    
    /* Start the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Run(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleFlush, "DSPNode_Run failed. Status = 0x%x\n",
                status);
        }
    }
    
    return (status);
}


/*
 *  ======== InitializeStreams ========
 *  Perform stream related initialization.
 */
static DSP_STATUS InitializeStreams(STRMCOPY_TASK  *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
	struct DSP_STREAMATTRIN strmAttrs;

	strmAttrs.cbStruct = sizeof(struct DSP_STREAMATTRIN);
	strmAttrs.uTimeout = 5000;
	strmAttrs.uSegment = 0;
	strmAttrs.uAlignment = 0;
	strmAttrs.uNumBufs = 1;
	strmAttrs.lMode = (DSP_STRMMODE) STRMMODE_PROCCOPY;
	strmAttrs.uDMAChnlId = 0;
	strmAttrs.uDMAPriority = 0;

    /* Open an input data stream from the host to the strmcopy node. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPStream_Open(copyTask->hNode, DSP_TONODE, 0, 
            &strmAttrs, &copyTask->hOutStream);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleFlush, 
                "DSPStream_Open of input stream failed. Status = 0x%x\n",
                status);
        }
    }

    /* Open an output stream from the strmcopy node to the host. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPStream_Open(copyTask->hNode, DSP_FROMNODE, 0, 
            &strmAttrs, &copyTask->hInStream);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleFlush, 
                "DSPStream_Open of output stream failed. Status = 0x%x\n",
                status);
        }
    }
    
    /* Allocate and issue buffer to input data stream. */
    if (DSP_SUCCEEDED(status)) 
    {
        copyTask->ppInBufs = (BYTE**)OsSpecificMemAlloc(sizeof(BYTE*) * DEFAULTNBUFS);

        status = DSPStream_AllocateBuffers(copyTask->hOutStream, 
            NodeArgs.iBufferSize, copyTask->ppInBufs, DEFAULTNBUFS);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleFlush, 
                "DSPStream_AllocateBuffers for input stream failed. "
                "Status = 0x%x\n", status);
        }
    }
    
    /* Allocate and issue buffer to output data stream. */
    if (DSP_SUCCEEDED(status)) 
    {
        copyTask->ppOutBufs = (BYTE**)OsSpecificMemAlloc(sizeof(BYTE*) * DEFAULTNBUFS);

        status = DSPStream_AllocateBuffers(copyTask->hInStream, 
            NodeArgs.iBufferSize, copyTask->ppOutBufs, DEFAULTNBUFS);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandleFlush, 
                "DSPStream_AllocateBuffers for output stream failed. "
                "Status = 0x%x\n", status);
        }
    }
    
    return (status);
}


/*
 *  ======== RunTask ========
 *  Run strmcopy task.
 */
static DSP_STATUS RunTask(STRMCOPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
    BYTE            *pInBuf = NULL;
    BYTE            *pOutBuf = NULL;
    DWORD           dwArg = 0;
    DWORD           dwBufSize = NodeArgs.iBufferSize;
	UINT			k;
	struct DSP_STREAMINFO  streamInfo;

	
    pInBuf = copyTask->ppInBufs[0];
    pOutBuf = copyTask->ppOutBufs[0];


	/* Idle a stream which is already in Idle state. It should return okay */
	status = DSPStream_Idle(copyTask->hOutStream, TRUE);
    if (DSP_FAILED(status)) 
	{
		TSTLOG_appendLog(pHandleFlush, "ERROR: DSPStream_Idle on IDLE stream Failed.\n");
		return (status);
    }

    status = DSPStream_GetInfo(copyTask->hOutStream, &streamInfo,
            sizeof(streamInfo));
	if (streamInfo.ssStreamState != STREAM_IDLE)
	{
	    TSTLOG_appendLog(pHandleFlush, 
			"ERROR: After DSPStream_Idle - incorrect state of STREAM - 0x%x\n",streamInfo.ssStreamState);
		return (status);
	}

	
    /* 
    Read a fixed-size buffer from a data stream, send buffer to DSP, 
    and receive the same buffer back from the DSP.
    Read a block of data from the input file:
    cBytesRead = fread(pInBuf, sizeof(BYTE), DEFAULTBUFSIZE, 
    inFile);*/

    //Add I/O request to DSP.

	// Fill the buffer
	*(pInBuf) = (SHORT)(NodeArgs.iBufferSize / 2);
	pInBuf++; pInBuf++;
	*(pInBuf) = (SHORT)(NodeArgs.iNumTxfrs);
	pInBuf++; pInBuf++;
	
	for ( k =0; k < (NodeArgs.iBufferSize/2 - 2); k++)
	{
			*(pInBuf) = (SHORT)k;
			pInBuf++; pInBuf++;
	}

	/* The foll Issue Reclaim send command to the DSP side for data transfer */
	/* Buffer size and Number of transfers are sent to DSP side */
    status = DSPStream_Issue(copyTask->hOutStream, copyTask->ppInBufs[0], 
    NodeArgs.iBufferSize, NodeArgs.iBufferSize, 0);

    if (DSP_FAILED(status)) 
	{
        TSTLOG_appendLog(pHandleFlush, "DSPStream_Issue Failed.\n");
    }

    status = DSPStream_Reclaim(copyTask->hOutStream, 
        &copyTask->ppInBufs[0], &dwBufSize, NULL, &dwArg);    // pInBuf

	 if (DSP_FAILED(status)) {
         TSTLOG_appendLog(pHandleFlush, "DSPStream_Reclaim Input Buffer Failed\n");
     }


	for( k = 0; k < NodeArgs.iNumTxfrs; k++)
	{
        status = DSPStream_Issue(copyTask->hInStream, pOutBuf, 
            NodeArgs.iBufferSize, NodeArgs.iBufferSize, 0);
        
        if (DSP_FAILED(status)) 
		{
             TSTLOG_appendLog(pHandleFlush, "DSPStream_Issue OutBuffer Failed.\n");
        }
        status = DSPStream_Reclaim(copyTask->hInStream, 
            &pOutBuf, &dwBufSize, NULL, &dwArg);
        
        if (status == DSP_ETIMEOUT) 
		{
			TSTLOG_appendLog(pHandleFlush,"DSPStream_Reclaim timed out\n");
        }
        else if (DSP_FAILED(status)) 
		{
            TSTLOG_appendLog(pHandleFlush, "DSPStream_Reclaim OutBuffer Failed, 0x%x.\n", 
                status);
        }
		if((k % 500 == 0) && (status == DSP_SOK))
			 TSTLOG_appendLog(pHandleFlush, "Issue Reclaim Status at iteration %d is OK\n", k); 

	} // end of for( k = 0; k < NodeArgs.iNumTxfrs; k++)       

   	// Put a buffer into the input channel to get back buffer sent.
    status = DSPStream_Issue(copyTask->hInStream, pOutBuf, 
        NodeArgs.iBufferSize, NodeArgs.iBufferSize, 0);
    
    if (DSP_FAILED(status)) 
	{
        TSTLOG_appendLog(pHandleFlush, "DSPStream_Issue OutBuffer Failed.\n");
    }
	status = DSPStream_Idle(copyTask->hInStream, TRUE);

	if (DSP_SUCCEEDED(status))
	{
		status = DSPStream_Reclaim(copyTask->hInStream, 
			&pOutBuf, &dwBufSize, NULL, &dwArg);
		if (status == DSP_ETIMEOUT) 
		{
			TSTLOG_appendLog(pHandleFlush, "DSPStream_Reclaim timed out\n");
		}
		else if (DSP_FAILED(status)) 
		{
			TSTLOG_appendLog(pHandleFlush, "DSPStream_Reclaim Input Buffer Failed, 0x%x.\n", 
				status);
		}
	}
	status = DSPStream_Idle(copyTask->hInStream, FALSE);
	/* ---------------------------------------------------- */
	/* Idle the OUTPUT stream with Flush = TRUE so that Reclaim succeeds */
    status = DSPStream_Issue(copyTask->hOutStream, copyTask->ppInBufs[0], 
						    NodeArgs.iBufferSize, NodeArgs.iBufferSize, 0);
    if (DSP_FAILED(status)) 
	{
        TSTLOG_appendLog(pHandleFlush, "DSPStream_Issue Failed.\n");
    }
	status = DSPStream_Idle(copyTask->hOutStream, TRUE);
    status = DSPStream_Reclaim(copyTask->hOutStream, 
        &copyTask->ppInBufs[0], &dwBufSize, NULL, &dwArg);    // pInBuf
    if (!DSP_SUCCEEDED(status)) {
      TSTLOG_appendLog(pHandleFlush, "DSPStream_Reclaim Input Buffer Failed\n");
     }

    /* Idle the OUTPUT stream with Flush = FALSE so that Reclaim fails */
    status = DSPStream_Issue(copyTask->hOutStream, copyTask->ppInBufs[0], 
						    NodeArgs.iBufferSize, NodeArgs.iBufferSize, 0);

	if (DSP_FAILED(status)) 
	{
	    TSTLOG_appendLog(pHandleFlush, "DSPStream_Issue Failed.\n");
    }

	status = DSPStream_Idle(copyTask->hOutStream, FALSE);
	status = DSPStream_Reclaim(copyTask->hOutStream, 
        &copyTask->ppInBufs[0], &dwBufSize, NULL, &dwArg);    // pInBuf
    
    if (DSP_FAILED(status)) {
         TSTLOG_appendLog(pHandleFlush, "DSPStream_Reclaim Input Buffer Failed AS EXPECTED\n");
     }
	status = DSPStream_Idle(copyTask->hOutStream, TRUE);
	return (status);
}

/*
 *  ======== CleanupStreams ========
 *  Perform stream related cleanup.
 */ 
static DSP_STATUS CleanupStreams(STRMCOPY_TASK *copyTask)
{
    struct DSP_STREAMINFO  streamInfo;
    DSP_STATUS      status = DSP_SOK;
    DWORD           dwArg = 0;
    DWORD           dwBufsize = NodeArgs.iBufferSize;


    /* Reclaim and free stream buffers. */
    if (copyTask->hOutStream) {
        DSPStream_Idle(copyTask->hOutStream, TRUE);
        status = DSPStream_GetInfo(copyTask->hOutStream, &streamInfo,
            sizeof(streamInfo));

        if (copyTask->ppInBufs) {
            /* Reclaim the buffer if it's still in the stream */
            if (streamInfo.uNumberBufsInStream) {
                status = DSPStream_Reclaim(copyTask->hOutStream,
                    &copyTask->ppInBufs[0], &dwBufsize, NULL, &dwArg);
            }
            status = DSPStream_FreeBuffers(copyTask->hOutStream, 
                    copyTask->ppInBufs, DEFAULTNBUFS);
            if (DSP_FAILED(status)) {
                 TSTLOG_appendLog(pHandleFlush,  "DSPStream_FreeBuffer of input buffer failed. "
                    "Status = 0x%x\n", status);
            }

            copyTask->ppInBufs = NULL;
            /* Close open streams. */
            status = DSPStream_Close(copyTask->hOutStream);
            if (DSP_FAILED(status)) {
                 TSTLOG_appendLog(pHandleFlush, "DSPStream_Close of input stream failed. "
                    "Status = 0x%x\n", status);
            }

            copyTask->hOutStream = NULL;
        }
    }
    
    if (copyTask->hInStream) {
        DSPStream_Idle(copyTask->hInStream, TRUE);

        /* Need to determine the number of buffers left in the stream */
        status = DSPStream_GetInfo(copyTask->hInStream, &streamInfo,
            sizeof(streamInfo));

        if (copyTask->ppOutBufs) {
            if (streamInfo.uNumberBufsInStream) {
                status = DSPStream_Reclaim(copyTask->hInStream, 
                        &copyTask->ppOutBufs[0], &dwBufsize, NULL, &dwArg);
            }

            /* Free buffer. */
            status = DSPStream_FreeBuffers(copyTask->hInStream, 
                copyTask->ppOutBufs, DEFAULTNBUFS);
            if (DSP_FAILED(status)) {
                 TSTLOG_appendLog(pHandleFlush, 
                    "DSPStream_FreeBuffer of output buffer failed. "
                    "Status = 0x%x\n", status);
            }
            copyTask->ppOutBufs = NULL;
        }

        status = DSPStream_Close(copyTask->hInStream);
        if (DSP_FAILED(status)) {
             TSTLOG_appendLog(pHandleFlush, "DSPStream_Close of output stream failed . "
                "Status = 0x%x\n", status);
        }

        copyTask->hInStream = NULL;
    }
    
    return (status);
}


/*
 *  ======== CleanupNode ========
 *  Perform node related cleanup.
 */ 
static DSP_STATUS CleanupNode(STRMCOPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
    DSP_STATUS      exitStatus;
    
    if (copyTask->hNode) {
        status = DSPNode_Terminate(copyTask->hNode, &exitStatus);
        if (DSP_FAILED(status)) 
        {
             TSTLOG_appendLog(pHandleFlush, "DSPNode_Terminate failed: 0x%x\n", status);
        }

        /* Delete DSP node. */
        status = DSPNode_Delete(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
             TSTLOG_appendLog(pHandleFlush, "DSPNode_Delete failed: 0x%x\n", status);
        }

        copyTask->hNode = NULL;
    }

    return (status);
}


/*
 *  ======== CleanupProcessor ========
 *  Perform processor related cleanup.
 */ 
static DSP_STATUS CleanupProcessor(STRMCOPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;

    if (copyTask->hProcessor) {
        /* Detach from processor. */
        status = DSPProcessor_Detach(copyTask->hProcessor);
        if (DSP_FAILED(status))  
        {
             TSTLOG_appendLog(pHandleFlush, "DSPProcessor_Detach failed.\n");
        }

        copyTask->hProcessor = NULL;
    }

    return (status);
}


/* 
 *  ======== ProcessArgs ========
 *  Process command line arguments.
 */
static DSP_STATUS ProcessArgs(int argc, char ** argv)
{
    DSP_STATUS      status = DSP_SOK;
	INT             opt = 0;
	NodeArgs.iSegmentId = DEFAULTSEGMENT;
	NodeArgs.iMode  = 0;
	optind = 0;

        while ((opt = getopt_test(argc, (char *const)argv, "+vl:x:b:n:ZPR:")) != EOF) {
        switch (opt) {
         case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
			break;

	      case 'l':
            /* use log and store with name location */
            strncpy(szLogNameFlush, optarg, sizeof(szLogNameFlush));
			TST_verbose("Opening %s\n", szLogNameFlush);
            pHandleFlush = TSTLOG_openLog(szLogNameFlush);
	        break;

          case 'x':
            /* use given executable */
            aArgvFlush[0] = (char *) optarg;
			fLoadDspFlush = TRUE;
			fLoadDsp = TRUE;
			break;

		case 'b':
            /* set Buffer Size */
            NodeArgs.iBufferSize = CSL_Atoi(optarg);
			break;

		case 'n':
            /* set Number of transfers */
            NodeArgs.iNumTxfrs = CSL_Atoi(optarg);
			break;

		case 'Z':
			/* set Transfer Mode as STRMMODE_ZEROCOPY */
			NodeArgs.iMode = STRMMODE_ZEROCOPY;
			NodeArgs.iSegmentId = DSP_SHMSEG0;
		    break;
		case 'R':
			/* set Transfer Mode as STRMMODE_RDMA */
			NodeArgs.iMode = STRMMODE_RDMA;
			break;
		
        default:
            TST_verbose(("Invalid option: %c\n"), opt);
            break;
        }
    }
		return (status);
}

