/*  ============================================================================
 *   File    IVAMultiThreadAttach.c
 *
 *   Path    $(TESTSUITEROOT)\Behaviour\ProcAttach
 * 
 *   Desc:   Tests the Behaviour of the Bridge for Processor
 *           Attach and Detach calls by multiple Threads.
 *
 *  Usage:
 *        IVAMULTHRDATTACH [-v] [-l "log path/filename"] 
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -t: Number of Threads to be created
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   AUG 21, 2002    REF=ORG             		-Swamy MC
 *   JULY 22,2004    Modified to support IVA 		-RAMESH GUPTA G
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>

#include "ProcAttach.h"

#define DEFNAMESIZE      100
#define NUMOFCOMPARGS      1	/* Compulsory args: t */
#define MAXNUMOFTHREADS   20
#define PASS               0
#define FAIL               1

#define SHORT short

#define DSPEXEC        "iva.dof"


/* Function to exit gracefully */

/* Thread function for Multi thread data transfer test case */
/*
 * Note: Address of args is being passed from the main thread,		*
 * dont alter the contents as this will be used/needed by all the	*
 * threads created. Copy the contents to local variables and use	*
 */
static INT IVAthreadFxn(MultiThreadArgs *targs)
{

	DSP_STATUS			status;

	/* Processor related information */
	DSP_HPROCESSOR			hProcessor;
        struct DSP_PROCESSORATTRIN		*pAttrIn;

	/* Logging related information */
	FILE				*pHandle;
	BOOL				bUseLog=FALSE;
	BOOL				bVerbose=FALSE;

	struct DSP_PROCESSORINFO       	dspInfo;
        UINT                    	numProcs;
        UINT                    	index=0;
        INT                     	procId;


	/* Copy the contents of passed args to local vars */
	/* This strategy can be changed if the variables are just
	   used/read and not modified. (Swamy) */
	pHandle = targs->pHandle;
	bUseLog = targs->bUseLog;
	bVerbose = targs->bVerbose;
    	pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		// Error
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		return FAIL;
	}
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

	/*  while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
        }*/

    /* Attach to Processor */
    status = DSPProcessor_Attach(targs->procId, NULL, &hProcessor);
    if (DSP_FAILED(status)) 
	{
        /* Report Error */

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Attach():IVA\n");
        }
        return FAIL;
    }
    
    DSPProcessor_Detach(hProcessor);
    
	/* Return from this Thread */
	return PASS;
}



/*
 *  ======== main ========
 *
 */
INT  IVAmultiThreadAttach_BEHAVIOUR (INT argc, CHAR * argv[])
{
	BOOL                bUseLog=FALSE;
    	BOOL                bVerbose=FALSE;
    	FILE                *pHandle = NULL;
	CHAR                szLogName[DEFNAMESIZE];
	INT                 iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS          status;
	THANDLE		    threadID[MAXNUMOFTHREADS];
	INT		    thandle[MAXNUMOFTHREADS];
	MultiThreadArgs     thrdArgs;
	INT                 iNumThreads =0;
	INT		    opt;
	INT		    index;
    	INT                 retVal = PASS;
	DWORD		    threadStatus ;
	DSP_HPROCESSOR	    hProcessor;

	struct DSP_PROCESSORINFO               dspInfo;
        UINT                            numProcs;
        UINT                            pindex=0;
        INT                             procId;


	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Create requested number of Threads */
	/* In each of the Thread do the following: */
		/* Attach to the Processor */
		/* Wait for some time, eg: 500 millisecond
		 * - This is for sync purpose. This can also be 
		 *	 achieved using Semaphores. 
		 */
		/* Do a Detach */
	/* End !! */
	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt_test(argc, (char *const)argv, "+vl:t:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            pHandle = TSTLOG_openLog(szLogName);
            break;
		case 't':
			/* Number of Threads to be created */
			iNumThreads = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);

            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt_test(argc, (char *const)argv, "+vl:t:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
	}

   while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
                        procId = pindex;
                        status = DSP_SOK;
                        break;
                }
                pindex++;
        }

	/* Do an attach in main thread also .. so that we can detach in the end */
	status = DSPProcessor_Attach(procId, NULL, &hProcessor);

	if (DSP_FAILED(status))
        {
        	if (bUseLog)
                {
            	TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Attach():IVA\n");
        	}
    	}


	/* Fill the thrdArgs which are to be passed on to the Thread */
	thrdArgs.pHandle = pHandle;
	thrdArgs.bUseLog = bUseLog;
	thrdArgs.bVerbose = bVerbose;
	thrdArgs.procId = procId;
	/* Create requested number of Threads and pass the
	   input params to these Threads */
	for (index = 0; index < iNumThreads; index ++)
	{
		thandle[index] = 
			OsSpecificThreadCreate(&IVAthreadFxn, &thrdArgs, (void *)&threadID[index]);
		if (thandle[index] == 0)
		{
			TSTLOG_appendLog(pHandle, "ERROR: Could not create Thread ! Exit !!\n");
		}
	}
	// Wait for the threads to finish before doing a Detach.
	for (index = 0; index < iNumThreads; index ++)
	{
		status = OsSpecificWaitForThreadExit(threadID[index],&threadStatus);
		
		if(status)
		{
			TSTLOG_appendLog(pHandle, "ERROR: Thread %d Failed\n",index);
			retVal = FAIL;
		}
	}

    DSPProcessor_Detach(hProcessor);

    if (retVal == PASS) 
	{
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}




