/*
*  Copyright 2004 by Texas Instruments Incorporated.
*  All rights reserved. Property of Texas Instruments Incorporated.
*  Restricted rights to use, duplicate or disclose this code are
*  granted through contract.
*
*  @(#) DSP/BIOS Bridge 
*/
/*  ============================================================================
*   File    TwlCheck.c
*
*   Path    $(TESTSUITEROOT)\Behaviour\twlcheck
* 
*   Desc    Tests the TWL (Table Walking Logic) status .
*
*   Rev     1.0.0
*
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2003
*
*   ============================================================================
*   Revision History
*
*   OCT 30, 2006  REF=ORG             Ramesh Gupta Guntha
*      Original Version.
*   ============================================================================
*/

#include <stdio.h> 
#include <dbapi.h>
#include <rms_sh.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <string.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>   
#include <msgtxfr.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/mman.h>
#define IVA2_MMUBASE    0x5d000000
#define IVA2_MMUSIZE    0x100
#define PRCM_BASE 0x49006000
#define PRCM_SIZE 0x1000


volatile unsigned int regVal;

static void CheckTwl();


/* 
*  ======== TwlCheck ========
*/
int TwlCheck(int argc, char ** argv)
{
    
    COPY_TASK       copyTask;
    NODEARGS        NodeArgs;
    DSP_STATUS      status = DSP_SOK;
    INT             opt = 0;
    
    fLoadDsp = FALSE;
    copyTask.hProcessor = NULL;
    copyTask.hNode = NULL;
    copyTask.hInStream = NULL;
    
    optind = 0;
    while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:n:ZPR:")) != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;
            
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            pHandle = TSTLOG_openLog(szLogName);
            fOpenLog = TRUE;
            TSTLOG_appendLog(pHandle, "Log File Set to %s\n", szLogName);
            break;
            
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
            
        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            break;
        }
    }
   
    TSTLOG_appendLog(pHandle, "Checking TWL Status before loading the Image\n"); 
    CheckTwl();
    
    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }
    
    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, NULL, &copyTask.hProcessor);
        if(DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach Pass.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
    if (DSP_SUCCEEDED(status)) 
    {
        /* Load user specified DSP Executable and start the processor. */
        if(fLoadDsp)
            status = InitializeProcessor(&copyTask);
    }
    
    if (DSP_SUCCEEDED(status)) {
        /* Perform node level initialization. */
	 TSTLOG_appendLog(pHandle, "Checking TWL Status after loading the Image\n");
    	 CheckTwl();
	 if(regVal&0x4)
	{
	 TSTLOG_appendLog(pHandle, "PASS\n");
	  status = DSP_SOK;
	}
	else
	{
	 TSTLOG_appendLog(pHandle, "FAIL\n");
	 status = DSP_EFAIL;
	}
    }
    
    if (copyTask.hProcessor) 
    {
        DSPProcessor_Detach(copyTask.hProcessor);
        copyTask.hProcessor = NULL;
    }
    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
*  ======== InitializeProcessor ========
*  Perform processor related initialization.
*/
static DSP_STATUS InitializeProcessor(COPY_TASK *copyTask)
{
    DSP_STATUS          status = DSP_SOK;
    INT                 iArgc = 1 ;
    
    status = DSPProcessor_Load(copyTask->hProcessor, iArgc, aArgv, aEnvp);  
    if (DSP_SUCCEEDED(status)) 
    {
        if(fOpenLog)
            TSTLOG_appendLog(pHandle, "DSPProcessor_Load succeeded\n");
        status = DSPProcessor_Start(copyTask->hProcessor);
        if (DSP_FAILED(status)) 
        {
            if(fOpenLog)
                TSTLOG_appendLog(pHandle, "Error in DSPProcessor Start.Status = 0x%x\n", status);
        }
    }
    else
    {
        if(fOpenLog)
            TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Load. Status = 0x%x\n",status);
    }
    
    return (status);
}

static void CheckTwl()
{

    int fd;
    unsigned int mmu_base = 0;
    unsigned int prcm_base = 0;


    fd = open ("/dev/mem", O_RDWR|O_SYNC);
    if (fd) {

        mmu_base = (unsigned int) (char *) mmap(0,
                IVA2_MMUSIZE,
                PROT_READ | PROT_WRITE,
                MAP_SHARED,
                fd,
                (unsigned long)IVA2_MMUBASE);

        prcm_base = (unsigned int) (char *) mmap(0,
                PRCM_SIZE,
                PROT_READ | PROT_WRITE,
                MAP_SHARED,
                fd,
                (unsigned long)PRCM_BASE);

        close (fd);

    }

    else {

         TSTLOG_appendLog(pHandle,"Failed to open the mmap device \n");

    }
    if (mmu_base) 
    {

    if (prcm_base) 
	{

		*((unsigned int*)(prcm_base + 0x800)) = 0x01;

		*((unsigned int*)(prcm_base + 0x850)) = 0x00;

		regVal = *((unsigned int*)(prcm_base + 0x8e0));

		regVal &= 0x00000003;

	        if (regVal != 0x0) 
		{

			regVal = *((unsigned int*)(prcm_base + 0x8e0));

			regVal |= 0x00040000;

			*((unsigned int*)(prcm_base + 0x8E0)) = regVal;

			sleep(1);
       		}

		regVal = *((unsigned int*)(prcm_base + 0x8e0));

		regVal &= 0xFFFBFFFC;

		*((unsigned int*)(prcm_base + 0x8E0)) = regVal;

		TSTLOG_appendLog(pHandle,"Set PWSTCTRL_DSP to 0x%x \n",regVal);

		sleep(1);

	}


	TSTLOG_appendLog(pHandle,"************MMU CTRL REG STATUS**********\n");

        regVal = *((unsigned int*)(mmu_base + 0x44));

        TSTLOG_appendLog(pHandle,"DSP_MMU CTRL = 0x%x \n", regVal);

	if(regVal&0x4)
	 {

	  TSTLOG_appendLog(pHandle,"TWL is ENABLED\n");
	 }
	 else
	 {
	  TSTLOG_appendLog(pHandle,"TWL is DISABLED\n");
	 }
         munmap((char *)mmu_base, IVA2_MMUSIZE);
   }

}
