/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*
 *  ======== dbtypeosal.h ========
 *  Purpose:
 *      This header defines data types for DirectDSP APIs and device 
 *      driver modules. It also defines the Hungarian 
 *      prefix to use for each base type.
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *! Revision History:
 *! =================
 *! 13-Feb-2001 kc: Name changed from ddsptype.h dbtype.h.
 *! 09-Oct-2000 jeh Added CHARACTER.
 *! 14-Sep-2000 jeh Moved PSTRING from dspdefs.h to dbtype.h. Define
 *!                 DEF_WINCE_ so PSTRING gets defined.
 *! 11-Aug-2000 ag: Added 'typedef void VOID'. 
 *! 08-Apr-2000 ww: Cloned. 
 */

#ifndef DBTYPEOSAL_
#define DBTYPEOSAL_

#ifndef DEF_LINUX_
#define DEF_LINUX_
#endif

#include <dbtype.h>



typedef struct _PROCESS_INFORMATION { 
	HANDLE hProcess; 
	HANDLE hThread; 
	DWORD dwProcessId; 
	DWORD dwThreadId; 
} PROCESS_INFORMATION; 

typedef PROCESS_INFORMATION	*LPPROCESS_INFORMATION; /* lp */

#if defined(DEF_LINUX_)
/*------------------------------ LINUX -------------------------------------*/
typedef long int int_64; 
typedef int_64 LONGLONG;

#endif /* defined(DEF_LINUX_) */
/*------------------------------ LINUX -------------------------------------*/

typedef LARGE_INTEGER *PLARGE_INTEGER;


#if defined(DEF_WINCE_)
/*------------------------------ WINCE ---------------------------------------*/


typedef unsigned short  WCHAR   ;   /* wch  */   

#if defined(UNICODE)
typedef WCHAR           TCHAR   ;
#else
typedef CHAR            TCHAR   ;
#endif

typedef WCHAR           *PWCHAR ;
typedef TCHAR           *PSTRING;       /* Generic character string type*/
typedef TCHAR           CHARACTER;


#ifdef ERROR            /* Definition of ERROR in wingdi.h clashes with gt.h  */
#undef ERROR
#endif

typedef HANDLE  	THANDLE ;   	/* th    */
typedef HANDLE		PHANDLE;        /* ph    */     
#endif /* defined(DEF_WINCE_) */ 
/*------------------------------ WINCE ---------------------------------------*/


#if defined(DEF_EPOC_)
/*------------------------------ EPOC ----------------------------------------*/

/*------------------------------ EPOC ----------------------------------------*/
#endif /* defined(DEF_EPOC_) */


#if defined(DEF_LINUX_)
/*------------------------------ LINUX -------------------------------------*/


#ifndef RETSUCCESS
#define RETSUCCESS 1
#endif
#ifndef RETFAILURE 
#define RETFAILURE 0
#endif

typedef   PDWORD          LPDWORD;

#ifdef ERROR            /* Definition of ERROR in wingdi.h clashes with gt.h  */
#undef ERROR
#endif

#ifdef __KERNEL__
typedef void	*THANDLE;   /* th    */
typedef void	*PHANDLE;   
#else
typedef pthread_t	*THANDLE;   /* th    */
typedef pid_t		PHANDLE;   
#endif

/*------------------------------ LINUX -------------------------------------*/
#endif /* defined(DEF_LINUX_) */


/*============================================================================*/
/*  Standard calling conventions                                              */
/*============================================================================*/

#define STATIC          static
#define EXTERN          extern

#if defined(DEF_WINCE_)
/*------------------------------ WINCE ---------------------------------------*/

#ifndef CDECL
#define CDECL           _cdecl
#endif

#ifndef WINAPI
#if (_MSC_VER >= 800) || defined(_STDCALL_SUPPORTED)
#define WINAPI          __stdcall
#else
#define WINAPI
#endif
#endif

#define STDCALL         WINAPI

#define DLLIMPORT       __declspec(dllexport)
#define DLLEXPORT       __declspec(dllexport)

/*------------------------------ WINCE ---------------------------------------*/
#endif /* defined(DEF_WINCE_) */


#if defined(DEF_EPOC_)
/*------------------------------ EPOC ----------------------------------------*/

#define CDECL           __cdecl

#define STDCALL


#if defined(__VC32__)
#define DLLIMPORT       __declspec(dllexport)
#define DLLEXPORT       __declspec(dllexport)
#endif


#if defined(__GCC32__)
#define DLLIMPORT
#define DLLEXPORT       __declspec(dllexport)
#endif

/*------------------------------ EPOC ----------------------------------------*/
#endif /* defined(DEF_EPOC_) */

/*REM_SB*/
#if defined(DEF_LINUX_)
/*------------------------------ NUCLEUS -------------------------------------*/

#define CDECL           

#define WINAPI

#define STDCALL

#define DLLIMPORT
#define DLLEXPORT

/* Define DSPAPIDLL correctly in dspapi.h */
#define _DSPSYSDLL32_

/*------------------------------ NUCLEUS -------------------------------------*/
#endif /* defined(DEF_LINUX_) */


/*============================================================================*/
/*  Derived calling conventions                                               */
/*============================================================================*/

#define DSPNORMALAPI    STDCALL
#define DSPKERNELAPI    CDECL
#define DSPEXPORTAPI    DLLEXPORT

#endif /* DBTYPE_ */

