/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DelayTransfer.c
 *
 *   Path    $(TESTSUITEROOT)\Stress\Delay\DelayTransfer.c
 * 
 *   Desc:
 *           This test case tests the Data Transfer by 
 *           introducing delays at certain points during Data Transfer.
 *
 *  Usage:
 *        DelayTransfer [-v] [-l "log path/filename"] 
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *        -h: Host Delay (in Milli Seconds)
 *        -n: Number of Transfers for each Delay
 *        -s: Buffer Size
 *        -d: DSP side Initial Value of Delay (in NOPs)
 *        -i: DSP side Delay Increment (in NOPs)
 *        -e: DSP side Delay End Value
 *        -m: DSP side Delay Multiple
 *        -p: DSP side Delay Position (0..8)
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 25, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>

#define DEFNAMESIZE    100
#define DEFNUMBUFS         1
#define NUMOFCOMPARGS      8	/* Compulsory args: h,n,s,d,i,e,m,p */
#define DEFAULTDSPUNIT     0
#define SHORT short

#define DSPEXEC        "timing_nortdx_int_24xx.dof55L"


/* Function to exit gracefully  */
static INT graceful_exit(DSP_HPROCESSOR hProcessor)
{
	DSPProcessor_Detach(hProcessor);
	return(0);
}

/* Function for DSP Delay Position mapping */
void DspDelayPosition(SHORT iDspDelayPos, BYTE **aDataPos)
{
    switch (iDspDelayPos)
    {
     case 1 :
        *aDataPos[0]++ = (SHORT ) 0 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 0 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 1 ; aDataPos[0]++;
        break ;

     case 2 :
        *aDataPos[0]++ = (SHORT ) 0 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 1 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 0 ; aDataPos[0]++;
        break ;

     case 3 :
        *aDataPos[0]++ = (SHORT ) 0 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 1 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 1 ; aDataPos[0]++;
        break ;

     case 4 :
        *aDataPos[0]++ = (SHORT ) 1 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 0 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 0 ; aDataPos[0]++;
        break ;

     case 5 :
        *aDataPos[0]++ = (SHORT ) 1 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 0 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 1 ; aDataPos[0]++;
        break ;

     case 6 :
        *aDataPos[0]++ = (SHORT ) 1 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 1 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 0 ; aDataPos[0]++;
        break ;

     case 7 :
        *aDataPos[0]++ = (SHORT ) 1 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 1 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 1 ; aDataPos[0]++;
        break ;

     default:
        *aDataPos[0]++ = (SHORT ) 0 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 0 ; aDataPos[0]++;
        *aDataPos[0]++ = (SHORT ) 0 ; aDataPos[0]++;
        break ;
    }
}



/*
 *  ======== main ========
 *
 */
INT delayTransfer_STRESS(INT argc, CHAR * argv[])
{
    BOOL           bUseLogDelay=FALSE;
    FILE           *pHandle = NULL;
	DWORD          iHostDelay = 0;
    INT            iNumTransfers = 0;
    INT            iBuffSize = 0;
    SHORT          iDspStartVal = 0;
    SHORT          iDspIncrement = 0 ;
    SHORT          iDspStopVal = 0 ;
    INT	           iDspMult = 0 ;
    SHORT          iDspDelayPos = 0 ;
	CHAR           szLogName[DEFNAMESIZE];
	INT            iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS     status;
    ULONG          argTemp1;
    ULONG          argTemp2;
    DWORD          argTemp3;
	INT            i;


    struct DSP_PROCESSORATTRIN		*pAttrIn;
	DSP_HPROCESSOR			hProcessor;
    CHAR					*aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
	INT						iArgc = 1;
	CHAR					*pszUuid;
	struct DSP_UUID				gidNodeID;
    
    struct DSP_NODEATTRIN			nodeAttrin;
    DSP_HNODE				hNode;
    struct DSP_STRMATTR			attrs;
	DSP_HSTREAM				hStreamIn;
	DSP_HSTREAM				hStreamOut;
	BYTE					**arrBuf;
	BYTE					**arrBuf1;
	INT					timeValue;
	INT					opt;
	DSP_STATUS          exitStatus = DSP_SOK;
	SHORT				* pData ;
	BOOL				fLoadDsp = FALSE;

	

	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Load the Image on DSP */
	/* Allocate Buffers for Transfer of Data */
	/* Set the Properties, Allocate and Create the Node on DSP */
	/* Open the Streams */
	/* Run the task */
	/* Check for Data Transfer with different Delays */
	/* End !! */


    /* Initialising attributes */
    attrs.uSegid = 0;
    attrs.uBufsize = 512;
    attrs.lMode = STRMMODE_PROCCOPY;
    attrs.uNumBufs = DEFNUMBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;

	nodeAttrin.cbStruct = sizeof(struct DSP_NODEATTRIN);
    nodeAttrin.uTimeout = 1000;
    nodeAttrin.iPriority = 1;

#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif


	arrBuf = OsSpecificMemAlloc(sizeof(BYTE *) * DEFNUMBUFS);
	arrBuf1 = OsSpecificMemAlloc(sizeof(BYTE *) * DEFNUMBUFS);

	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt_test(argc, (char *const)argv, "+vl:x:h:n:s:d:i:e:m:p:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DelayTransfer_STRESS: Verbose Mode On\n");
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLogDelay = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TST_verbose("Log opened as: %s\n", szLogName);
            // TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
		case 'h':
			/* Host Delay (in Milli Seconds) */
			iHostDelay = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Host side Delay in MilliSeconds = %d\n",iHostDelay);
            TSTLOG_appendLog(pHandle, "Host side Delay in MilliSeconds = %d\n",iHostDelay);
			break;
		case 'n':
			/* Number of Transfers for each Delay */
			iNumTransfers = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of Transfers for each Delay = %d\n",iNumTransfers);
            TSTLOG_appendLog(pHandle, "Number of Transfers for each Delay = %d\n",iNumTransfers);
			break;
		case 's':
			/* Buffer Size */
			iBuffSize = CSL_Atoi(optarg);
			iMandArgCount--;

			TST_verbose("Buffer size = %d\n",iBuffSize);
            TSTLOG_appendLog(pHandle, "Buffer size = %d\n",iBuffSize);
			break;
		case 'd':
			/* DSP side Initial Value of Delay (in NOPs) */
			iDspStartVal = CSL_Atoi(optarg);
			iMandArgCount--;

			TST_verbose("DSP side Initial Delay Val = %d\n",iDspStartVal);
            TSTLOG_appendLog(pHandle, "DSP side Initial Delay Val = %d\n",iDspStartVal);
			break;
		case 'i':
			/* DSP side Delay Increment (in NOPs) */
			iDspIncrement = CSL_Atoi(optarg);
			iMandArgCount--;

			TST_verbose("DSP side Delay Increment = %d\n",iDspIncrement);
            TSTLOG_appendLog(pHandle, "DSP side Delay Increment = %d\n",iDspIncrement);
			break;
		case 'e':
			/* DSP side Delay End Value */
			iDspStopVal = CSL_Atoi(optarg);
			iMandArgCount--;

			TST_verbose("DSP side Delay End Val = %d\n",iDspStopVal);
            TSTLOG_appendLog(pHandle, "DSP side Delay End Val = %d\n",iDspStopVal);
			break;
		case 'm':
			/* DSP side Delay Multiple */
			iDspMult = CSL_Atoi(optarg);
			iMandArgCount--;

			TST_verbose("DSP side Multiple value = %d\n",iDspMult);
            TSTLOG_appendLog(pHandle, "DSP side Multiple value = %d\n",iDspMult);
			break;
		case 'p':
			/* DSP side Delay Position (0..8) */
			iDspDelayPos = CSL_Atoi(optarg);
			iMandArgCount--;

			TST_verbose("DSP side Delay position is: %d\n",iDspDelayPos);
            TSTLOG_appendLog(pHandle, "DSP side Delay position is: %d\n",iDspDelayPos);
			break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);

            if (bUseLogDelay) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt_test(argc, (char *const)argv, "+vl:x:h:n:s:d:i:e:m:p:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		/* Error */
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
	}

    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    

    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		/* Error */
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		graceful_exit(NULL);
	}
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogDelay) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Attach()\n");
        }
        graceful_exit(NULL);
    }

    if(fLoadDsp)
    {
    /* Load default program */
    status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);
	TST_verbose("Processor Load returned = %x\n",status);
    if (!DSP_SUCCEEDED(status)) 
	{
		TSTLOG_appendLog(pHandle, "Status Val: %x\n",status);
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLogDelay) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Load()\n");
        }
		graceful_exit(hProcessor);
    }

    status = DSPProcessor_Start(hProcessor);
    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLogDelay) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Start()\n");
        }
        graceful_exit(hProcessor);
	    }
    }


	pszUuid = "F95CA07C_EE7D_11D5_A8DD_00B0D055F6D1";  // taken from timing_nortdx_int.cdb file
    UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

    /* Allocate Node */
	/* We can also make nodeAttrin as NULL and default values will be taken */
    status = DSPNode_Allocate(hProcessor, &gidNodeID, NULL, &nodeAttrin,
                   &hNode);
	TST_verbose("Node Allocate returned = %x\n",status);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLogDelay) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Allocate().\n");
        }
        graceful_exit(hProcessor);
    }

    status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogDelay) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        graceful_exit(hProcessor);
    }

    status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogDelay) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        graceful_exit(hProcessor);
    }


    /* Create Node on DSP */
    status = DSPNode_Create(hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogDelay) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Create()\n");
        }
        graceful_exit(hProcessor);
    }

    status = DSPNode_Run(hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogDelay) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Run()\n");
        }
        graceful_exit(hProcessor);
    }

	TST_verbose("Node Run returned = %x\n",status);

    status = DSPStream_Open(hNode, DSP_TONODE, 0, NULL, &hStreamOut);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogDelay) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStream_Open()\n");
        }
        graceful_exit(hProcessor);
    }
	TST_verbose("Stream Open One returned status = %x hStreamOUt = 0x%x\n",status,hStreamOut);

    status = DSPStream_AllocateBuffers(hStreamOut, iBuffSize, arrBuf, DEFNUMBUFS);
	TST_verbose("Stream Alloc One returned = %x\n",status);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogDelay) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers()\n");
        }
        graceful_exit(hProcessor);
    }

    status = DSPStream_Open(hNode, DSP_FROMNODE, 0, NULL, &hStreamIn);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogDelay) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in second DSPStream_Open()\n");
        }
        graceful_exit(hProcessor);
    }
	TST_verbose("Stream Open Two returned status = %x hStreamIn = 0x%x\n",status,hStreamIn);

    status = DSPStream_AllocateBuffers(hStreamIn, iBuffSize, arrBuf1, DEFNUMBUFS);
	TST_verbose("Stream Alloc Two returned = %x\n",status);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogDelay) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in second DSPStream_AllocateBuffers()\n");
        }
        graceful_exit(hProcessor);
    }

	/* Check for Data Transfer with different Delays */

    timeValue = iDspStartVal ;

	/* iterate till the End Delay value has reached */
    while (timeValue < iDspStopVal)
    {
		if ((timeValue >= 64000) || (iDspMult >= 64000))
		{
			/* Report Error */
	        TST_verbose("FAIL\n");

		    if (bUseLogDelay) 
			{
				TSTLOG_appendLog(pHandle,"FAIL\n");
				TSTLOG_appendLog(pHandle,"ERROR:Time Overflow. Delay: %d   Multiple: %d",
					timeValue, iDspMult);
			}
			graceful_exit(hProcessor);
		}

		pData = (SHORT *)arrBuf[0] ;

		pData[0] = (SHORT) timeValue;
		pData[1] = (SHORT) iDspMult;

		DspDelayPosition(iDspDelayPos, (BYTE **)&pData);

        /* fill the remaining buffer */
        for (i = 5; i < (iBuffSize / 2); i++) 
	    {
			pData[1] = (SHORT) i;
        }

        /*  Data transfer for number of iterations .. specified by the
         *  user as parameters to the test ... set in the constructor
	     */
		TST_verbose("Entering the For loope \n");
        for (i = 0 ; i < iNumTransfers ; i++)
	    {
			/* Issue Data */
			status = DSPStream_Issue(hStreamOut, arrBuf[0], iBuffSize, iBuffSize, 0);
			TST_verbose("DSPStream_Issue returned = %x\n",status);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLogDelay) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStream_Issue()\n");
				}
				graceful_exit(hProcessor);
			}

			/* Wait for GPP side time */


			TST_verbose("Slept for %d milliseconds. Now calling Reclaim\n",iHostDelay);
			/* Reclaim Data */
			status = DSPStream_Reclaim(hStreamOut, &arrBuf[0], &argTemp1, &argTemp2, &argTemp3);
			TST_verbose("DSPStream_Reclaim returned = %x\n",status);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLogDelay) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStream_Reclaim()\n");
				}
				graceful_exit(hProcessor);
			}

			/* Issue Buffer on Input Stream */
			status = DSPStream_Issue(hStreamIn, arrBuf1[0], iBuffSize, iBuffSize, 0);
			TST_verbose("DSPStream_Issue returned = %x\n",status);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLogDelay) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStream_Issue()\n");
				}
				graceful_exit(hProcessor);
			}

			/* Wait for GPP side time */

			/* Reclaim Data */
			status = DSPStream_Reclaim(hStreamIn, &arrBuf1[0], &argTemp1, &argTemp2, &argTemp3);
			TST_verbose("DSPStream_Reclaim returned = %x\n",status);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLogDelay) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStream_Reclaim()\n");
				}
				graceful_exit(hProcessor);
			}

            if (DSP_SUCCEEDED(status)) 
            {
                if(i%2 ==0)
                    TST_verbose("Data Tansfer %d is OK\n", i);
            }
			/* Check for validity of Data */

	    } // End of For loop - iNumTransfers

	/* Increment the time by the step specified. */
    timeValue += iDspIncrement ;
	TST_verbose("Incremented Time Value = %d\n",timeValue);

	} // End of While - timeValue < iDspStopVal

	TST_verbose("Finished the While loop Successfully !!\n");

	status = DSPStream_Idle(hStreamOut, TRUE);
	status = DSPStream_Idle(hStreamIn, TRUE);

	
	status = DSPStream_FreeBuffers(hStreamIn, arrBuf1, DEFNUMBUFS);
	status = DSPStream_FreeBuffers(hStreamOut, arrBuf, DEFNUMBUFS);

    status = DSPStream_Close(hStreamIn);
    status = DSPStream_Close(hStreamOut);
    status = DSPNode_Terminate(hNode, &exitStatus);
    status = DSPNode_Delete(hNode);

    DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);
    OsSpecificMemFree(arrBuf);
    OsSpecificMemFree(arrBuf1);

    /* Report in LOG module */
    if (bUseLogDelay) {
        TSTLOG_appendLog(pHandle, "PASS\n");
    }
    TST_verbose("PASS\n");

    /* Close Log */
    if (bUseLogDelay) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

