/*  ============================================================================
 *   File    IvaLowResource.c
 *
 *   Path    $(TESTSUITEROOT)\Stress\LowResource\IvaLowResource.c
 * 
 *   Desc:
 *           This test case stress tests the Bridge by 
 *           consuming memory in chunks and testing it for proper
 *           functioning under these low resource conditions.
 *
 *  Usage:
 *        LowResource [-v] [-l "log path/filename"] 
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *        -c: Chunk size to be consumed each iteration(bytes)
 *        -n: Maximum number of Iterations, after which the test is stopped
 *        -s: Buffer Size
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   AUG 07, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>

#include "LowResource.h"

#define DEFNAMESIZE        100
#define NUMOFCOMPARGS      3	/* Compulsory args: c,n,s */
#define NUMTRANSFERS       10
#define DEFBUFSIZE         64
#define NONE               0
#define FAILURE            1
#define PASS               0

#define SHORT short
static int     funccount = 1;

#define DSPEXEC        "iva.dof"


/* Global variables */
static IvaListElem 	  *firstElem; /* address of first elem in linked list */
static BOOL       bUseLogLowRes=FALSE;
static FILE              *pHandleLowRes;
static BOOL	  fLoadDsp = FALSE;

/* Local function declaration */
static INT delAllNodes(IvaListElem *elem);


static INT addNodeToList(INT sizeOfBuff);


/* Set of functions to implement a linked list which is used
 * to consume memory on GPP side and cause a Low Resource condition
 * retval -> 0 success, 1 fail.
 */



static INT delAllNodes(IvaListElem *elem)
{
    if(funccount == 1)
    	TST_verbose("Entering DelAll Nodes\n");
    if(elem->next != NULL)
	{
        delAllNodes(elem->next);
        funccount++;
	}
    else
    {
        OsSpecificMemFree(elem);
        TST_verbose("Number of Nodes deleted = %d\n", funccount);
        
    }
        
    return 0;

}


static INT addNodeToList(INT sizeOfBuff)
{
	static IvaListElem *prevElem=NULL;
	IvaListElem *elem;
	if (prevElem == NULL)
	{
		/* First time */
		firstElem = (IvaListElem *) OsSpecificMemAlloc(sizeof(IvaListElem));
		if (firstElem == NULL)
		{
			/* error - add a print statement */
			return FAILURE;
		}
		prevElem = firstElem;
	}
	elem = (IvaListElem *) OsSpecificMemAlloc(sizeof(IvaListElem));
	if (elem == NULL)
	{
		/* error */
        TST_verbose("ELEM == NULL. Returning Error\n");
		return FAILURE;
	}
	prevElem->next = elem;
	elem->content = (char *)OsSpecificMemAlloc(sizeOfBuff);
	elem->next = NULL;
	prevElem=elem;
	return PASS;
} /* end of addNodeToList */



/* -------------------------------------------------- */
/* Check Bridge Status part of the program            */
/* -------------------------------------------------- */

static INT checkBridgeStatus(CHAR **aArgv, INT iBuffSize)
{
    
    

    IVAMSG_TASK     ivaMsgTask;
    DSP_STATUS      status = DSP_SOK;
    struct DSP_MSG         ivaMsg;
    ivaMsgTask.hProcessorIva = NULL;
    ivaMsgTask.hNodeIva = NULL;
    ivaMsgTask.numMsg = NULL;
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId;
    struct DSP_NODEATTRIN          nodeAttrin;
    INT   iNodeCount;
    INT                     retVal = FAILURE;
    struct DSP_PROCESSORATTRIN         *pAttrIn;
    INT                     iArgc = 1;
    struct DSP_UUID                gidNodeID;
    INT 		    k=0;
    UINT                    uTimeout = 15000;
    DSP_STATUS              exitStatus = DSP_SOK;
    /* Initialising attributes */
    nodeAttrin.cbStruct = sizeof(struct DSP_NODEATTRIN);
    nodeAttrin.uTimeout = 1000;
    nodeAttrin.iPriority = 1;

    iNodeCount = 0; /* Initialise */


    gidNodeID=Iva_Msguuid;
    	TST_verbose("IVA Executable: %s\n", aArgv[0]);

	pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		/* Error */
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		return retVal;
	}
    	pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    	pAttrIn->uTimeout = 50;

    /* Attach to Processor */

	while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        TSTLOG_appendLog(pHandleLowRes,"IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
        }
    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status))
	 {
	    status = DSPProcessor_Attach(procId, pAttrIn,ivaMsgTask.hProcessorIva);
               if (DSP_SUCCEEDED(status)) 
		{
            		TSTLOG_appendLog(pHandleLowRes, "IVADSPProcessor_Attach succeeded.\n");
        	}
            
	      else 
		{
           		 TSTLOG_appendLog(pHandleLowRes, "IVADSPProcessor_Attach failed. Status = 0x%x\n",status);
        	}
    	 }
        else
            TSTLOG_appendLog(pHandleLowRes, "Failed to get the desired processor \n");




    if (!DSP_SUCCEEDED(status)) 
	{
          /* Report Error */
        TST_verbose("FAIL\n");

          if (bUseLogLowRes) 
   	   {
               TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
               TSTLOG_appendLog(pHandleLowRes,
               "Error in IvaDSPProcessor_Attach()\n");
           }
        
		return retVal;
       }

	
    /* Load default program */
    status = DSPProcessor_Load(ivaMsgTask.hProcessorIva, iArgc, (CONST CHAR **)aArgv, NULL);
    if (!DSP_SUCCEEDED(status)) 
	{
		TSTLOG_appendLog(pHandleLowRes, "Status Val: %x\n",status);
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLogLowRes) 
		{
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,
            "Error in DSPProcessor_Load()\n");
        }
		
        delAllNodes(firstElem);
	DSPProcessor_Detach(ivaMsgTask.hProcessorIva);
	return retVal;
    }

    status = DSPProcessor_Start(ivaMsgTask.hProcessorIva);

    if (!DSP_SUCCEEDED(status)) 
     {
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLogLowRes) 
		{
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,
            "Error in DSPProcessor_Start()\n");
        }
        
        delAllNodes(firstElem);
	DSPProcessor_Detach(ivaMsgTask.hProcessorIva);
	return retVal;
     }


    /* Allocate Node */
    status = DSPNode_Allocate(ivaMsgTask.hProcessorIva, &gidNodeID, NULL, &nodeAttrin,&ivaMsgTask.hNodeIva);
	TST_verbose("DSPNode_Allocate ret val = %x\n",status);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLogLowRes) 
	    {
            TSTLOG_appendLog(pHandleLowRes, "FAIL status = 0x%x\n", status);
            TSTLOG_appendLog(pHandleLowRes,"Error in DSPNode_Allocate().\n");
            }
        
        
          delAllNodes(firstElem);
	  DSPProcessor_Detach(ivaMsgTask.hProcessorIva);
	  return retVal;
        }


    /* Create Node on DSP */
    status = DSPNode_Create(ivaMsgTask.hNodeIva);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogLowRes) 
	  {
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,"Error in DSPNode_Create()\n");
          }
        
        
        	delAllNodes(firstElem);
   		DSPProcessor_Detach(ivaMsgTask.hProcessorIva);

		return retVal;
        }

    status = DSPNode_Run(ivaMsgTask.hNodeIva);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogLowRes) 
	   {
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,"Error in DSPNode_Run()\n");
           }
        
        
        	delAllNodes(firstElem);
   		DSPProcessor_Detach(ivaMsgTask.hProcessorIva);
		return retVal;
        }
	

       for(k = 0; k < 10; k++)
          {

	ivaMsg.dwCmd =  RMS_USER+6; /* Multiply the number by 2*/
        ivaMsg.dwArg1 = 10;

	status = DSPNode_PutMessage(ivaMsgTask.hNodeIva, &ivaMsg, uTimeout);

	if(!DSP_SUCCEEDED(status))
    		 {
        		/* Report Error */
        		TST_verbose("FAIL\n");

        		if (bUseLogLowRes)
                	{
            		  TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            		  TSTLOG_appendLog(pHandleLowRes,"Error in DSPNode_Run()\n");
        	         }
		 delAllNodes(firstElem);
                 DSPProcessor_Detach(ivaMsgTask.hProcessorIva);
                 return retVal;
		}


		status = DSPNode_GetMessage(ivaMsgTask.hNodeIva, &ivaMsg, uTimeout);
		if(!DSP_SUCCEEDED(status))
                 {
                        /* Report Error */
                        TST_verbose("FAIL\n");

                        if (bUseLogLowRes)
                        {
                          TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
                          TSTLOG_appendLog(pHandleLowRes,"Error in DSPNode_GetMessage()\n");
                         }
                 delAllNodes(firstElem);
                 DSPProcessor_Detach(ivaMsgTask.hProcessorIva);
                 return retVal;
                }
	     else
                {
				
		   if(ivaMsg.dwArg1!=k*2)
			{
			  
			  TSTLOG_appendLog(pHandleLowRes,"Error in Data check\n");

			}


		}

    		}

		status = DSPNode_Terminate(ivaMsgTask.hNodeIva, &exitStatus);
    		status = DSPNode_Delete(ivaMsgTask.hNodeIva);

    		DSPProcessor_Detach(ivaMsgTask.hProcessorIva);
    		OsSpecificMemFree(pAttrIn);

		retVal = PASS;
		return retVal;
}
/* -------------------------------------------------- */


/*
 *  ======== main ========
 *
 */
INT IvaLowResource_STRESS(INT argc, CHAR * argv[])
{
	INT            			iChunkSize = 0;
	INT            			iNumIterations = 0;
        INT            			iBuffSize = DEFBUFSIZE;
	CHAR           			szLogName[DEFNAMESIZE];
	INT            			iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS     			status;
        CHAR				*aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
	INT                             iNodeCount;
	INT				opt;
	


	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Add elem to linked list to increase mem consumption */
	/* Check Bridge Status by Loading the Image on IVA and
	 * doing a sample messaging */
	/* Continue the above two steps until the Bridge can not
	 * do any more messaging or till the specified 
	 * number of iterations has reached */
	/* End !! */
	/* Foll is the function to test the Bridge Status */
		/* Load the Image on DSP */
		/* Allocate Buffers for Transfer of Data */
		/* Set the Properties, Allocate and Create the Node on DSP */
		/* Open the Streams */
		/* Run the task */
		/* Check for Data Transfer */



	iNodeCount = 0; /* Initialise */

    TST_verbose("Starting Low Resource Tests\n");

	/* Process arguments and Store the values */
    	optind = 0;
	opt = getopt_test(argc, (char * const)argv, "+vl:x:c:n:s:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("IVALowResource_STRESS: Verbose Mode On\n");
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLogLowRes = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandleLowRes = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandleLowRes, "DSPStream_Issue:");
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
	case 'c':
			/* Chunk size to be consumed each iteration(bytes) */
	    iChunkSize = CSL_Atoi(optarg);
	    iMandArgCount--;
	    TST_verbose("Chunk size = %d\n",iChunkSize);
	    TSTLOG_appendLog(pHandleLowRes, "Chunk size = %d\n",iChunkSize);
	    break;
	case 'n':
			/* Maximum number of Iterations, after which the test is stopped */
	    iNumIterations = CSL_Atoi(optarg);
	    iMandArgCount--;
	    TST_verbose("Number of Iterations = %d\n",iNumIterations);
            TSTLOG_appendLog(pHandleLowRes, "Number of Iterations = %d\n",iNumIterations);
	    break;
	case 's':
		/* Buffer Size */
	    iBuffSize = CSL_Atoi(optarg);
	    iMandArgCount--;
	    TST_verbose("Buffer Size = %d\n",iBuffSize);
            TSTLOG_appendLog(pHandleLowRes, "Buffer Size = %d\n",iBuffSize);
	    break;
        default:
            TSTLOG_appendLog(pHandleLowRes, "Invalid option: %c\n", opt);

            if (bUseLogLowRes) {
                TSTLOG_appendLog(pHandleLowRes,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt_test(argc, (char* const)argv, "+vl:x:c:n:s:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing FAIL \n");
		return 0;
	}


	while (1)
	{
		status = addNodeToList(iChunkSize);
		if(status != PASS)
		{
			/* failure */
			TST_verbose("Could not add any more nodes to Link List!\n");
			TST_verbose("Calling Delete All nodes\n");
           		delAllNodes(firstElem);
	                TST_verbose("Deleted all nodes\n");
		}
		status = checkBridgeStatus(aArgv, iBuffSize);
		if(status == FAILURE)
		{
			TST_verbose("checkBridgeStatus FAILED\n");
			TSTLOG_appendLog(pHandleLowRes, "checkBridgeStatus FAILED\n");
			return 0;
		}

		iNodeCount++;
        	TST_verbose("iNodeCount = %d\n", iNodeCount);

		/* After every 50 iterations, print a message */
		if((iNodeCount % 50) == 0)
		{
			TST_verbose("Num of Nodes till now = %d\n",iNodeCount);
			TSTLOG_appendLog(pHandleLowRes, "Num of Nodes till now = %d\n",iNodeCount);
		}

		if (iNodeCount >= iNumIterations)
		{
			TST_verbose("Reached the Maximum number of Iterations.Stop now.\n");
			TSTLOG_appendLog(pHandleLowRes, "Reached the Maximum number of Iterations.Stop now.\n");
            		return 0;
		}
	}



return 0;
}

