/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    MultiProcessData.c
 *
 *   Path    $(TESTSUITEROOT)\Stress\MultiProcess\MultiProcessData.c
 * 
 *   Desc:
 *           This test case tests the Data Transfer by 
 *           creating multiple processes.
 *
 *  Usage:
 *        STRESS MULPROC [-v] [-l "log path/filename"] 
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *        -p: Number of Processs to be created
 *        -n: Number of Transfers for each Iteration
 *        -s: Buffer Size
 *        -t: Test for Kill Task
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   AUG 05, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>

#include "MultiProcessData.h"

#include <errno.h>
extern int errno;

#define DEFNAMESIZE      100
#define DEFBUFSIZE        64
#define DEFNUMBUFS         1
#define DEFNUMTXFRS    10000
#define DEFNUMPROCS        4
#define NUMOFCOMPARGS      3	/* Compulsory args: p,n,s */
#define MAXNUMOFPROCS      6    /* This number taken from Symbian Test Suite */
#define PASS               0
#define FAIL               1
#define DEFAULTDSPUNIT     0
#define SHORT short

#define DSPEXEC        "mttask_nortdxidl.dof"

/* Function to exit gracefully */
static INT graceful_exit()
{
	/*printf("FATAL ERROR: Exiting the Test Case !\n");
	exit(0);*/
}

/* Process function for Multi Process data transfer test case */
/*
 * Note: Address of args is being passed from the main Process,		*
 * dont alter the contents as this will be used/needed by all the	*
 * Processs created. Copy the contents to local variables and use	*
 */
static INT processFxn(MultiProcessArgs *targs)
{

    	struct DSP_STRMATTR			attrs;
	BYTE					**arrBuf;
    	struct DSP_NODEATTRIN			nodeAttrin;
    	DSP_HNODE				hNode = NULL;
	DSP_HSTREAM				hStream;
	CHAR					*pszUuid;
	struct DSP_UUID				gidNodeID;
    	struct DSP_CBDATA				*cbData;
	DSP_STATUS				status;
	INT						i,k;
	DSP_STATUS				exitStatus = DSP_SOK;
	struct DSP_STREAMATTRIN        strmAttrs;

	/* Processor related information */
	DSP_HPROCESSOR			hProcessor;

	/* Test case specific information */
	INT						iNumTransfers;
	INT						iBuffSize;
	INT						procNum;

	INT						killTask;
	/* Logging related information */
	FILE					*pHandle;
	BOOL					bUseLog=FALSE;
	BOOL					bVerbose=FALSE;
    	BYTE                    argsBuf[32 + sizeof(ULONG)];
    	DWORD				bufSize ;
    	DWORD				dataSize ;
    	DWORD				dwArg ;
    	DSP_HSTREAM		hStream1;
    	BYTE				**arrBuf1;
    	SHORT *			sendBuffer ;
    	SHORT *			receiveBuffer ;
    	SHORT *			pData ;
    	INT				transferMode;



	/* Copy the contents of passed args to local vars */
	/* This strategy can be changed if the variables are just
	   used/read and not modified. */
	iNumTransfers = targs->iNumTransfers;
	iBuffSize = targs->iBuffSize;
	pHandle = targs->pHandle;
	bUseLog = targs->bUseLog;
	bVerbose = targs->bVerbose;
	procNum = targs->procNum;
	transferMode = targs->transferMode;
	killTask = targs->killTask;

    /* Initialising attributes */
    attrs.uBufsize = iBuffSize;
    attrs.lMode = transferMode;
    attrs.uNumBufs = DEFNUMBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;
	if (transferMode  == STRMMODE_PROCCOPY)
    {
		attrs.uSegid = 0;
    }
    else
    {
		attrs.uSegid = DSP_SHMSEG0;    /* Use shared memory segment 0 */
    }

    nodeAttrin.iPriority = 5 ;
    nodeAttrin.uTimeout = 10000 ;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif


	
   cbData = (struct DSP_CBDATA *)argsBuf ;    
 
   cbData->cbData = 32;
	if(killTask == 1) {
	cbData->cData[0] = 0x31;
    	}

	arrBuf = OsSpecificMemAlloc(sizeof(BYTE *));
	arrBuf1 = OsSpecificMemAlloc(sizeof(BYTE *));

    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, NULL, &hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Attach()\n");
        }
		return FAIL;
     }


	pszUuid = "F95CA07A_EE7D_11D5_A8DD_00B0D055F6D1";  // taken from mttask_nortdx_int.cdb file
    UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

    /* Allocate Node */
    status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, &nodeAttrin,
                   &hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Allocate().\n");
        }
        return FAIL;
    }

    status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        return FAIL;
    }

    status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        return FAIL;
    }


    /* Create Node on DSP */
    status = DSPNode_Create(hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Create()\n");
        }
        return FAIL;
    }

    status = DSPNode_Run(hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Run()\n");
        }
        return FAIL;
    }


	strmAttrs.cbStruct = sizeof(struct DSP_STREAMATTRIN);
	strmAttrs.uTimeout = 5000;
	strmAttrs.uAlignment = 0;
	strmAttrs.uNumBufs = 4;   /* To have 4 buffers in the Queue, for this test case */
	strmAttrs.lMode = transferMode;
	strmAttrs.uDMAChnlId = 0;
	strmAttrs.uDMAPriority = 0;

	if (strmAttrs.lMode  == STRMMODE_PROCCOPY)
    {
        strmAttrs.uSegment = 0;
    }
    else
    {
		strmAttrs.uSegment = DSP_SHMSEG0;    /* Use shared memory segment 0 */
    }



    status = DSPStream_Open(hNode, DSP_TONODE, 0, &strmAttrs, &hStream);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStraem_Open()\n");
        }
        return FAIL;
    }

    status = DSPStream_AllocateBuffers(hStream, iBuffSize, arrBuf, DEFNUMBUFS);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers()\n");
        }
        return FAIL;
    }

    status = DSPStream_Open(hNode, DSP_FROMNODE, 0, &strmAttrs, &hStream1);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStraem_Open()\n");
        }
        return FAIL;
    }

    status = DSPStream_AllocateBuffers(hStream1, iBuffSize, arrBuf1, DEFNUMBUFS);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers()\n");
        }
        return FAIL;
    }

	/* Do the Data Transfer */


	    pData = (SHORT *)arrBuf[0] ;
	    receiveBuffer = (SHORT *) arrBuf1[0] ;

        /*  Data transfer for number of iterations .. specified by the
         *  user as parameters to the test ... set in the constructor
	     */

        sendBuffer = OsSpecificMemAlloc(iBuffSize) ;

        for (k = 0 ; k < iNumTransfers ; k++)
	    {
		    for(i=0;i<iBuffSize/2;i++)
            {
	            pData[i]  = sendBuffer[i] = procNum + i + k ;
	        }

			/* Issue Full Data Buffer */
			status = DSPStream_Issue(hStream, (BYTE *)pData, iBuffSize, iBuffSize, 0);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLog) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Issue()\n");
				}
				return FAIL;
			}

			/* Reclaim Empty Buffer */
			status = DSPStream_Reclaim(hStream, (BYTE**)&pData, &dataSize, &bufSize,&dwArg);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLog) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Reclaim()\n");
				}
				return FAIL;
			}

			/* Issue Empty Buffer */
			status = DSPStream_Issue(hStream1, (BYTE *)receiveBuffer, iBuffSize, iBuffSize, 0);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLog) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Issue()\n");
				}
				return FAIL;
			}

			/* Reclaim Full Buffer */
			status = DSPStream_Reclaim(hStream1, (BYTE**)&receiveBuffer, &dataSize, &bufSize,&dwArg);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLog) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Reclaim()\n");
				}
				return FAIL;
			}

			/* Check for validity of Data */
		    for( i = 0 ;i< iBuffSize/2 ;i++)
		    {
				if(sendBuffer[i] != receiveBuffer[i])
				{
					status = DSP_EFAIL ;
		        	////printf("Data Verification Failed: Expected 0x%x Received 0x%x\n",sendBuffer[i],receiveBuffer[i]) ;
					if (bUseLog) 
					{
						TSTLOG_appendLog(pHandle, "FAIL\n");
						TSTLOG_appendLog(pHandle,"Data Verification Failed: Expected 0x%x Received 0x%x\n",
							sendBuffer[i],receiveBuffer[i]);
					}
				}
            }

			if ((k % 500) == 0)
            {
			////	printf("Iteration num = %d\n",k);
		    }

	    } // End of For loop - iNumTransfers

	/* Free the Allocated Buffers */
	status = DSPStream_FreeBuffers(hStream, arrBuf, DEFNUMBUFS);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPStraem_FreeBuffers() for hStream\n");
		}
		///printf("Error in DSPStraem_FreeBuffers()\n");
		return FAIL;
	}

	status = DSPStream_FreeBuffers(hStream1, arrBuf1, DEFNUMBUFS);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPStraem_FreeBuffers() for hStream1\n");
		}
	///	printf("Error in DSPStraem_FreeBuffers() for hStream1\n");
		return FAIL;
	}

    status = DSPStream_Close(hStream);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPStraem_Close() for hStream\n");
		}
		///printf("Error in DSPStraem_Close() for hStream\n");
		return FAIL;
	}

	status = DSPStream_Close(hStream1);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPStraem_Close() for hStream1\n");
		}
		////printf("Error in DSPStraem_Close() for hStream1\n");
		return FAIL;
	}

    status = DSPNode_Terminate(hNode, &exitStatus);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPNode_Terminate()\n");
		}
		////printf("Error in DSPNode_Terminate\n");
		return FAIL;
	}

    status = DSPNode_Delete(hNode);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPNode_Delete()\n");
		}
		////printf("Error in DSPNode_Delete()\n");
		return FAIL;
	}

    OsSpecificMemFree(arrBuf);
	OsSpecificMemFree(arrBuf1);
	OsSpecificMemFree(sendBuffer) ;
    /* Return from this proc */

    DSPProcessor_Detach(hProcessor);

	return 0;
}



/*
 *  ======== main ========
 *
 */
INT multiProcessData_STRESS(INT argc, CHAR * argv[])
{
    BOOL           bUseLog=FALSE;
    BOOL           bVerbose=FALSE;
    FILE           *pHandle = NULL;
	INT            iNumProcs = DEFNUMPROCS;
    INT            iNumTransfers = DEFNUMTXFRS;
    INT            iBuffSize = DEFBUFSIZE;
	CHAR           szLogName[DEFNAMESIZE];
	INT            iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS     status;
	PHANDLE		   procId[MAXNUMOFPROCS];
	MultiProcessArgs procArgs;
    struct DSP_PROCESSORATTRIN		*pAttrIn;
	DSP_HPROCESSOR			hProcessor;
    CHAR					*aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
	INT						iArgc = 1;
	INT					opt;
	INT					index;
    INT                 retVal = PASS;
	INT					procStatus ;
	CHAR				sImageName[DEFNAMESIZE];
	CHAR				sCmdLine[DEFNAMESIZE];
	LPDWORD				    lpExitCode = 0;
	INT 				transferMode = STRMMODE_PROCCOPY ;
	BOOL				fLoadDsp = FALSE;
	INT                 killTask = 0;



	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Load the Image on DSP */
	/* Create requested number of Processes and pass the
	   input params to these Procs */
	/* In each of the Process do the following: */
		/* Allocate Buffers for Transfer of Data */
		/* Set the Properties, Allocate and Create the Node on DSP */
		/* Open the Streams */
		/* Run the task */
		/* Do the Data Transfer */
	/* End !! */


	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt(argc, argv, "+vl:x:p:n:s:t:RZP:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("MultiProcessDataTx_STRESS: Verbose Mode On\n");
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "DSPStream_Issue:");
            TST_verbose("Log opened as: %s\n", szLogName);
            // TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
		case 'p':
			/* Number of Procs to be created */
			iNumProcs = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of Procs to be Created = %d\n",iNumProcs);
            		TSTLOG_appendLog(pHandle,"Number of Procss to be Created = %d\n",iNumProcs);
			break;
		case 'n':
			/* Number of Transfers */
			iNumTransfers = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of transfers set to %d\n",iNumTransfers);
		        TSTLOG_appendLog(pHandle, "Number of transfer set to %d",iNumTransfers);
			break;
		case 's':
			/* Buffer Size */
			iBuffSize = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Buffer Size set to %d\n",iBuffSize);
		        TSTLOG_appendLog(pHandle, "Buffer Size set to %d\n",iBuffSize);
			break;

		case 'R':
       			transferMode = STRMMODE_RDMA ;
			TSTLOG_appendLog(pHandle, "RDMA Transfer Mode\n") ;
            		break ;

		

		case 't':
			/* Enabling the kill task */
			killTask = CSL_Atoi ((CHAR *)optarg);
			TST_verbose ("KillTask set to %d\n", killTask);
			TSTLOG_appendLog (pHandle, "KillTask set to %d\n", killTask);
		        break ;
		case 'P':
			transferMode = STRMMODE_PROCCOPY ;
			TSTLOG_appendLog(pHandle, "Processor Copy transfer mode\n") ;
			break ;
        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);

            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt(argc, argv, "+vl:x:p:n:s:t:RZP:");
    }


	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		/* Error */
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
		///printf("Number of Mandatory Arguments is incorrect.\n");
		return 0;
	}

    TST_verbose("DSP Executable: %s\n", aArgv[0]);


    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		/* Error */
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		graceful_exit();
	}
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Attach to Processor */
    status = DSPProcessor_Attach(0, pAttrIn, &hProcessor);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Attach()\n");
        }
        graceful_exit();
    }

    if(fLoadDsp)
    {
    /* Load default program */
    status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR**)aArgv, NULL);
    if (!DSP_SUCCEEDED(status)) 
	{
		TSTLOG_appendLog(pHandle, "Status Val: %x\n",status);
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Load()\n");
        }
		graceful_exit();
    }

    status = DSPProcessor_Start(hProcessor);
    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Start()\n");
        }
        graceful_exit();
	    }
    }

	DSPProcessor_Detach(hProcessor);

	/* Fill the procArgs which are to be passed on to the Process */
	procArgs.hProcessor = hProcessor;
	procArgs.iNumTransfers = iNumTransfers;
	procArgs.iBuffSize = iBuffSize;
	procArgs.pHandle = pHandle;
	procArgs.bUseLog = bUseLog;
	procArgs.bVerbose = bVerbose;
	procArgs.transferMode = transferMode ;
	procArgs.killTask = killTask;


	/* Create requested number of Processes and pass the
	   input params to these Procs */
	for (index = 0; index < iNumProcs; index ++)
	{
		procArgs.procNum = index;
		procId[index] = OsSpecificProcessCreate(sImageName,sCmdLine,NULL);
		if (procId[index] < 0)
		{
			/* Failed to Create a Process */
			TST_verbose("Failed to Create Process\n");
			////printf("Failed to Create Process\n");
			return 0;
		}
		if (procId[index] == 0)
		{
			/* Child Process */
			procStatus = processFxn(&procArgs);
			return (procStatus);
		}
		else
		{
			/* Parent Process */
		}
	}

	sleep(5);

	for (index = 0; index < iNumProcs; index ++)
	{
		procStatus = OsSpecificWaitForProcessExit(procId[index],lpExitCode);
		if (procStatus < 0)
		{
			// error
			////printf("ERROR: Proc with index %d FAILED\n",index);
			retVal = FAIL;
			TSTLOG_appendLog(pHandle, "FAIL \t");
			TSTLOG_appendLog(pHandle, "ERROR: Proc with index %d FAILED\n",index);
			////printf("ERROR: Proc with index %d FAILED\n",index);
		}
	}

    OsSpecificMemFree(pAttrIn);

    if (retVal == PASS) 
	{
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

