/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPNode_Connect.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPNode\DSPNode_Connect.c
 * 
 *   Desc:
 *        DSPNode API test
 *
 *  DSPNode API function tested:
 *        DSPNode_Connect(DSP_HNODE hNode, UINT uStream,
 *                        DSP_HNODE hOtherNode, UINT uOtherStream,
 *			  DSP_STRMATTR *pAttr)
 *  Usage:
 *        DSPNode_Connect [-v] [-l "log path/filename"] [-x "valid executable"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>   
#include <DSPProcessor_OEM.h>

#define DEFNAMESIZE    80
#define DEFAULTDSPUNIT 0
#define DSPEXEC         "dataxfr_nortdxidl.dof"

/*
 *  ======== main ========
 *
 */
INT DSPNode_Connect_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT            opt;
    INT            cRetVal = 1;
    INT            i, j, k, l, m;
    BOOL        bUseLog=FALSE;
    INT            iArgc = 1;
    FILE        *pHandle=NULL;
    CHAR        szLogName[DEFNAMESIZE];
    CHAR                *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR        *pszUuid;
    DSP_HPROCESSOR     hProcessor;
    struct DSP_UUID        gidNodeID;
    DSP_STATUS        status;
    DSP_STATUS        tmpStatus;
    struct DSP_PROCESSORATTRIN *pAttrIn;
    struct DSP_NODEATTRIN     *nodeAttrin;
    struct DSP_CBDATA        *cbData;
    DSP_STATUS      exitStatus = DSP_SOK;

    /* INPUT PARAMETERS */
    DSP_HNODE arg1[] =      /* arg1: handle of first node to connect to */
    {
        (DSP_HNODE)DSP_HGPPNODE, /* Valid handle to be allocated */
        NULL, /* Invalid NULL handle */
    };

    UINT arg2[] =                    /* arg2: stream index on first node */
    {
        0, /* valid index */
        50                              /* invalid index out of range */
    };

    DSP_HNODE arg3[] =               /* arg3: handle of second node */
    {
        NULL, /* Valid handle to be allocated */
        NULL                         /* Invalid NULL handle */
    };

    UINT arg4[] =                    /* arg4: stream index on second node */
    {
        0, /* valid index */
        50                           /* invalid index out of range */
    };

    struct DSP_STRMATTR *arg5[] =           /* stream attributes structure */
    {
        NULL, /* valid attrs for PROCCOPY */
        NULL, /* valid attrs for ZEROCOPY */
        NULL                 /* invalid strmmode in attrs */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HNODE))
    #define NUMARG2 (sizeof(arg2)/sizeof(UINT))
    #define NUMARG3 (sizeof(arg3)/sizeof(DSP_HNODE))
    #define NUMARG4 (sizeof(arg4)/sizeof(UINT))
    #define NUMARG5 (sizeof(arg5)/sizeof(struct DSP_STRMATTR *))

    /* SOLUTION SET */
    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] [NUMARG4] [NUMARG5] =
                                      /* arg1 arg2 arg3 arg4 arg5 */
    {
        {
            {
                {
                    {
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][0][0] */
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][0][1] */
                        {DSP_ESTRMMODE, TST_SHOULDEQUAL}, /* [0][0][0][0][2] */
                    },
                    {
                        {DSP_EVALUE, TST_SHOULDEQUAL}, /* [0][0][0][1][0] */
                        {DSP_EVALUE, TST_SHOULDEQUAL}, /* [0][0][0][1][1] */
                        {DSP_EVALUE, TST_SHOULDEQUAL}, /* [0][0][0][1][2] */
                    },

                },
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][0][1][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][0][1][0][1] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][0][1][0][2] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][0][1][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][0][1][1][1] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][0][1][1][2] */
                    },
                },
            },
            {
                {
                    {
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][1][0][0][0] */
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][1][0][0][1] */
                        {DSP_ESTRMMODE, TST_SHOULDEQUAL}, /* [0][1][0][0][2] */
                    },
                    {
                        {DSP_EVALUE, TST_SHOULDEQUAL}, /* [0][1][0][1][0] */
                        {DSP_EVALUE, TST_SHOULDEQUAL}, /* [0][1][0][1][1] */
                        {DSP_EVALUE, TST_SHOULDEQUAL}, /* [0][1][0][1][2] */
                    },
                },
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][1][1][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][1][1][0][1] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][1][1][0][2] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][1][1][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][1][1][1][1] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][1][1][1][2] */
                    },
                },
            },
        },
        {
            {
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0][1] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0][2] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1][1] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1][2] */
                    },
                },
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0][1] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0][2] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1][1] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1][2] */
                    },
                },
            },
            {
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0][1] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0][2] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1][1] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1][2] */
                    },
                },
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0][1] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0][2] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1][1] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1][2] */
                    },
                },
            },
        },
    };

    /* GOOD ATTRS */
    arg5[0] = (struct DSP_STRMATTR *)OsSpecificMemAlloc(sizeof(struct DSP_STRMATTR));

    arg5[0]->uSegid = 0;
    arg5[0]->lMode = STRMMODE_PROCCOPY;
    arg5[0]->uBufsize = 512;
    arg5[0]->uNumBufs = 1;
    arg5[0]->uAlignment = 0;
    arg5[0]->uTimeout = DSP_FOREVER;

    arg5[1] = (struct DSP_STRMATTR *) OsSpecificMemAlloc(sizeof(struct DSP_STRMATTR));
    arg5[1]->uSegid = 0;
    arg5[1]->lMode = STRMMODE_PROCCOPY;
    
    arg5[1]->uBufsize = 512;
    arg5[1]->uNumBufs = 1;
    arg5[1]->uAlignment = 0;
    arg5[1]->uTimeout = DSP_FOREVER;

    /* BAD ATTRS */
    arg5[2] = (struct DSP_STRMATTR *)OsSpecificMemAlloc(sizeof(struct DSP_STRMATTR));
    arg5[2]->uSegid = DSP_SHMSEG0;
    arg5[2]->lMode = 93;
    arg5[2]->uBufsize = 512;
    arg5[2]->uNumBufs = 1;
    arg5[2]->uAlignment = 0;
    arg5[2]->uTimeout = DSP_FOREVER;

    pAttrIn = (struct DSP_PROCESSORATTRIN *)OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;


    cbData = (struct DSP_CBDATA *)OsSpecificMemAlloc(sizeof(struct DSP_CBDATA));
    cbData->cbData=1;
    cbData->cData[0] = 1;


    nodeAttrin = (struct DSP_NODEATTRIN *)OsSpecificMemAlloc(sizeof(struct DSP_NODEATTRIN));
    nodeAttrin->cbStruct = sizeof(struct DSP_NODEATTRIN);
    nodeAttrin->uTimeout = 2000;
    nodeAttrin->iPriority = 1;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin->uProfileID = 0xff;
#endif


    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DSPNode_Connect: Verbose Mode On\n");
            break;
       case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
         bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "DSPNode_Connect:\n");
            /* TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
         break;
    case 'x':
        /* use given executable */
        aArgv[0] = (char *) optarg;
        break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);
            cRetVal = 0;
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
    }
    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    TST_verbose("DSPNode_Connect: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;
    
        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
        
            status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
        }
    
        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Attach() and status = 0x%x\n",status);
            }
            break;
        }
        status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);
        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Load() and status = 0x%x\n",status);
            }
            break;
        }

        status = DSPProcessor_Start(hProcessor);
        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Start() and status = 0x%x\n",status);
            }
            break;
        }

        for (i = 0; i < NUMARG1; i++) {
            for (j = 0; j < NUMARG2; j++) {
                for (k = 0; k < NUMARG3; k++) {
                    for (l = 0; l < NUMARG4; l++) {
                        for (m = 0; m < NUMARG5; m++) {

						  pszUuid = "F95CA072_EE7D_11D5_A8DD_00B0D055F6D1"; // dataxfr
                          UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);


                            status = DSPNode_Allocate(hProcessor,
                                     &gidNodeID, cbData, nodeAttrin,
                                     &(arg3[0]));

			   tmpStatus = status;

                            if (!DSP_SUCCEEDED(status)) {
                                /* Report Error */
                                cRetVal = 0;
                                if (bUseLog) {
                                    TST_verbose("FAIL\n");
                                    TSTLOG_appendLog(pHandle, "FAIL\n");
                                    TSTLOG_appendLog(pHandle,
                                    "Error in DSPNode_Allocate() and status = 0x%x\n",status);
                            }
                            break;
                            }

                            /* Connect DSP Node to other DSP Node */
                            status = DSPNode_Connect(arg1[i], arg2[j],
                                                     arg3[k], arg4[l],
                                                     arg5[m]);

                            if (!TST_CheckVal(status,
                                checkArray[i][j][k][l][m].dwValue,
                                checkArray[i][j][k][l][m].fEqualFlag)) {
                                cRetVal = 0;
                                TST_verbose(
                                "Connect [%d][%d][%d][%d][%d] returned \
                                           %x\n", i, j, k, l, m, status);

                                if (bUseLog) {
                                    TSTLOG_appendLog(pHandle, "FAIL status = 0x%x\n",status);
                                    TSTLOG_appendLog(pHandle,
                                    "DSPNode_Connect [%d][%d][%d][%d][%d] \
                                    failed.\n", i, j, k, l, m);
                                }

	 
                            }
			if(DSP_SUCCEEDED(tmpStatus))
			{

				status = DSPNode_Terminate(arg3[0],&exitStatus);
	 			if(DSP_SUCCEEDED(status))
				{
                    			TSTLOG_appendLog(pHandle, "DSPNode_Terminate PASS\n");
				}	 

				status = DSPNode_Delete(arg3[0]);
	 			if(!DSP_SUCCEEDED(status))
				{
                    			TSTLOG_appendLog(pHandle, "DSPNode_Delete FAIL status = 0x%x\n",status);
				}	 

                        }   	

                        }
                    }
                }
            }
        }

	break;
}
    	        status = DSPProcessor_Detach(hProcessor);
	        if(!DSP_SUCCEEDED(status))
	        {
                   TSTLOG_appendLog(pHandle, "DSPNode_Detach FAIL status = 0x%x\n",status);
	        }


    OsSpecificMemFree(nodeAttrin);
    OsSpecificMemFree(pAttrIn);
    OsSpecificMemFree(arg5[0]);
    OsSpecificMemFree(arg5[1]);
    OsSpecificMemFree(arg5[2]);

    if (cRetVal && (DSP_SUCCEEDED(status))) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }
    else
    {
            TSTLOG_appendLog(pHandle, "FAIL and status=0x%x and cRetVal = 0x%x\n",status,cRetVal);
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

