/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPNode_Run.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPNode\DSPNode_Run.c
 * 
 *   Desc:
 *        DSPNode API test
 *
 *  DSPNode API function tested:
 *        DSPNode_Run(DSP_HNODE hNode);
 *  Usage:
 *        DSPNode_Run [-v] [-l "log path/filename"] [-x "valid executable"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>

#define DEFNAMESIZE    80
#define DEFAULTDSPUNIT 0
#define DSPEXEC         "dataxfr_nortdxidl.dof"

/*
 *  ======== main ========
 *
 */
INT DSPNode_Run_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT            opt;
    INT            cRetVal = 1;
    INT            i;
    INT            iArgc = 1;
    BOOL        bUseLog=FALSE;
    FILE        *pHandle=NULL;
    CHAR        szLogName[DEFNAMESIZE];
    CHAR                *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR         *pszUuid;
    DSP_HPROCESSOR     hProcessor;
    struct DSP_UUID        gidNodeID;
    DSP_STATUS        status;
    DSP_STATUS          exitStatus = DSP_SOK;
    struct DSP_PROCESSORATTRIN *pAttrIn;
    struct DSP_NODEATTRIN    *nodeAttrin;
    struct DSP_CBDATA        *cbData;
    struct DSP_STRMATTR      attrs;


    /* INPUT PARAMETERS */
    DSP_HNODE arg1[] =       /* arg1: handle of node to create */
    {
        NULL, /* Valid handle to be allocated */
        NULL, /* Invalid NULL handle */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HNODE))

    /* SOLUTION SET */
    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] =
    {
        {DSP_SOK, TST_SHOULDEQUAL}, /* [0] */
        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1] */
    };

    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    nodeAttrin = OsSpecificMemAlloc(sizeof(struct DSP_NODEATTRIN));
    nodeAttrin->cbStruct = sizeof(struct DSP_NODEATTRIN);
    nodeAttrin->uTimeout = 2000;
    nodeAttrin->iPriority = 1;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin->uProfileID = 0xff;
#endif


    cbData = OsSpecificMemAlloc(sizeof(struct DSP_CBDATA));
    cbData->cbData = 1;

    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DSPNode_Run: Verbose Mode On\n");
            break;
       case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
         bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "DSPNode_Run:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
    case 'x':
        /* use given executable */
        aArgv[0] = (char *) optarg;
        break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);
            cRetVal = 0;

            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
    }
    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    TST_verbose("DSPNode_Run: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;

        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
    
            status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
        }

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                    "Error in DSPProcessor_Attach()\n");
            }
            break;
        }

        arg1[1] = arg1[0];
        /* Load default program */
        status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Load()\n");
            }
            break;
        }

        status = DSPProcessor_Start(hProcessor);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Start()\n");
            }
            break;
        }

		pszUuid = "F95CA072_EE7D_11D5_A8DD_00B0D055F6D1";
        UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

        /* Allocate Node */
        status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, nodeAttrin,
                   &arg1[0]);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Allocate().\n");
            }
            break;
        }

		/* The following two Connect calls are added to accomodate the
		   usage of dataxfr dof instead of the apitest0 dof. The DSP
		   side expects the Connect calls. So these are added here */
		attrs.uSegid = 0;
		attrs.uBufsize = 512;
		attrs.lMode = STRMMODE_PROCCOPY;
		attrs.uNumBufs = 1;
		attrs.uAlignment = 0;
		attrs.uTimeout = 5000;

        status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, arg1[0], 0,
                                 &attrs);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Connect()\n");
            }
            break;
        }

        status = DSPNode_Connect(arg1[0], 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                                 &attrs);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Connect()\n");
            }
            break;
        }

        /* Create Node on DSP */
        status = DSPNode_Create(arg1[0]);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Create()\n");
            }
            break;
        }

        for (i = 0; i < NUMARG1; i++) {

             /* Start Node running on DSP */
             status = DSPNode_Run(arg1[i]);

             if (!TST_CheckVal(status, checkArray[i].dwValue,
                 checkArray[i].fEqualFlag)) {
                 cRetVal = 0;
                 TST_verbose("Run [%d] returned %x\n", i, status);

                 if (bUseLog) {
                     TSTLOG_appendLog(pHandle, "FAIL\n");
                     TSTLOG_appendLog(pHandle,
                     "DSPNode_Run [%d] failed.\n", i);
                 }
             }
        }

        status = DSPNode_Terminate(arg1[0], &exitStatus);
        status = DSPNode_Delete(arg1[0]);
        break;
    }

    status = DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);
    OsSpecificMemFree(nodeAttrin);
    OsSpecificMemFree(cbData);

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

    return 0;
}

