/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPProcessor_Detach.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPProcessor\DSPProcessor_Detach.c
 * 
 *   Desc:
 *        DSPProcessor API test
 *
 *  DSPProcessor API function tested:
 *        DSPProcessor_Detach(DSP_HPROCESSOR *phProcessor);
 *  Usage:
 *        DSPProcessor_Detach [-v] [-l "log path/filename"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>

#define DEFNAMESIZE    80
#define DEFAULTDSPUNIT 0
/*
 *  ======== main ========
 *
 */
INT DSPProcessor_Detach_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT            opt;
    INT            i;
    INT            cRetVal = 1;
    BOOL        bUseLog=FALSE;
    FILE        *pHandle=NULL;
    CHAR        szLogName[DEFNAMESIZE];
    DSP_STATUS        status;
    struct DSP_PROCESSORATTRIN    *pAttrIn;


    /* INPUT PARAMETERS */
    DSP_HPROCESSOR arg1[] =       /* arg1: handle of processor to detach */
    {
        NULL, /* Valid handle to be attached */
        NULL, /* Invalid NULL handle */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HPROCESSOR))

    /* SOLUTION SET */
    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] =
    {
        {DSP_SOK, TST_SHOULDEQUAL}, /* [0] */
        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1] */
    };

    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DSPProcessor_Detach: Verbose Mode On\n");
            break;
     case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));

         bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);

            TSTLOG_appendLog(pHandle, "DSPProcessor_Detach:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
         break;
        case 'x':
             /* Use DSP executable */

             /*
              *  Not used in this test - SKIP
              *  Leave option here, however, in case executable argument
              *  is passed in from 'apitst'
              */

             break;
        default:
            TST_verbose(("Invalid option: %c\n"), opt);
            cRetVal = 0;
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
    }

    TST_verbose("DSPProcessor_Detach: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;
    
        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
        
            status = DSPProcessor_Attach(procId, pAttrIn, &arg1[0]);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
        }
    
        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Attach()\n");
            }
            break;
        }

        for (i = 0; i < NUMARG1; i++) {
             status = DSPProcessor_Detach(arg1[i]);

             if (!TST_CheckVal(status,
                 checkArray[i].dwValue,
                 checkArray[i].fEqualFlag)) {
                 cRetVal = 0;
                 TST_verbose("Detach [%d] returned %x\n", i, status);

                 if (bUseLog) {
                     TSTLOG_appendLog(pHandle, "FAIL\n");
                     TSTLOG_appendLog(pHandle,
                     "DSPProcessor_Detach [%d] failed.\n", i);
                 }
             }
        }
        break;
    }

    
    OsSpecificMemFree(pAttrIn);

    if (cRetVal != 0) {
        /* Report in LOG module */
#ifdef __KERNEL__
		printk("PASS\n");
#endif
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }
    return 0;
}

