/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPStream_Reclaim.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPStream\DSPStream_Reclaim.c
 *
 *   Desc:
 *        DSPStream_Reclaim API test
 *
 *  DSPStream API function tested:
 *        DSPStream_Reclaim(DSP_HSTREAM hStream, BYTE *pBuffer,
 *                          ULONG *pDataSize, ULONG *pBufSize,
 *			                DWORD *pdwArg);
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 *
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ===========================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ===========================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>

#define DEFNAMESIZE    80
#define DEFAULTDSPUNIT 0
#define DSPEXEC        "inout_nortxidl.dof"

/*
 *  ======== main ========
 *
 */
INT DSPStream_Reclaim_API (INT argc, CHAR * argv[], INT iRunTime)
{
    INT             opt;
    INT             cRetVal = 1;
    INT             i, j, k, l, m;
    INT             iArgc = 1;
    BOOL            bUseLog=FALSE;
    FILE            *pHandle=NULL;
    CHAR            szLogName[DEFNAMESIZE];
    CHAR            *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR            *pszUuid;
    BYTE            **tmpInData;
    DSP_HPROCESSOR  hProcessor;
    DSP_HNODE       hNode = NULL;
    DSP_STATUS      status;
    DSP_STATUS      exitStatus = DSP_SOK;
    struct DSP_PROCESSORATTRIN    *pAttrIn;
    struct DSP_UUID        gidNodeID;
    struct DSP_CBDATA      *cbData;
    struct DSP_NODEATTRIN  nodeAttrin;
    struct DSP_STRMATTR    attrs;
    BYTE            argsBuf[32 + sizeof(ULONG)];
    BYTE            ptrBuf[60];
    ULONG           pBytes;
    DWORD           pdwArg;
    struct DSP_STREAMATTRIN strmAttrIn;
    DSP_HSTREAM     tmpStrmHandle;


    /* INPUT PARAMETERS */
    DSP_HSTREAM arg1[] =        /* arg 1: Handle of a stream */
    {
        NULL,                   /* valid handle to be allocated */
        NULL                    /* invalid NULL handle */
    };

    BYTE **arg2[] =             /* arg 2: location for pointer to buffer */
    {
        NULL,                   /* valid buffer */
        NULL                    /* invalid location */
    };

    ULONG *arg3[] =             /* arg 3: location for number of data bytes */
    {
        NULL,                   /* valid number */
        NULL                    /* invalid number */
    };

    ULONG *arg4[] =             /* arg 4: location for actual data size */
    {
        NULL,                   /* valid number */
        NULL                    /* invalid number */
    };

    DWORD *arg5[] =             /* arg 5: location for user argument */
    {
        NULL,                   /* valid location */
        NULL                    /* invalid location */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HSTREAM))
    #define NUMARG2 (sizeof(arg2)/sizeof(BYTE *))
    #define NUMARG3 (sizeof(arg3)/sizeof(ULONG *))
    #define NUMARG4 (sizeof(arg4)/sizeof(ULONG *))
    #define NUMARG5 (sizeof(arg5)/sizeof(DWORD *))

    /* SOLUTION SET */
    struct  {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] [NUMARG4] [NUMARG5] =
    {
        {
            {
                {
                    {
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][0][0][1] */

                    },
                    {
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][0][1] */
                    },
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][1][1] */
                    },
                },
            },
            {
                {
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][0][1] */
                    },
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][0][1] */
                    },
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][1][1] */
                    },
                },
            },
        },
        {
            {
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1][1] */
                    },
                },
            },
            {
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1][1] */
                    },
                },
            },
        },
    };

    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 5000;

	strmAttrIn.cbStruct = sizeof(struct DSP_STREAMATTRIN);
	strmAttrIn.uTimeout = 5000;
	strmAttrIn.uSegment = 0;
	strmAttrIn.uAlignment = 0;
	strmAttrIn.uNumBufs = 1;
	strmAttrIn.lMode = STRMMODE_PROCCOPY;

    nodeAttrin.uTimeout = 2000;
    nodeAttrin.iPriority = 1;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif


    cbData = (struct DSP_CBDATA *)argsBuf;
    cbData->cbData = 32;

    pBytes = 60;
    pdwArg = 0x0;

    tmpInData = OsSpecificMemAlloc(sizeof(BYTE *));

	arg2[0] = OsSpecificMemAlloc(sizeof(BYTE *));
	*(arg2[0]) = NULL;
    arg2[1] = NULL;

	arg3[0] = OsSpecificMemAlloc(sizeof(ULONG));
    *(arg3[0]) = 60;
    arg3[1] = NULL;

	arg4[0]  = OsSpecificMemAlloc(sizeof(ULONG));
    *(arg4[0]) = 60;
    arg4[1] = NULL;

	arg5[0]  = OsSpecificMemAlloc(sizeof(DWORD));
    *(arg5[0]) = 0x0;
    arg5[1] = NULL;


    attrs.uSegid = 0;
    attrs.uBufsize = 512;
    attrs.lMode = STRMMODE_PROCCOPY;
    attrs.uNumBufs = 1;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;

    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:ZR:")) != EOF) {
        switch (opt) {
            case 'v':
                /* verbose */
                TST_setVerbose(TRUE);
                TST_verbose("DSPStream_Reclaim: Verbose Mode On\n");
                break;
            case 'l':
                /* use log and store with name location */
                strncpy(szLogName, optarg, sizeof(szLogName));
                bUseLog = TRUE;
                TST_verbose("Opening %s\n", szLogName);
                pHandle = TSTLOG_openLog(szLogName);
                TSTLOG_appendLog(pHandle, "DSPStream_Reclaim:\n");
                /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
                TST_verbose("Log opened as: %s\n", szLogName);
                break;
            case 'x':
                /* use given executable */
                aArgv[0] = (char *) optarg;
                break;
            case 'Z':
                /* Use "zero-copy" streaming mode */
                attrs.lMode = STRMMODE_ZEROCOPY;
                attrs.uSegid = DSP_SHMSEG0;   /* use SM seg 0 on DSP */
                TST_verbose("Mode of transfer set to ZERO COPY\n");
                TSTLOG_appendLog(pHandle, "Mode of transfer set to ZERO COPY");
                break;
            case 'R':
                /* Use "RDMA" (Remote DMA) streaming mode */
                attrs.lMode = STRMMODE_RDMA;
                TST_verbose("Mode of transfer set to RDMA\n");
                TSTLOG_appendLog(pHandle, "Mode of transfer set to RDMA");
                break;
            default:
                TST_reportError(("Invalid option: %c\n"), opt);
                cRetVal = 0;

                if (bUseLog) {
                    TSTLOG_appendLog(pHandle,
                            "Invalid option specified.\n");
                }
                break;
        }
    }

    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    TST_verbose("DSPStream_Reclaim: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;
    
        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
        
            status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
        }
    
        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                        "Error in DSPProcessor_Attach()\n");
            }
            break;
        }

        arg1[1] = arg1[0];
        /* Load default program */
        status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv,
                NULL);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                        "Error in DSPProcessor_Load()\n");
            }
            break;
        }

        status = DSPProcessor_Start(hProcessor);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                        "Error in DSPProcessor_Start()\n");
            }
            break;
        }

        pszUuid = "F95CA076_EE7D_11D5_A8DD_00B0D055F6D1"; // inout
        UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

        /* Allocate Node */
        status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, &nodeAttrin,
                &hNode);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                        "Error in DSPNode_Allocate().\n");
            }
            break;
        }

        status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
                &attrs);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                        "Error in DSPNode_Connect()\n");
            }
            break;
        }

        status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                &attrs);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                        "Error in DSPNode_Connect()\n");
            }
            break;
        }

        /* Create Node on DSP */
        status = DSPNode_Create(hNode);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                        "Error in DSPNode_Create()\n");
            }
            break;
        }

        status = DSPNode_Run(hNode);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                        "Error in DSPNode_Run()\n");
            }
            break;
        }


        status = DSPStream_Open(hNode, DSP_TONODE, 0, NULL, &arg1[0]);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                        "Error in DSPStream_Open()\n");
            }

            break;
        }

        status = DSPStream_Open(hNode, DSP_FROMNODE, 0, NULL, &tmpStrmHandle);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                        "Error in DSPStraem_Open()\n");
            }

            break;
        }


        status = DSPStream_AllocateBuffers(arg1[0], 64, arg2[0], 1);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                        "Error in DSPStream_AllocateBuffers()\n");
            }

            break;
        }

        status = DSPStream_AllocateBuffers(tmpStrmHandle, 64, tmpInData, 1);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                        "Error in DSPStream_AllocateBuffers()\n");
            }

            break;
        }

        /* fill buffer */
        for (i = 0; i < (INT) pBytes; i++) {
            ptrBuf[i] = (BYTE) 'a';
        }

        for (i = 0; i < NUMARG1; i++) {
            for (j = 0; j < NUMARG2; j++) {
                for (k = 0; k < NUMARG3; k++) {
                    for (l = 0; l < NUMARG4; l++) {
                        for (m = 0; m < NUMARG5; m++) {

                            /* Not sure if this will work */
                            if (i == 0 && j == 0 && k == 0 &&
                                    ((l == 0) || (l == 1)) && m == 0) {
                                status = DSPStream_Issue(arg1[0], ptrBuf,
                                        pBytes, pBytes, pdwArg);
                            }

                            status = DSPStream_Reclaim(arg1[i], arg2[j],
                                    arg3[k], arg4[l], arg5[m]);

                            if (!TST_CheckVal(status,
                                        checkArray[i][j][k][l][m].dwValue,
                                        checkArray[i][j][k][l][m].fEqualFlag)) {
                                cRetVal = 0;
                                TST_verbose(
                                   "Reclaim [%d][%d][%d][%d][%d] returned %x\n",
                                   i, j, k, l, m, status);

                                if (bUseLog) {
                                    TSTLOG_appendLog(pHandle, "FAIL\n");
                                    TSTLOG_appendLog(pHandle,
                                       "DSPStream_Reclaim [%d][%d][%d][%d][%d] \
                                       failed.\n", i, j, k, l, m);
                                }
                            }

                            /* The following operations ensure that the ARM
                             * application is in synch with the execution of
                             * the 'inout' DSP node. */
                            if (status == DSP_SOK)
                            {
                                if (l == 0) {
                                    status = DSPStream_Issue(arg1[0],
                                            ptrBuf, pBytes, pBytes,
                                            pdwArg);
                                }

                                status = DSPStream_Issue(tmpStrmHandle,
                                        tmpInData[0], pBytes, pBytes, 0);

                                status = DSPStream_Reclaim(tmpStrmHandle,
                                        &tmpInData[0], arg3[k],
                                        arg4[l], arg5[m]);

                                if (l == 0) {
                                    status = DSPStream_Reclaim(arg1[i],
                                            arg2[j], arg3[k], arg4[l],
                                            arg5[m]);
                                }
                            }
                        }
                    }
                }
            }
        }

        break;
    }

    status = DSPStream_Close(arg1[0]);
    status = DSPStream_Close(tmpStrmHandle);
    status = DSPNode_Terminate(hNode, &exitStatus);
    status = DSPNode_Delete(hNode);

    status = DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);
    OsSpecificMemFree(arg2[0]);
    OsSpecificMemFree(arg3[0]);
    OsSpecificMemFree(arg4[0]);
	OsSpecificMemFree(arg5[0]);
    OsSpecificMemFree(tmpInData);

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

    return 0;
}
