/*  ============================================================================
 *   File    IVADSPNode_Allocate.c
 *
 *   Path    $(TESTSUITEROOT)\TestIVAAPI\IVADSPNode
 * 
 *   Desc:
 *        DSPNode API test
 *
 *  DSPNode API function tested:
 *        DSPNode_Allocate(DSP_HPROCESSOR hProcessor, struct DSP_UUID pNodeID,
 *                         DSP_CBDATA *pArgs, struct DSP_NODEATTRIN *pAttrIn,
 *                         DSP_HNODE *phNode)
 *  Usage:
 *        DSPNode_Allocate [-v] [-l "log path/filename"] [-x "valid executable"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *   AUG 05, 2004    REF=Based on DSP API tests             Ramesh Gupta G
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>
#include <msgtxfr.h>

#define DEFNAMESIZE    80

#define DSPEXEC         "iva.dof"

/*
 *  ======== main ========
 *
 */
INT IVADSPNode_Allocate_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT            opt;
    INT            cRetVal = 1;
    INT            i, j, k, l, m;
    INT            iArgc = 1;
    BOOL           bUseLog=FALSE;
    FILE           *pHandle;
    CHAR           szLogName[DEFNAMESIZE];
    CHAR           *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR           *pszUuid;
    DSP_STATUS     status;
    struct DSP_PROCESSORATTRIN    *pAttrIn;

    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId;


    /* INPUT PARAMETERS */
    DSP_HPROCESSOR arg1[] =     /* arg 1: attached processor handle */
    {
        1, /* valid handle to be allocated */
        NULL                    /* invalid NULL handle */
    };

    struct DSP_UUID *arg2[] =      /* arg 2: pointer to struct DSP_UUID */
    {
        NULL, /* valid pointer to be allocated */
        NULL                /* invalid NULL pointer */
    };

    struct DSP_CBDATA *arg3[] =    /* arg 3: pointer to CBDATA structure */
    {
        NULL, /* valid pointer to be allocated */
        NULL              /* invalid NULL pointer */
    };

    struct DSP_NODEATTRIN *arg4[] =  /* arg 4: pointer to NODEATTRIN structure */
    {
        NULL, /* valid pointer to be allocated */
        NULL              /* invalid NULL pointer */
    };

    /*
     * Use pointers to the handle so that we can test a NULL pointer
     * as well as a valid one that points to a valid handle
     */
    DSP_HNODE *arg5[] =   /* arg 5: pointer location for node handle */
    {
        NULL, /* valid location to be allocated */
        NULL                /* NULL pointer */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HPROCESSOR))
    #define NUMARG2 (sizeof(arg2)/sizeof(struct DSP_UUID *))
    #define NUMARG3 (sizeof(arg3)/sizeof(struct DSP_CBDATA *))
    #define NUMARG4 (sizeof(arg4)/sizeof(struct DSP_NODEATTRIN *))
    #define NUMARG5 (sizeof(arg5)/sizeof(DSP_HNODE))

    /* SOLUTION SET */
    struct  {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] [NUMARG4] [NUMARG5] =
                                                /* arg1 arg2 arg3 arg4 arg5 */
    {
        {
            {
                {
                    {
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][0][0][1] */
                    },
                    {
                        /*{DSP_ERANGE, TST_SHOULDEQUAL},*/ /* [0][0][0][1][0] */
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][0][1][1] */
                    },

                },
                {
                    {
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][1][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][0][1] */
                    },
                    {
                        /*{DSP_ERANGE, TST_SHOULDEQUAL},*/ /* [0][0][1][1][0] */
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][1][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][1][1] */
                    },
                },
            },
            {
                {
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][0][1] */
                    },
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][0][1] */
                    },
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][1][1] */
                    },
                },
            },
        },
        {
            {
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1][1] */
                    },
                },
            },
            {
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1][1] */
                    },
                },
            },
        },
    };


    /* Initialise pHandle */
    pHandle = (FILE *)NULL;
    
    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    arg2[0] = OsSpecificMemAlloc(sizeof(struct DSP_UUID));

	pszUuid = "6562DF2A_7068_42EF_8459_75584C1CDB8F"; // ivaMessage
	////pszUuid = "6562DF2A_7068_42EF_8459_75584C1CDB8F";
    UUID_UuidFromString((CHAR *)pszUuid, arg2[0]);

    arg3[0] = OsSpecificMemAlloc(sizeof(struct DSP_CBDATA));
    arg3[0]->cbData=1;

    arg4[0] = OsSpecificMemAlloc(sizeof(struct DSP_NODEATTRIN));
    arg4[0]-> cbStruct = sizeof(struct DSP_NODEATTRIN);
    arg4[0]->uTimeout = 1000;
    arg4[0]->iPriority = 1;

    arg5[0] = OsSpecificMemAlloc(sizeof(DSP_HNODE));

    /* iPriority out of range */
    arg4[1]  = OsSpecificMemAlloc(sizeof(struct DSP_NODEATTRIN));
    arg4[1]-> cbStruct = sizeof(struct DSP_NODEATTRIN);
    arg4[1]->uTimeout = 1000;
    arg4[1]->iPriority = 30;

    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DSPNode_Allocate: Verbose Mode On\n");
            break;
       case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
         bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "DSPNode_Allocate:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
         break;
    case 'x':
        /* use given executable */
        aArgv[0] = (char *) optarg;
        break;
        default:
            TST_reportError("Invalid option: %c\n", opt);
            cRetVal = 0;
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified");
            }
            break;
        }
    }
    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    TST_verbose("DSPNode_Allocate: ");
    TST_verbose("%i: ", iRunTime);

        while (cRetVal > 0) 
 	{


	  while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
       		 {
                	if (dspInfo.uProcessorType == IVA_ARM7)
                	{
                        TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                	}
                	index++;
                 }

    /* Attach to IVA */
		    if (DSP_SUCCEEDED(status)) 
			{
		        status = DSPProcessor_Attach(procId, NULL,&arg1[0]);
        		       if (DSP_SUCCEEDED(status)) 
				{
            		 	  TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach succeeded.\n");
        			}
        		        else 
				{
		                  TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach failed. Status = 0x%x\n",status);
	                          break;
        		     	}
    			}
        		else
			{
           		TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
	                break;
			}
            
            /* Load executable */
            status = DSPProcessor_Load(arg1[0], iArgc, (CONST CHAR **)aArgv, NULL);
            if (!DSP_SUCCEEDED(status)) {
                /* Report Error */
                cRetVal = 0;
                TST_verbose("FAIL\n");
                if (bUseLog) {
                    TSTLOG_appendLog(pHandle, "FAIL\n");
                    TSTLOG_appendLog(pHandle,
                    "Error in DSPProcessor_Load():IVA");
                }
                break;
            }


            /* Start Processor */
            status = DSPProcessor_Start(arg1[0]);
            if (!DSP_SUCCEEDED(status)) {
                /* Report Error */
                cRetVal = 0;
                TST_verbose("FAIL\n");
                if (bUseLog) {
                    TSTLOG_appendLog(pHandle, "FAIL\n");
                    TSTLOG_appendLog(pHandle,
                    "Error in DSPProcessor_Start():IVA");
                }
                break;
            }

            for (i = 0; i < NUMARG1; i++) {
                for (j = 0; j < NUMARG2; j++) {
                    for (k = 0; k < NUMARG3; k++) {
                        for (l = 0; l < NUMARG4; l++) {
                            for (m = 0; m < NUMARG5; m++) {

                                ///status = DSPNode_Allocate(arg1[i], arg2[j],arg3[k], arg4[l], arg5[m]);
                                status = DSPNode_Allocate(arg1[i], arg2[j],arg3[k], arg4[l], arg5[m]);
			/*	TSTLOG_appendLog(pHandle,"Allocate:in Main loop for iteration [%d][%d][%d][%d][%d] returned %x\n",i, j, k, l, m, status);*/

		
                                if (!TST_CheckVal(status,
                                    checkArray[i][j][k][l][m].dwValue,
                                    checkArray[i][j][k][l][m].fEqualFlag)) {
                                    cRetVal = 0;
                                    TSTLOG_appendLog(pHandle,"Allocate [%d][%d][%d][%d][%d] returned %x\n",i, j, k, l, m, status);

                                    if (bUseLog) {
                                        TSTLOG_appendLog(pHandle, "FAIL\n");
                                        TSTLOG_appendLog(pHandle,"DSPNode_Allocate [%d][%d][%d][%d][%d] \t failed. and status is 0x%x\n", i, j, k, l, m,status);
                                    }
                                }
                            }
                        }
                    }
                }
            }

            break;
        }

    status = DSPProcessor_Detach(arg1[0]);

    OsSpecificMemFree(pAttrIn);
    OsSpecificMemFree(arg3[0]);
    OsSpecificMemFree(arg4[0]);
    OsSpecificMemFree(arg4[1]);
    OsSpecificMemFree(arg5[0]);
	
    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

