/*  ============================================================================
 *   File    IVADSPNode_GetAttr.c
 *
 *   Path    $(TESTSUITEROOT)\TestIVAAPI\IVADSPNode
 * 
 *   Desc:
 *        DSPNode API test
 *
 *  DSPNode API function tested:
 *        DSPNode_GetAttr(DSP_HNODE hNode, DSP_NODEATTR *pAttr;
 *                        UINT uAttrSize)
 *  Usage:
 *        DSPNode_GetAttr [-v] [-l "log path/filename"]
 *                       [-x "valid executable"]
 *                       [-m "message to be sent"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: name of valid DSP executable
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *   AUG 05, 2004    REF=Based on DSP API tests             Ramesh Gupta G
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>
#include <msgtxfr.h>

#define DEFNAMESIZE    80

#define DSPEXEC         "iva.dof55l"

/*
 *  ======== main ========
 *
 */
INT IVADSPNode_GetAttr_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT            opt;
    INT            cRetVal = 1;
    INT            iArgc = 1;
    INT            i, j, k;
    BOOL           bUseLog=FALSE;
    FILE           *pHandle=NULL;
    CHAR           szLogName[DEFNAMESIZE];
    CHAR           *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR           *pszUuid;
    DSP_HPROCESSOR    hProcessor;
    struct DSP_UUID        gidNodeID;
    DSP_STATUS      status;
    struct DSP_PROCESSORATTRIN *pAttrIn;
    struct DSP_NODEATTR    *pAttr;
    struct DSP_NODEATTRIN  *nodeAttrin;
    struct DSP_CBDATA      *cbData;
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId;


    /* INPUT PARAMETERS */
    DSP_HNODE arg1[] =     /* arg1: handle of node */
    {
        NULL, /* Valid handle to be allocated */
        NULL, /* Invalid NULL handle */
    };

    struct DSP_NODEATTR *arg2[] = /* arg2: pointer to DSP_NODEATTR structure */
    {
        NULL, /* valid pointer to be allocated */
        NULL                    /* invalid NULL pointer */
    };

    UINT arg3[] =          /* arg3: size of DSP_NODEATTR structure */
    {
        sizeof(struct DSP_NODEATTR), /* valid pointer location */
        0                       /* invalid pointer */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HNODE))
    #define NUMARG2 (sizeof(arg2)/sizeof(struct DSP_NODEATTR *))
    #define NUMARG3 (sizeof(arg3)/sizeof(UINT))

    /* SOLUTION SET */
    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] =
    {
        {
            {
                {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0] */
                {DSP_ESIZE, TST_SHOULDEQUAL}, /* [0][0][1] */
            },
            {
                {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0] */
                {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1] */
            },

        },
        {
            {
                {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0] */
                {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1] */
            },
            {
                {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0] */
                {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1] */
            },
        },
    };

    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;


    pAttr = OsSpecificMemAlloc(sizeof(struct DSP_NODEATTR));
    
    arg2[0] = OsSpecificMemAlloc(sizeof(struct DSP_NODEATTR));

    nodeAttrin = OsSpecificMemAlloc(sizeof(struct DSP_NODEATTRIN));
    nodeAttrin->cbStruct = sizeof(struct DSP_NODEATTRIN);
    nodeAttrin->uTimeout = 1000;
    nodeAttrin->iPriority = 1;


    cbData = OsSpecificMemAlloc(sizeof(struct DSP_CBDATA));
    cbData->cbData = 0;

    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:m:")) != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DSPNode_GetAttr: Verbose Mode On\n");
            break;
       case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
         bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "DSPNode_GetAttr:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
         break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);
            cRetVal = 0;

            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid Option Specified.\n");
            }
            break;
        }
    }
    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    TST_verbose("DSPNode_GetAttr: ");
    TST_verbose("%i: ", iRunTime);

        while (cRetVal > 0) {
//////////////////////////////////////////////////////////////////////////
	while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
          {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
          }

    /* Attach to IVA */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, NULL,&hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach succeeded.\n");
        }
        else {
                TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach failed. Status = 0x%x\n",status);
		cRetVal = 0;
		break;
        }
    }
        else
           TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");



//////////////////////////////////////////////////////////////////////////
            arg1[1] = arg1[0];
            /* Load Processor with Executable */
            status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);

            if (!DSP_SUCCEEDED(status)) {
                /* Report Error */
                cRetVal = 0;

                TST_verbose("FAIL\n");
                if (bUseLog) {
                    TSTLOG_appendLog(pHandle, "FAIL\n");
                    TSTLOG_appendLog(pHandle,
                    "Error in DSPProcessor_Load():IVA\n");
                }
                break;
            }

            /* Start Processor */
            status = DSPProcessor_Start(hProcessor);

            if (!DSP_SUCCEEDED(status)) {
                /* Report Error */
                cRetVal = 0;

                TST_verbose("FAIL\n");
                if (bUseLog) {
                    TSTLOG_appendLog(pHandle, "FAIL\n");
                    TSTLOG_appendLog(pHandle,
                    "Error in DSPProcessor_Load():IVA\n");
                }

                break;
            }
			pszUuid = "6562DF2A_7068_42EF_8459_75584C1CDB8F";
            UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);
            gidNodeID=Iva_Msguuid;

            DSPNode_Allocate(hProcessor, &gidNodeID, cbData, nodeAttrin, &arg1[0]);

            if (!DSP_SUCCEEDED(status)) {
                /* Report Error */
                cRetVal = 0;

                TST_verbose("FAIL\n");
                if (bUseLog) {
                    TSTLOG_appendLog(pHandle, "FAIL\n");
                    TSTLOG_appendLog(pHandle,
                    "Error in DSPNode_Allocate():IVA\n");
                }

                break;
            }

            for (i = 0; i < NUMARG1; i++) {
                for (j = 0; j < NUMARG2; j++) {
                    for (k = 0; k < NUMARG3; k++) {

                        status = DSPNode_GetAttr(arg1[i], arg2[j], arg3[k]);

                        if (!TST_CheckVal(status,
                            checkArray[i][j][k].dwValue,
                            checkArray[i][j][k].fEqualFlag)) {
                            cRetVal = 0;
                            TST_verbose("GetAttr [%d] [%d] [%d] returned %x\n",
                                        i, j, k, status);

                            if (bUseLog) {
                                TSTLOG_appendLog(pHandle, "FAIL\n");
                                TSTLOG_appendLog(pHandle,
                                "DSPNode_GetAttr [%d][%d][%d] failed.\n", i, j, k);
                            }
                        }
                    }
                }
            }

            break;
        }


    status = DSPNode_Delete(arg1[0]);
    status = DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);
    OsSpecificMemFree(nodeAttrin);
    OsSpecificMemFree(cbData);
    OsSpecificMemFree(pAttr);
	OsSpecificMemFree(arg2[0]);

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

