/*  ============================================================================
 *   File    IVADSPProcessor_GetState.c
 *
 *   Path    $(TESTSUITEROOT)\TestIVAAPI\IVADSPProcessor\
 * 
 *   Desc:
 *        DSPProcessor API test
 *
 *  DSPProcessor API function tested:
 *        DSPProcessor_GetState(DSP_HPROCESSOR hProcessor,
 *                              DSP_PROCESSORSTATE *pProcStatus
 *                              UINT uStateInfoSize);
 *  Usage:
 *        DSPProcessor_GetState [-v] [-l "log path/filename"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *   AUG 04, 2005    REF=Based on DSP API tests             Ramesh Gupta G
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>
#include <msgtxfr.h>

#define DEFNAMESIZE    80

#define DSPEXEC        "iva.dof"

/*
 *  ======== main ========
 *
 */
INT IVADSPProcessor_GetState_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT            opt;
    INT            cRetVal = 1;
    INT            iArgc = 1;
    INT            i, j, k;
    BOOL        bUseLog=FALSE;
    FILE        *pHandle=NULL;
    CHAR        szLogName[DEFNAMESIZE];
    CHAR        *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    DSP_HPROCESSOR     hProcessor;
    struct DSP_PROCESSORSTATE    *pProcStatus;
    DSP_STATUS        status;
    struct DSP_PROCESSORATTRIN *pAttrIn;

    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId;

    /* INPUT PARAMETERS */
    DSP_HPROCESSOR arg1[] =   /* attached processor handle */
    {
        1, /* Valid handle to be attached */
        NULL        /* Invalid handle */
    };

    struct DSP_PROCESSORSTATE *arg2[] =  /* Pointer to state structure */
    {
        NULL, /* Valid pointer to be set */
        NULL            /* Invalid NULL pointer */
    };

    UINT arg3[] =        /* Size of DSP_PROCESSORSTATE structure */
    {
        sizeof(struct DSP_PROCESSORSTATE), /* Valid size */
        0                /* Invalid size */
    };


    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HPROCESSOR))
    #define NUMARG2 (sizeof(arg2)/sizeof(struct DSP_PROCESSORSTATE *))
    #define NUMARG3 (sizeof(arg3)/sizeof(UINT))

    /* solution set */
    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1][NUMARG2][NUMARG3] =
    {
        {
            {
                /*{DSP_EHANDLE, TST_SHOULDEQUAL},*/ /* [0] [0] [0] */
                {DSP_SOK, TST_SHOULDEQUAL}, /* [0] [0] [0] */
                {DSP_ESIZE, TST_SHOULDEQUAL}, /* [0] [0] [1] */
            },
            {
                {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0] [1] [0] */
                {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0] [1] [1] */
            },
        },
        {
            {
                /*{DSP_SOK, TST_SHOULDEQUAL},*/ /* [1] [0] [0] */
                {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1] [0] [0] */
                {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1] [0] [1] */
            },
            {
                {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1] [1] [0] */
                {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1] [1] [1] */
            },
        },
    };


    /* Allocate pointer to dummy set of attributes */
    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pProcStatus=OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORSTATE));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Allocate pointer to storage location for processor state information */
    pProcStatus = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORSTATE));

    arg2[0] = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORSTATE));

    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
         switch (opt) {
         case 'v':
             /* verbose */
             TST_setVerbose(TRUE);
             TST_verbose("DSPProcessor_GetState: Verbose Mode On\n");
             break;
         case 'l':
             /* use log and store with name location */
             strncpy(szLogName, optarg, sizeof(szLogName));

             bUseLog = TRUE;
             TST_verbose("Opening %s\n", szLogName);
             pHandle = TSTLOG_openLog(szLogName);

             TSTLOG_appendLog(pHandle, "DSPProcessor_GetState:\n");
             /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
             TST_verbose("Log opened as: %s\n", szLogName);
             break;
         case 'x':
             /* Use DSP executable */
		  	 aArgv[0] = (char *) optarg;
             break;
         default:
             TST_verbose(("Invalid option: %c\n"), opt);
             cRetVal = 0;
             if (bUseLog) {
                 TSTLOG_appendLog(pHandle,
                 "Invalid option specified.\n");
             }
             break;
         }
    }

    TST_verbose("DSPProcessor_GetState: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {

        /* Attached to IVA Processor */
///////////////////////////////////////////////////////////////
	 while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
          {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
          }

    /* Attach to IVA */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, pAttrIn,&arg1[0]);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach failed. Status = 0x%x\n",status);
	    cRetVal=0;
	    break;
        }
    }
        else
           TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");

///////////////////////////////////////////////////////////////

        for (i = 0; i < NUMARG1; i++) {
            for (j = 0; j < NUMARG2; j++) {
                for (k = 0; k < NUMARG3; k++) {

                    status = DSPProcessor_GetState(arg1[i], arg2[j], arg3[k]);

/*		    TSTLOG_appendLog(pHandle,"DSPProcessor_GetState in main loop [%d][%d][%d] and status is 0x%x.\n", i, j, k,status); for testing*/

                    if (!TST_CheckVal(status, checkArray[i][j][k].dwValue,
                        checkArray[i][j][k].fEqualFlag)) {
                        cRetVal = 0;
                        TST_verbose("GetState [%d][%d][%d] returned %x\n",
                                     i, j, status);

                        if (bUseLog) {
                            TSTLOG_appendLog(pHandle, "FAIL\n");
                            TSTLOG_appendLog(pHandle,
                            "DSPProcessor_GetState [%d][%d][%d] \
                            failed.\n", i, j, k);
                        }
                    }
                }
            }
        }

        /* Detach Processor */
        status = DSPProcessor_Detach(arg1[0]);

        /* Additional testing */
///////////////////////////////////////////////////////////////

        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
          {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
          }

    /* Attach to IVA */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, pAttrIn,&hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach failed. Status = 0x%x\n",status);
	    cRetVal=0;
        }
    }
        else
           TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");

///////////////////////////////////////////////////////////////
        if(DSP_SUCCEEDED(status)) {
            /* Place processor in LOADED state */
            DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);
            status = DSPProcessor_GetState(hProcessor, pProcStatus, arg3[0]);
            if (!DSP_SUCCEEDED(status)) {
                cRetVal = 0;
                if (bUseLog) {
                    TSTLOG_appendLog(pHandle, "FAIL\n");
                    TSTLOG_appendLog(pHandle,
                    "Error in DSPProcessor_GetState:IVA\n");
                }
            }
            else {
                if (pProcStatus->iState != PROC_LOADED) {
                    cRetVal = 0;
                    if (bUseLog) {
                        TSTLOG_appendLog(pHandle, "FAIL\n");
                        TSTLOG_appendLog(pHandle,
                        "Action not performed in DSPProcessor_GetState:IVA");
                    }
                }
            }

            /* Place processor in RUNNING state */
            DSPProcessor_Start(hProcessor);
            status = DSPProcessor_GetState(hProcessor, pProcStatus, arg3[0]);
            if (!DSP_SUCCEEDED(status)) {
                cRetVal = 0;
                if (bUseLog) {
                    TSTLOG_appendLog(pHandle, "FAIL\n");
                    TSTLOG_appendLog(pHandle,
                    "Error in DSPProcessor_GetState:IVA");
                }
            }
            else {
                if (pProcStatus->iState != PROC_RUNNING) {
                    cRetVal = 0;
                    if (bUseLog) {
                        TSTLOG_appendLog(pHandle, "FAIL\n");
                        TSTLOG_appendLog(pHandle,
                        "Action not performed in DSPProcessor_GetState():IVA\n");
                    }
                }
            }
            status = DSPProcessor_Detach(hProcessor);
            if (!DSP_SUCCEEDED(status)) {
                /* Report Error */
                cRetVal = 0;
                if (bUseLog) {
                    TSTLOG_appendLog(pHandle, "FAIL\n");
                    TSTLOG_appendLog(pHandle,
                    "Error in DSPPRocessor_Detach():IVA\n");
                }
            }

        }
        break;
    }

	OsSpecificMemFree(pAttrIn);
	OsSpecificMemFree(pProcStatus);
	OsSpecificMemFree(arg2[0]);

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }
    return 0;
}

