/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*
 *   File    kern_testsuite.c
 *
 *   Path    $(TESTSUITEROOT)\TestKern\
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *   ---------------------------------------------------------------------------
 *
 *! Revision History
 *! ================
 *! xx-Feb-2003 swa: Introduced to test the Kernel API Feature
 *!
 */
 
/*  ----------------------------------- OS */

#include <linux/config.h>
#if defined(CONFIG_SMP)
#define __SMP__
#endif

#if defined(CONFIG_MODVERSIONS)
#define	MODVERSIONS
#include <linux/modversions.h>
#endif

#ifdef	MODULE
#include <linux/module.h>	/* Kernel module definitions */
#endif
#include "test_suite_kern.h"

/*  ----------------------------------- Macros */

#include <linux/init.h>
#include <linux/kernel.h>	/* kernel mode of execution */
#include <linux/moduleparam.h> /*added for 2.6 compilation*/

/*  ----------------------------------- DSP/BIOS Bridge */
#include <std.h>
#include <dbdefs.h>
#include <errbase.h>

/*  ----------------------------------- Trace & Debug */
#include <gt.h>
#include <dbc.h>

/*  ----------------------------------- OS Adaptation Layer */
#include <osal.h>
#include <sync.h>
#include <reg.h>

/*  ----------------------------------- Platform Manager */
#include <wcdioctl.h>
#include <_dcd.h>           /* Class driver Lib */
#include <wcd.h>
#include <dbreg.h>

/*  ----------------------------------- This */
#include "kern_testsuite.h"		/* Stuff for this driver only */

/*  ----------------------------------- Globals */
#define	DRIVER_NAME	    "DummyTestDriver"	
#define	DRIVER_MAJOR		0		/* Linux assigns our Major dev number */
/* Declaration of functions */
INT apitst_main (INT , CHAR **);
INT stress_main (INT , CHAR **);
INT behvr_main (INT , CHAR **);
INT anlysis_main (INT , CHAR **);

INT	driver_major = DRIVER_MAJOR;
MODULE_PARM(driver_major, "i"); /* Driver's major number */

MODULE_PARM_DESC(driver_major, "Major device number, default = 0 (auto)");

/*MODULE_LICENSE("Texas Instruments Inc.") ;*/
MODULE_LICENSE("GPL") ;

char * driver_name = DRIVER_NAME ;

static	struct	file_operations driver_fops = {
    open:       api_test_driver_open,
    release:    api_test_driver_release,
    ioctl:      api_test_driver_ioctl,
    read:       api_test_driver_read,
    write:      api_test_driver_write,
    /*ioctl:      api_test_driver_ioctl, changed -Ramesh*/
};

/* Initialization routine.
 * Executed when the driver is loaded (as a kernel module), or
 * When the system is booted (when included as part of the kernel image).
 */
/*static int __init api_test_driver_init()*/
 int api_test_driver_init()
{
    int     status;

    status = register_chrdev (driver_major, driver_name, &driver_fops) ;

    if (status>= 0)
    {
        driver_major = status ;
        status = 0 ;
    }

    printk(" <- kern_test_driver_init.\r\n") ;

    printk(" Driver name = %s\n", driver_name);

    return status;
}

/*static void __exit api_test_driver_exit()*/
static void  api_test_driver_exit()
{
	int	    status ;

    printk("-> kern_test_driver_exit \r\n");

    status = unregister_chrdev(driver_major, driver_name) ;

    return;
}

int api_test_driver_open(struct inode *ip, struct file *filp)		/* Open */
{
    int status = 0 ;

    return status;
}

int api_test_driver_release(struct inode *ip, struct file *filp)	/* Release */
{
    int status = 0 ;

    return status;
}

ssize_t api_test_driver_read(struct file *filp, char *buf, size_t count, loff_t *f_pos)	/* Read */
{
    return count ;
}

ssize_t api_test_driver_write(struct file *filp,const char *buf,size_t count,loff_t *f_pos) /* Write */
{
    return count ;
}

int api_test_driver_ioctl (struct inode *ip, struct file *filp, int tmp_argc, char **tmp_argv)
{
    int status = DSP_SOK;
    DWORD retval = DSP_SOK;
    int i;
    
        
    printk(" -> kern_test_driver_ioctl \r\n") ;
    
    
    /* Discard Higher order bits(Device Type & Access) and right shift by
     * 2(Function Code was shifted while encoding.Deduct one for the
     * CMD_BASE.
     */
    printk("LOG: TESTARGS: ");
    for(i =0; i < tmp_argc; i++)
        printk("%s  ",tmp_argv[i]);

    if (!strcmp(tmp_argv[0], "API"))
    {
	    apitst_main(tmp_argc, tmp_argv);
    }
    else if (!strcmp(tmp_argv[0], "STRESS"))
    {
    	stress_main(tmp_argc, tmp_argv);
    }
    else if (!strcmp(tmp_argv[0], "BEHAVIOUR"))
    {
    	behvr_main(tmp_argc, tmp_argv);
    }
    else if (!strcmp(tmp_argv[0], "ANALYSIS"))
    {
    	anlysis_main(tmp_argc, tmp_argv);
    }
	else
	{
	   	printk("************ ERROR FAIL ERROR !! ******************\n");
        status = -1 ;
	}

    if (DSP_SUCCEEDED(status)) 
    {
        status = retval ;
    }
    else
    {
        status = -1 ;
    }

    printk(" <- kern_test_driver_ioctl \r\n") ;
    
    return 0;
}




module_init(api_test_driver_init);
module_exit(api_test_driver_exit);

