/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    tst.c
 *
 *   Path    $(TESTSUITEROOT)\Testutil\tst\tst.c
 * 
 *   Desc:
 *      Test utility library for BRD and CHNL API tests.
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


/* include files */
#ifndef __KERNEL__
#include <stdio.h>
#include <std.h>
#include <stdarg.h>
#endif
#include <dbtype.h>

#include <tst.h>

#define TST_BUFSIZE 512     /* max size of output buffer */

/* global flags */
BOOL g_fVerbose = FALSE;    /* default off */

/*
 *  ======== TST_reportError ========
 *  Purpose:
 *      TST module's printf.
 */
VOID TST_reportError(char * fmt, ...)
{
    va_list va;
    CHAR    tempBuf[TST_BUFSIZE];
    
    va_start(va, fmt);
    
    vsprintf(tempBuf, fmt, va);
#ifndef __KERNEL__
    fprintf(stdout, tempBuf);
#else
    printk("%s", tempBuf);
#endif
    
    va_end(va);
}

/*
 *  ======== TST_CheckVal ========
 *  Purpose:
 *      Validates the value based on entered parameters.
 *  Parameters:
 *      dwValidateValue     - The Value to check.
 *      dwBaseValue         - The Base value.
 *      fMustBe             - Should (Base == Value) or (Base != Value).
 *  Returns:
 *      True if the values pass, false if they do not.
 */
BOOL TST_CheckVal(DWORD dwValidateValue, DWORD dwBaseValue, BOOL fMustBe)
{
    BOOL    fRetVal;
    
    if (fMustBe) {
        fRetVal = (dwValidateValue == dwBaseValue) ? TRUE : FALSE;
    }
    else {
        fRetVal = (dwValidateValue != dwBaseValue) ? TRUE : FALSE;
    }

    return (fRetVal);
}


/*
 *  ======== TST_setVerbose ========
 *  Purpose:
 *      Turn print statements on or off.
 */
VOID TST_setVerbose(BOOL flag)
{
    g_fVerbose = flag;
}


/*
 *  ======== TST_verbose ========
 *  Purpose:
 *      Prints message to stdout if g_fVerbose flag set.
 */
VOID TST_verbose(char * fmt, ...)
{
    va_list va;
    CHAR    tempBuf[TST_BUFSIZE];
    
    if (g_fVerbose) {
        va_start(va, fmt);
        
        vsprintf(tempBuf, fmt, va);
#ifndef __KERNEL__
		fprintf(stdout, tempBuf);
#else
        printk("%s", tempBuf);
#endif
        
        va_end(va);
    }
}

